@@ -1,63 +1,109 @@
 #!/bin/sh
 
-# FZF navigation/previewing plugin for CliFM
+# FZF navigation/previewing/selection plugin for CliFM
 # Written by L. Abramovich
 # License: GPL3
 
-# Description: Navigate the filesystem (including file previews) with FZF.
-# Previews are shown by just hovering on the file. Each generated preview
-# is cached in $HOME/.cache/clifm/previews to speed up the previewing
-# process. A separate script is used for file previewing:
+# Description: Navigate the filesystem (including files preview and
+# selection) with FZF.
+
+# Previews are shown by just hovering on the file. Each generated preview,
+# provided it needs to be converted before (e.g. .gif, .pdf and .docx files),
+# is cached as image file(s) (using hashes as names) in
+# $HOME/.cache/clifm/previews to avoid subsequent convertions and speed up
+# thus the whole process.
+
+# A separate script is used for file previewing:
 # $HOME/.config/clifm/plugins/BFG.sh
 
-# Usage:
-# Left: go to parent directory
-# Right, Enter: cd into hovered directory or open hovered file and exit.
-# Home/end: go to first/last file
-# TAB: Select files
-# Ctrl-s: Confirm selection: send selected files to CliFM Selbox
-# Shift+up/down: Move one line up/down in the preview window
-# Alt+up/down: Move to the beginning/end in the preview window
-# At exit (pressing C-q) CliFM will change to the last directory visited
-# with FZF or open the last accepted file (Enter).
-# Press Esc to cancel and exit.
+# A configuration file is available:
+# $HOME/.config/clifm/plugins/BFG.cfg
+# or
+# $XDG_DATA_DIRS/clifm/plugins/BFG.cfg (usually /usr/local/share/clifm/plugins/BFG.cfg)
 
-if [ -n "$1" ] && { [ "$1" = "--help" ] || [ "$1" = "help" ]; }; then
-	name="$(basename "$0")"
-	printf "Navigate/preview files via FZF\n"
-	printf "Usage: %s\n" "$name"
-	exit 0
-fi
+# How the files previewer works?
+# 1. The config file is read looking for predefined options and previewing
+#     applications.
+# 2. Options not defined in the config file are set to the default value.
+#     If no opening application for a given file type was defined in the
+#     config file, the system is scanned for available applications.
+#    NOTE: Determining previewing applications is done only once, when
+#     when the fzfnav.sh script starts, instead of each time a file is
+#     hovered, which saves a significant amount of time.
+# 3. Export this info as environment variables.
+# 4. Launch FZF using the BFG.sh script to generate files preview using
+#     the information previosuly gathered. This script only takes care of
+#     determining file types and executing the corresponding previewing
+#     application.
+
+HELP="USAGE
+  Type in the prompt to filter the current list of files. Regular expressions are \
+allowed.
+
+  Use the --edit command line option to edit the configuration file.
+
+KEYBINDINGS
+
+  * Left: Change to parent directory
+  * Right/Enter: Change to the highlighted directory or open the highlighted file
+  * Home/end: Change to first/last file in the files list
+  * TAB: Select currently highlighted file
+  * Ctrl-s: Select all
+  * Ctrl-d: Deselect all
+  * Ctrl-t: Toggle selection
+  * Alt-s: Confirm selection (do it before changing directory or the current selection will be lost)
+  * Shift-up/down: Move one line up/down in the preview window
+  * Alt-up/down: Move to the beginning/end in the preview window
+  * Ctrl-q/Ctrl-c: Change to the last visited directory and exit
+  * Esc: Cancel and exit"
+
+HOME_PLUGINS_DIR="${XDG_CONFIG_HOME:-$HOME/.config}/clifm/plugins"
 
 uz_cleanup() {
-    rm "$FIFO_UEBERZUG" 2>/dev/null
+	# shellcheck disable=SC2317
+    rm -- "$FIFO_UEBERZUG" 2>/dev/null
+	# shellcheck disable=SC2317
     pkill -P $$ >/dev/null
 }
 
-get_bfg_cfg_file() {
-	FILE="${XDG_CONFIG_HOME:-$HOME/.config}/clifm/plugins/BFG.cfg"
+# Let's get the path to a plugin file, either in home or in data dirs.
+# Used for both BFG.cfg and BFG.sh
+get_file() {
+	[ -z "$1" ] && return
+	name="$1"
+
+	HOME_FILE="${HOME_PLUGINS_DIR}/$name"
+	FILE="$HOME_FILE"
 
 	if [ -z "$FILE" ] || ! [ -f "$FILE" ]; then
 		FILE=""
 		if [ -n "$XDG_DATA_DIRS" ]; then
 			dirs="$(echo "$XDG_DATA_DIRS" | sed 's/:/ /g')"
 			for dir in $dirs; do
-				if [ -f "$dir/clifm/plugins/BFG.cfg" ]; then
-					FILE="$dir/clifm/plugins/BFG.cfg"
+				if [ -f "$dir/clifm/plugins/$name" ]; then
+					FILE="$dir/clifm/plugins/$name"
 					break
 				fi
 			done
 		fi
 
 		if [ -z "$FILE" ]; then
-			if [ -f /usr/share/clifm/plugins/BFG.cfg ]; then
-				FILE="/usr/share/clifm/plugins/BFG.cfg"
-			elif [ -f /usr/local/share/clifm/plugins/BFG.cfg ]; then
-				FILE="/usr/local/share/clifm/plugins/BFG.cfg"
-			elif [ -f /boot/system/data/clifm/plugins/BFG.cfg ]; then
-				FILE="/boot/system/data/clifm/plugins/BFG.cfg"
-			elif [ -f /boot/system/non-packaged/data/clifm/plugins/BFG.cfg ]; then
-				FILE="/boot/system/non-packaged/data/clifm/plugins/BFG.cfg"
+			if [ "$CLIFM_PLUGINS_HELPER" ]; then
+				helper="$(dirname "$CLIFM_PLUGINS_HELPER")"
+			fi
+
+			if [ "$helper" ] && [ -f "$helper/$name" ]; then
+				FILE="$helper/$name"
+			elif [ -f "/usr/share/clifm/plugins/$name" ]; then
+				FILE="/usr/share/clifm/plugins/$name"
+			elif [ -f "/usr/local/share/clifm/plugins/$name" ]; then
+				FILE="/usr/local/share/clifm/plugins/$name"
+			elif [ -f "/boot/system/data/clifm/plugins/$name" ]; then
+				FILE="/boot/system/data/clifm/plugins/$name"
+			elif [ -f "/boot/system/non-packaged/data/clifm/plugins/$name" ]; then
+				FILE="/boot/system/non-packaged/data/clifm/plugins/$name"
+			elif [ -f "/data/data/com.termux/files/usr/share/clifm/plugins/$name" ]; then
+				FILE="/data/data/com.termux/files/usr/share/clifm/plugins/$name"
 			fi
 		fi
 	fi
@@ -71,23 +117,6 @@ start_ueberzug() {
 	| ueberzug layer --silent --parser json > /dev/null 2>&1 &
 }
 
-HELP="Usage:
-  Type in the prompt to filter the current list of files. Regular expressions are \
-allowed.
-
-  At exit (Ctrl-q) CliFM will change to the last directory visited or \
-open the last accepted file (Enter). Press Esc to cancel and exit.
-
-Keybindings:
-
-  * Left: Change to parent directory
-  * Right or Enter: Change to the highlighted directory or open the highlighted file and exit
-  * Home/end: Change to first/last file in the files list
-  * TAB: Select currently highlighted file
-  * Ctrl-s: Confirm selection (send files to CliFM's Selection Box)
-  * Shift-up/down: Move one line up/down in the preview window
-  * Alt-up/down: Move to the beginning/end in the preview window"
-
 fcd() {
 	if [ "$#" -ne 0 ]; then
 		cd "$@" || return
@@ -100,16 +129,19 @@ fcd() {
 
 	# Keep FZF running until the user presses Esc or C-q
 	# shellcheck disable=SC2154
+	# --header-first isn't available in fzf 0.27
+
 	while true; do
-		lsd=$(printf "\033[0;%sm..\n" "$dir_color"; $ls_cmd)
+		lsd=$(printf "\033[0;%sm..\033[0m\n" "$dir_color"; $ls_cmd)
 		file="$(printf "%s\n" "$lsd" | fzf \
 			--height="${fzfheight:-$fzf_height}" \
 			--color="$(get_fzf_colors)" \
-			--bind "ctrl-s:execute(touch $TMP_SEL)+accept" \
+			--bind "alt-s:execute(touch $TMP_SEL)+accept" \
+			--bind "ctrl-s:select-all,ctrl-d:deselect-all,ctrl-t:toggle-all" \
 			--bind "right:accept,left:top+accept" \
 			--bind "insert:clear-query" \
 			--bind "home:top,end:page-down" \
-			--bind "alt-h:preview(printf %s \"$HELP\")" \
+			--bind "alt-h:preview(printf '{\"action\": \"remove\", \"identifier\": \"clifm-preview\"}\n' > \"$FIFO_UEBERZUG\"; printf %s \"$HELP\")" \
 			--bind "alt-p:toggle-preview" \
 			--bind "shift-up:preview-up" \
 			--bind "shift-down:preview-down" \
@@ -118,7 +150,7 @@ fcd() {
 			--bind "esc:execute(rm $TMP)+abort" \
 			--bind "ctrl-q:abort" \
 			--ansi --prompt="${fzf_prompt}" --reverse --no-clear \
-			--no-info --keep-right --multi --header="Press 'Alt-h' for help
+			--inline-info --keep-right --multi --header="Press 'Alt-h' for help
 $PWD
 $FZF_HEADER" --marker="*" --preview-window=:wrap "$(fzf_borders)" \
 			--preview "printf \"\033[2J\"; $BFG_FILE {}")"
@@ -126,7 +158,7 @@ $FZF_HEADER" --marker="*" --preview-window=:wrap "$(fzf_borders)" \
 		# If FZF returned no file, exit
 		[ ${#file} -eq 0 ] && return 0
 
-		# If we have selected files, send them to CliFM Selbox
+		# If we have selected files, send them to the CliFM Selbox
 		if [ -f "$TMP_SEL" ]; then
 			echo "$file" > "$TMP_SEL"
 			while read -r line; do
@@ -155,27 +187,42 @@ $FZF_HEADER" --marker="*" --preview-window=:wrap "$(fzf_borders)" \
 	done
 }
 
+					######################
+					#   MAIN FUNCTION    #
+					######################
+
 main() {
 
 	if ! type fzf > /dev/null 2>&1; then
-		printf "CliFM: fzf: Command not found\n" >&2
-		exit 1
+		printf "clifm: fzf: Command not found\n" >&2
+		exit 127
 	fi
 
-	# The variables below are exported to the environment so that the
-	# previewer script, BFG.sh, executed from within FZF, can make use of
-	# them. Here we define cmd_exists application should be used for different
-	# file types. The implementation for each application is defined in
-	# the BFG file.
-
-	export TMP_SEL="/tmp/fzfnav.sel"
+	export TMP_SEL="${TMPDIR:-/tmp}/fzfnav.sel"
 	rm -rf -- "$TMP_SEL"
-	BFG_CFG_FILE="$(get_bfg_cfg_file)"
+	BFG_CFG_FILE="$(get_file BFG.cfg)"
 	if [ -z "$BFG_CFG_FILE" ]; then
-		printf "CliFM: BFG.cfg: No such file or directory\n" >&2
-		exit 1
+		printf "clifm: BFG.cfg: No such file or directory\n" >&2
+		exit 2
 	fi
 
+	if ! [ -f "${HOME_PLUGINS_DIR}/BFG.cfg" ]; then
+		cp -- "$BFG_CFG_FILE" "${HOME_PLUGINS_DIR}/BFG.cfg" 2>/dev/null
+		BFG_CFG_FILE="${HOME_PLUGINS_DIR}/BFG.cfg"
+	fi
+
+	# Do we have GNU ls?
+	if ls --version >/dev/null 2>&1; then
+		export ls_cmd="ls -Ap --group-directories-first --color=always --indicator-style=none"
+		export POSIX_LS=0
+	else
+		export ls_cmd="ls -Ap"
+		export POSIX_LS=1
+	fi
+
+	# OpenBSD file(1) version has no --mime-encoding option
+	[ "$(uname)" != "OpenBSD" ] && export FILE_HAS_MIME_ENCODING=1
+
 			#################################################
 			#	1. GET VALUES FROM THE CONFIGURATION FILE	#
 			#################################################
@@ -198,14 +245,6 @@ main() {
 				if echo "$value" | grep -qE "[0-9]+"; then
 					export fzfheight="$value"
 				fi ;;
-			LS)
-				if [ "${value:-posix}" = posix ]; then
-					export ls_cmd="ls -Ap"
-					export POSIX_LS=1
-				else
-					export ls_cmd="ls -Ap --group-directories-first --color=always --indicator-style=none"
-					export POSIX_LS=0
-				fi ;;
 			BFG_FILE)
 				if [ -z "$value" ]; then
 					BFG_FILE=""
@@ -335,7 +374,6 @@ main() {
 				case "$value" in
 					ueberzug) export UEBERZUG_OK=1 ;;
 					w3m|kitty|viu|catimg|img2txt|chafa|pixterm)
-#					kitty|viu|catimg|img2txt|chafa)
 						export IMG_VIEWER="$value" ;;
 					none) export IMG_VIEWER="true" ;;
 					*) IMG_VIEWER="" ;;
@@ -412,22 +450,27 @@ main() {
 	done < "$BFG_CFG_FILE"
 
 	export COLORS
-	COLORS="$(tput colors)"
-
-	if [ -z "$ls_cmd" ]; then
-		export ls_cmd="ls -Ap --group-directories-first --color=always --indicator-style=none"
+	if type tput >/dev/null 2>&1; then
+		COLORS="$(tput colors)"
+	else
+		COLORS="8"
 	fi
 
+#	if [ -z "$ls_cmd" ]; then
+#		export ls_cmd="ls -Ap --group-directories-first --color=always --indicator-style=none"
+#	fi
+
 	# This is the previewer script, similar to Ranger's scope.sh
 	if [ -z "$BFG_FILE" ]; then
 		export BFG_FILE
-		BFG_FILE="${XDG_CONFIG_HOME:-$HOME/.config}/clifm/plugins/BFG.sh"
+		BFG_FILE="$(get_file BFG.sh)"
 		if ! [ -f "$BFG_FILE" ]; then
-			BFG_FILE="${BFG_CFG_FILE%.*}.sh"
-			if ! [ -f "$BFG_FILE" ]; then
-				printf "CliFM: BFG.sh: No such file or directory\n" >&2
-				exit 1
-			fi
+			printf "clifm: BFG.sh: No such file or directory\n" >&2
+			exit 2
+		fi
+		if ! [ -x "$BFG_FILE" ]; then
+			printf "clifm: %s: Not an executable file\n" "$BFG_FILE" >&2
+			exit 1
 		fi
 	fi
 
@@ -438,7 +481,7 @@ main() {
 	! [ -d "$PREVIEWDIR" ] && mkdir -p "$PREVIEWDIR"
 
 	if [ "$USE_SCOPE" = 1 ]; then
-		[ -z "$SCOPE_FILE" ] && SCOPE_FILE="${XDG_CONFIG_HOME:-$HOME/.config}/ranger/scope.sh"
+		[ -z "$SCOPE_FILE" ] && export SCOPE_FILE="${XDG_CONFIG_HOME:-$HOME/.config}/ranger/scope.sh"
 	fi
 
 	# If some value was not set in the config file, check for available
@@ -452,7 +495,6 @@ main() {
 	# don't need to do it once and again each time a file is hovered
 
 	# Directories
-
 	if [ -z "$DIR_CMD" ]; then
 		if type lsd > /dev/null 2>&1; then
 			export DIR_CMD="lsd-tree"
@@ -468,7 +510,7 @@ main() {
 	# Images
 	if [ "$PREV_IMGS" = 1 ] && [ -z "$IMG_VIEWER" ] && \
 	[ -n "$DISPLAY" ]; then
-		if type ueberzug > /dev/null 2>&1; then
+		if [ -z "$WAYLAND_DISPLAY" ] && type ueberzug > /dev/null 2>&1; then
 			export UEBERZUG_OK=1
 		elif [ "$TERM" = "xterm-kitty" ]; then
 			export IMG_VIEWER="kitty"
@@ -651,12 +693,14 @@ main() {
 		export FIFO_UEBERZUG="$CACHEDIR/ueberzug-${PPID}"
 		trap uz_cleanup EXIT
 		start_ueberzug
+	else
+		export FIFO_UEBERZUG="/dev/null"
 	fi
 
-	TMP="$(mktemp /tmp/clifm.XXXXXX)"
+	TMP=$(mktemp "${TMPDIR:-/tmp}/clifm.XXXXXX")
 
 				#####################################
-				#	 3. RUN FZF, which CALLS BFG	#
+				#	 3. RUN FZF, WHICH CALLS BFG	#
 				#####################################
 
 	fcd "$@"
@@ -665,9 +709,37 @@ main() {
 	rm -f -- "$TMP" 2>/dev/null
 }
 
+
+					#####################
+					#       MAIN        #
+					#####################
+
+if [ -n "$1" ] && { [ "$1" = "--help" ] || [ "$1" = "-h" ]; }; then
+#	name="${CLIFM_PLUGIN_NAME:-$(basename "$0")}"
+	printf "Navigate/preview/select files via FZF\n"
+	printf "\n%s\n" "$HELP"
+	exit 0
+fi
+
+if [ -n "$1" ] && [ "$1" = "--edit" ]; then
+	f="$(get_file BFG.cfg)"
+	if [ -z "$f" ]; then
+		printf "clifm: BFG.cfg: No such file or directory\n" >&2
+		exit 2
+	fi
+
+	if ! [ -f "${HOME_PLUGINS_DIR}/BFG.cfg" ]; then
+		cp -- "$f" "${HOME_PLUGINS_DIR}/BFG.cfg" 2>/dev/null
+		f="${HOME_PLUGINS_DIR}/BFG.cfg"
+	fi
+
+	"${EDITOR:-VISUAL:-nano}" "$f" && exit 0
+	exit 1
+fi
+
 # Source our plugins helper
 if [ -z "$CLIFM_PLUGINS_HELPER" ] || ! [ -f "$CLIFM_PLUGINS_HELPER" ]; then
-	printf "CliFM: Unable to find plugins-helper file\n" >&2
+	printf "clifm: Unable to find plugins-helper file\n" >&2
 	exit 1
 fi
 # shellcheck source=/dev/null
