@@ -1,13 +1,15 @@
 #!/bin/sh
 
 # CliFM plugin to navigate the jump database via fzf/Rofi
+# Dependencies: fzf or rofi, and grep
+
 # Written by L. Abramovich
 # Lincese GPL3
 
-if [ -n "$1" ] && { [ "$1" = "--help" ] || [ "$1" = "help" ]; }; then
-	name="$(basename "$0")"
-	printf "Navigate CLiFM jump database via FZF or Rofi. Press Enter to cd into the selected directory\n"
-	printf "Usage: %s\n" "$name"
+if [ -n "$1" ] && { [ "$1" = "--help" ] || [ "$1" = "-h" ]; }; then
+	name="${CLIFM_PLUGIN_NAME:-$(basename "$0")}"
+	printf "Navigate CliFM's jump database via FZF or Rofi. Press Enter to cd into the selected directory\n"
+	printf "\n\x1b[1mUSAGE\x1b[0m\n  %s\n" "$name"
 	exit 0
 fi
 
@@ -17,11 +19,11 @@ if type fzf > /dev/null 2>&1; then
 elif type rofi > /dev/null 2>&1; then
 	finder="rofi"
 else
-	printf "CliFM: No finder found. Install either FZF or Rofi\n" >&2
+	printf "clifm: No finder found. Install either FZF or Rofi\n" >&2
 	exit 1
 fi
 
-FILE="${XDG_CONFIG_HOME:-${HOME}/.config}/clifm/profiles/$CLIFM_PROFILE/jump.cfm"
+FILE="${XDG_CONFIG_HOME:-${HOME}/.config}/clifm/profiles/$CLIFM_PROFILE/jump.clifm"
 
 if ! [ -f "$FILE" ]; then
 	exit 1
@@ -30,7 +32,7 @@ fi
 if [ "$finder" = "fzf" ]; then
 	# Source our plugins helper
 	if [ -z "$CLIFM_PLUGINS_HELPER" ] || ! [ -f "$CLIFM_PLUGINS_HELPER" ]; then
-		printf "CliFM: Unable to find plugins-helper file\n" >&2
+		printf "clifm: Unable to find plugins-helper file\n" >&2
 		exit 1
 	fi
 	# shellcheck source=/dev/null
