@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -28,22 +28,23 @@
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
-#include <sys/stat.h>
-#include <unistd.h>
 #include <readline/readline.h>
 
 #include "aux.h"
+#include "checks.h"
 #include "exec.h"
 #include "history.h"
 #include "jump.h"
 #include "listing.h"
+#include "misc.h"
 #include "navigation.h"
 #include "readline.h"
-#include "checks.h"
 
 #ifndef _NO_MAGIC
-#include "mime.h"
-#endif
+# include "mime.h"
+#else
+# include <unistd.h>
+#endif /* !_NO_MAGIC */
 
 #define OP_ISO    1
 #define OP_OTHERS 0
@@ -57,10 +58,10 @@ ask_user_for_path(void)
 	char *ext_path = (char *)NULL;
 
 	while (!ext_path) {
-		int bk = warning_prompt;
-		warning_prompt = 0;
+		int bk = conf.warning_prompt;
+		conf.warning_prompt = 0;
 		ext_path = readline(_("Extraction path ('q' to quit): "));
-		warning_prompt = bk;
+		conf.warning_prompt = bk;
 		if (!ext_path)
 			continue;
 		if (!*ext_path) {
@@ -77,10 +78,7 @@ static char *
 get_extraction_path(void)
 {
 	char *ext_path = ask_user_for_path();
-	if (!ext_path)
-		return (char *)NULL;
-
-	if (!*ext_path) {
+	if (!ext_path || !*ext_path) {
 		free(ext_path);
 		return (char *)NULL;
 	}
@@ -109,7 +107,8 @@ get_extraction_path(void)
 	return ext_path;
 }
 
-static char get_operation(const int mode)
+static char
+get_operation(const int mode)
 {
 	char sel_op = 0;
 	char *op = (char *)NULL;
@@ -168,12 +167,13 @@ extract_iso(char *file)
 	int exit_status = EXIT_SUCCESS;
 
 	/* 7z x -oDIR FILE (use FILE as DIR) */
-	char *o_option = (char *)xnmalloc(strlen(file) + 7, sizeof(char));
-	sprintf(o_option, "-o%s.dir", file); /* NOLINT */
+	size_t flen = strlen(file);
+	char *o_option = (char *)xnmalloc(flen + 7, sizeof(char));
+	snprintf(o_option, flen + 7, "-o%s.dir", file); /* NOLINT */
 
 	/* Construct and execute cmd */
 	char *cmd[] = {"7z", "x", o_option, file, NULL};
-	if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 		exit_status = EXIT_FAILURE;
 
 	free(o_option);
@@ -190,13 +190,14 @@ extract_iso_to_dir(char *file)
 	if (!ext_path)
 		return EXIT_FAILURE;
 
-	char *o_option = (char *)xnmalloc(strlen(ext_path) + 3, sizeof(char));
-	sprintf(o_option, "-o%s", ext_path); /* NOLINT */
+	size_t len = strlen(ext_path);
+	char *o_option = (char *)xnmalloc(len + 3, sizeof(char));
+	snprintf(o_option, len + 3, "-o%s", ext_path); /* NOLINT */
 	free(ext_path);
 
 	/* Construct and execute cmd */
 	char *cmd[] = {"7z", "x", o_option, file, NULL};
-	if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 		exit_status = EXIT_FAILURE;
 
 	free(o_option);
@@ -211,7 +212,7 @@ list_iso_contents(char *file)
 
 	/* 7z l FILE */
 	char *cmd[] = {"7z", "l", file, NULL};
-	if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 		exit_status = EXIT_FAILURE;
 
 	return exit_status;
@@ -224,7 +225,7 @@ test_iso(char *file)
 
 	/* 7z t FILE */
 	char *cmd[] = {"7z", "t", file, NULL};
-	if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 		exit_status = EXIT_FAILURE;
 
 	return exit_status;
@@ -240,17 +241,17 @@ create_mountpoint(char *file)
 	char *tfile = (p && *(++p)) ? p : file;
 
 	if (xargs.stealth_mode == 1) {
-		mountpoint = (char *)xnmalloc(strlen(tfile) + P_tmpdir_len + 15,
-					sizeof(char));
-		sprintf(mountpoint, "%s/clifm-mounts/%s", P_tmpdir, tfile); /* NOLINT */
+		size_t len = strlen(tfile) + P_tmpdir_len + 15;
+		mountpoint = (char *)xnmalloc(len, sizeof(char));
+		snprintf(mountpoint, len, "%s/clifm-mounts/%s", P_tmpdir, tfile);
 	} else {
-		mountpoint = (char *)xnmalloc(config_dir_len + strlen(tfile) + 9,
-					sizeof(char));
-		sprintf(mountpoint, "%s/mounts/%s", config_dir, tfile); /* NOLINT */
+		size_t len = config_dir_len + strlen(tfile) + 9;
+		mountpoint = (char *)xnmalloc(len, sizeof(char));
+		snprintf(mountpoint, len, "%s/mounts/%s", config_dir, tfile); /* NOLINT */
 	}
 
 	char *dir_cmd[] = {"mkdir", "-pm700", mountpoint, NULL};
-	if (launch_execve(dir_cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
+	if (launch_execv(dir_cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
 		free(mountpoint);
 		mountpoint = (char *)NULL;
 	}
@@ -258,11 +259,12 @@ create_mountpoint(char *file)
 	return mountpoint;
 }
 
+#if defined(__linux__)
 static int
 cd_to_mountpoint(char *file, char *mountpoint)
 {
 	if (xchdir(mountpoint, SET_TITLE) == -1) {
-		fprintf(stderr, "archiver: %s: %s\n", mountpoint, strerror(errno));
+		xerror("archiver: %s: %s\n", mountpoint, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
@@ -271,10 +273,8 @@ cd_to_mountpoint(char *file, char *mountpoint)
 	add_to_jumpdb(workspaces[cur_ws].path);
 
 	int exit_status = EXIT_SUCCESS;
-	if (autols) {
-		free_dirlist();
-		if (list_dir() != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
+	if (conf.autols == 1) {
+		reload_dirlist();
 		add_to_dirhist(workspaces[cur_ws].path);
 	} else {
 		printf("%s: Successfully mounted on %s\n", file, mountpoint);
@@ -282,10 +282,16 @@ cd_to_mountpoint(char *file, char *mountpoint)
 
 	return exit_status;
 }
+#endif /* __linux__ */
 
 static int
 mount_iso(char *file)
 {
+#if !defined(__linux__)
+	UNUSED(file);
+	xerror("%s\n", "mount: This feature is for Linux only");
+	return EXIT_SUCCESS;
+#else
 	char *mountpoint = create_mountpoint(file);
 	if (!mountpoint)
 		return EXIT_FAILURE;
@@ -296,9 +302,15 @@ mount_iso(char *file)
 		free(mountpoint);
 		return EXIT_FAILURE;
 	}
-	
+
 	char *cmd[] = {sudo, "mount", "-o", "loop", file, mountpoint, NULL};
-	if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
+	if (confirm_sudo_cmd(cmd) == 0) {
+		free(mountpoint);
+		free(sudo);
+		return EXIT_SUCCESS;
+	}
+
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
 		free(mountpoint);
 		free(sudo);
 		return EXIT_FAILURE;
@@ -308,6 +320,7 @@ mount_iso(char *file)
 	int exit_status = cd_to_mountpoint(file, mountpoint);
 	free(mountpoint);
 	return exit_status;
+#endif /* __linux__ */
 }
 
 /* Use 7z to
@@ -324,14 +337,12 @@ handle_iso(char *file)
 
 	char sel_op = get_operation(OP_ISO);
 
-	char *ret = strchr(file, '\\');
-	if (ret) {
+	if (strchr(file, '\\')) {
 		char *deq_file = dequote_str(file, 0);
 		if (deq_file) {
-			strcpy(file, deq_file); /* NOLINT */
+			xstrsncpy(file, deq_file, strlen(deq_file) + 1);
 			free(deq_file);
 		}
-		ret = (char *)NULL;
 	}
 
 	switch (sel_op) {
@@ -347,11 +358,13 @@ handle_iso(char *file)
 static int
 create_iso_from_block_dev(char *in_file, char *out_file)
 {
-	char *if_option = (char *)xnmalloc(strlen(in_file) + 4, sizeof(char));
-	sprintf(if_option, "if=%s", in_file); /* NOLINT */
+	size_t len = strlen(in_file) + 4;
+	char *if_option = (char *)xnmalloc(len, sizeof(char));
+	snprintf(if_option, len, "if=%s", in_file); /* NOLINT */
 
-	char *of_option = (char *)xnmalloc(strlen(out_file) + 4, sizeof(char));
-	sprintf(of_option, "of=%s", out_file); /* NOLINT */
+	len = strlen(out_file) + 4;
+	char *of_option = (char *)xnmalloc(len, sizeof(char));
+	snprintf(of_option, len, "of=%s", out_file); /* NOLINT */
 
 	char *sudo = get_sudo_path();
 	if (!sudo) {
@@ -363,8 +376,11 @@ create_iso_from_block_dev(char *in_file, char *out_file)
 	int exit_status = EXIT_SUCCESS;
 	char *cmd[] = {sudo, "dd", if_option, of_option, "bs=64k",
 	    "conv=noerror,sync", "status=progress", NULL};
-	if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
-		exit_status = EXIT_FAILURE;
+
+	if (confirm_sudo_cmd(cmd) == 1) {
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+			exit_status = EXIT_FAILURE;
+	}
 
 	free(sudo);
 	free(if_option);
@@ -378,14 +394,14 @@ create_iso(char *in_file, char *out_file)
 {
 	struct stat attr;
 	if (lstat(in_file, &attr) == -1) {
-		fprintf(stderr, "archiver: %s: %s\n", in_file, strerror(errno));
+		xerror("archiver: %s: %s\n", in_file, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
 	/* If IN_FILE is a directory */
 	if (S_ISDIR(attr.st_mode)) {
 		char *cmd[] = {"mkisofs", "-R", "-o", out_file, in_file, NULL};
-		if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 			return EXIT_FAILURE;
 		return EXIT_SUCCESS;
 	}
@@ -395,19 +411,18 @@ create_iso(char *in_file, char *out_file)
 		return create_iso_from_block_dev(in_file, out_file);
 
 	/* If any other file format */
-	fprintf(stderr, "archiver: %s: Invalid file format\nFile should "
-			"be either a directory or a block device\n", in_file);
+	xerror(_("archiver: %s: Invalid file format. File should be either "
+		"a directory or a block device\n"), in_file);
 	return EXIT_FAILURE;
 }
 
-/* Run the 'file' command on FILE and look for "ISO 9660" and
- * string in its output. Returns zero if found, one if not, and -1
- * in case of error */
+/* Check the MIME type of the file named FILE and look for "ISO 9660" in its
+ * output. Returns zero if found, one if not, and -1 in case of error. */
 static int
 check_iso(char *file)
 {
 	if (!file || !*file) {
-		fputs(_("Error querying file type\n"), stderr);
+		xerror("%s\n", _("Error querying file type"));
 		return (-1);
 	}
 
@@ -416,7 +431,7 @@ check_iso(char *file)
 #ifndef _NO_MAGIC
 	char *t = xmagic(file, TEXT_DESC);
 	if (!t) {
-		fputs(_("Error querying file type\n"), stderr);
+		xerror("%s\n", _("Error querying file type"));
 		return (-1);
 	}
 
@@ -432,23 +447,22 @@ check_iso(char *file)
 
 	char iso_tmp_file[PATH_MAX];
 	if (xargs.stealth_mode == 1)
-		snprintf(iso_tmp_file, PATH_MAX - 1, "%s/.clifm%s", P_tmpdir, rand_ext);
+		snprintf(iso_tmp_file, PATH_MAX - 1, "%s/.temp%s", P_tmpdir, rand_ext);
 	else
-		snprintf(iso_tmp_file, PATH_MAX - 1, "%s/.clifm%s", tmp_dir, rand_ext);
+		snprintf(iso_tmp_file, PATH_MAX - 1, "%s/.temp%s", tmp_dir, rand_ext);
 	free(rand_ext);
 
 	int fd;
-	FILE *fp = open_fstream_w(iso_tmp_file, &fd);
+	FILE *fp = open_fwrite(iso_tmp_file, &fd);
 	if (!fp) {
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, iso_tmp_file,
-				strerror(errno));
+		xerror("archiver: %s: %s\n", iso_tmp_file, strerror(errno));
 		return (-1);
 	}
 
 	FILE *fpp = fopen("/dev/null", "w");
 	if (!fpp) {
-		fprintf(stderr, "%s: /dev/null: %s\n", PROGRAM_NAME, strerror(errno));
-		close_fstream(fp, fd);
+		xerror("archiver: /dev/null: %s\n", strerror(errno));
+		fclose(fp);
 		return (-1);
 	}
 
@@ -457,25 +471,25 @@ check_iso(char *file)
 
 	/* Redirect stdout to the desired file */
 	if (dup2(fileno(fp), STDOUT_FILENO) == -1) {
-		fprintf(stderr, "%s: %s\n", PROGRAM_NAME, strerror(errno));
-		close_fstream(fp, fd);
+		xerror("archiver: %s\n", strerror(errno));
+		fclose(fp);
 		fclose(fpp);
 		return (-1);
 	}
 
 	/* Redirect stderr to /dev/null */
 	if (dup2(fileno(fpp), STDERR_FILENO) == -1) {
-		fprintf(stderr, "%s: %s\n", PROGRAM_NAME, strerror(errno));
-		close_fstream(fp, fd);
+		xerror("archiver: %s\n", strerror(errno));
+		fclose(fp);
 		fclose(fpp);
 		return (-1);
 	}
 
-	close_fstream(fp, fd);
+	fclose(fp);
 	fclose(fpp);
 
 	char *cmd[] = {"file", "-b", file, NULL};
-	int retval = launch_execve(cmd, FOREGROUND, E_NOFLAG);
+	int retval = launch_execv(cmd, FOREGROUND, E_NOFLAG);
 
 	dup2(stdout_bk, STDOUT_FILENO); /* Restore original stdout */
 	dup2(stderr_bk, STDERR_FILENO); /* Restore original stderr */
@@ -486,26 +500,24 @@ check_iso(char *file)
 		return (-1);
 
 	if (access(iso_tmp_file, F_OK) == 0) {
-		fp = open_fstream_r(iso_tmp_file, &fd);
+		fp = open_fread(iso_tmp_file, &fd);
 		if (fp) {
 			char line[255] = "";
 			if (fgets(line, (int)sizeof(line), fp) == NULL) {
-				close_fstream(fp, fd);
+				fclose(fp);
 				unlink(iso_tmp_file);
 				return EXIT_FAILURE;
 			}
 			char *ret = strstr(line, "ISO 9660");
 			if (ret)
 				is_iso = 1;
-			close_fstream(fp, fd);
+			fclose(fp);
 		}
 		unlink(iso_tmp_file);
 	}
 #endif /* !_NO_MAGIC */
 
-	if (is_iso)
-		return EXIT_SUCCESS;
-	return EXIT_FAILURE;
+	return is_iso == 1 ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 /* Run the 'file' command on FILE and look for "archive" and
@@ -517,10 +529,10 @@ check_iso(char *file)
  * well (and there is no need to run two functions (is_compressed and
  * check_iso), when we can run just one) */
 int
-is_compressed(char *file, int test_iso)
+is_compressed(char *file, const int test_iso)
 {
 	if (!file || !*file) {
-		fputs(_("Error querying file type\n"), stderr);
+		xerror("%s\n", _("Error querying file type"));
 		return (-1);
 	}
 
@@ -529,7 +541,7 @@ is_compressed(char *file, int test_iso)
 #ifndef _NO_MAGIC
 	char *t = xmagic(file, TEXT_DESC);
 	if (!t) {
-		fputs(_("Error querying file type\n"), stderr);
+		xerror("%s\n", _("Error querying file type"));
 		return (-1);
 	}
 
@@ -557,23 +569,24 @@ is_compressed(char *file, int test_iso)
 
 	char archiver_tmp_file[PATH_MAX];
 	if (xargs.stealth_mode == 1)
-		snprintf(archiver_tmp_file, PATH_MAX - 1, "%s/.clifm%s", P_tmpdir, rand_ext);
+		snprintf(archiver_tmp_file, sizeof(archiver_tmp_file), "%s/.clifm%s",
+			P_tmpdir, rand_ext);
 	else
-		snprintf(archiver_tmp_file, PATH_MAX - 1, "%s/.clifm%s", tmp_dir, rand_ext);
+		snprintf(archiver_tmp_file, sizeof(archiver_tmp_file), "%s/.clifm%s",
+			tmp_dir, rand_ext);
 	free(rand_ext);
 
 	int fd;
-	FILE *fp = open_fstream_w(archiver_tmp_file, &fd);
+	FILE *fp = open_fwrite(archiver_tmp_file, &fd);
 	if (!fp) {
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME,
-		    archiver_tmp_file, strerror(errno));
+		xerror("archiver: %s: %s\n", archiver_tmp_file, strerror(errno));
 		return (-1);
 	}
 
 	FILE *fpp = fopen("/dev/null", "w");
 	if (!fpp) {
-		fprintf(stderr, "%s: /dev/null: %s\n", PROGRAM_NAME, strerror(errno));
-		close_fstream(fp, fd);
+		xerror("archiver: /dev/null: %s\n", strerror(errno));
+		fclose(fp);
 		return (-1);
 	}
 
@@ -582,25 +595,25 @@ is_compressed(char *file, int test_iso)
 
 	/* Redirect stdout to the desired file */
 	if (dup2(fileno(fp), STDOUT_FILENO) == -1) {
-		fprintf(stderr, "%s: %s\n", PROGRAM_NAME, strerror(errno));
-		close_fstream(fp, fd);
+		xerror("archiver: %s\n", strerror(errno));
+		fclose(fp);
 		fclose(fpp);
 		return (-1);
 	}
 
 	/* Redirect stderr to /dev/null */
 	if (dup2(fileno(fpp), STDERR_FILENO) == -1) {
-		fprintf(stderr, "%s: %s\n", PROGRAM_NAME, strerror(errno));
-		close_fstream(fp, fd);
+		xerror("archiver: %s\n", strerror(errno));
+		fclose(fp);
 		fclose(fpp);
 		return -1;
 	}
 
-	close_fstream(fp, fd);
+	fclose(fp);
 	fclose(fpp);
 
 	char *cmd[] = {"file", "-b", file, NULL};
-	int retval = launch_execve(cmd, FOREGROUND, E_NOFLAG);
+	int retval = launch_execv(cmd, FOREGROUND, E_NOFLAG);
 
 	dup2(stdout_bk, STDOUT_FILENO); /* Restore original stdout */
 	dup2(stderr_bk, STDERR_FILENO); /* Restore original stderr */
@@ -611,11 +624,11 @@ is_compressed(char *file, int test_iso)
 		return (-1);
 
 	if (access(archiver_tmp_file, F_OK) == 0) {
-		fp = open_fstream_r(archiver_tmp_file, &fd);
+		fp = open_fread(archiver_tmp_file, &fd);
 		if (fp) {
 			char line[255];
 			if (fgets(line, (int)sizeof(line), fp) == NULL) {
-				close_fstream(fp, fd);
+				fclose(fp);
 				unlink(archiver_tmp_file);
 				return EXIT_FAILURE;
 			}
@@ -636,7 +649,7 @@ is_compressed(char *file, int test_iso)
 				}
 			}
 
-			close_fstream(fp, fd);
+			fclose(fp);
 		}
 
 		unlink(archiver_tmp_file);
@@ -656,7 +669,7 @@ add_default_extension(char *name)
 
 	char *t = savestring(name, name_len);
 	name = (char *)xrealloc(name, (name_len + 8) * sizeof(char));
-	sprintf(name, "%s.tar.gz", t); /* NOLINT */
+	snprintf(name, name_len + 8, "%s.tar.gz", t); /* NOLINT */
 	free(t);
 
 	return name;
@@ -670,7 +683,9 @@ get_archive_filename(void)
 	       "(defaults to .tar.gz)\nExample: myarchive.xz"));
 	char *name = (char *)NULL;
 	while (!name) {
+		flags |= NO_FIX_RL_POINT;
 		name = rl_no_hist(_("File name ('q' to quit): "));
+		flags &= ~NO_FIX_RL_POINT;
 		if (!name)
 			continue;
 		if (!*name) {
@@ -689,7 +704,7 @@ get_archive_filename(void)
 			return add_default_extension(name);
 
 		if (dot == name) { /* Dot is first char */
-			fprintf(stderr, _("Invalid file name\n"));
+			xerror("%s\n", _("Invalid file name"));
 			free(name);
 			name = (char *)NULL;
 			continue;
@@ -705,24 +720,24 @@ get_archive_filename(void)
  * tells if we have one or multiple file. Returns zero on success and
  * one on error */
 static int
-zstandard(char *in_file, char *out_file, char mode, char op)
+zstandard(char *in_file, char *out_file, const char mode, const char op)
 {
 	int exit_status = EXIT_SUCCESS;
 	char *deq_file = dequote_str(in_file, 0);
 	if (!deq_file) {
-		fprintf(stderr, _("archiver: %s: Error dequoting file name\n"), in_file);
+		xerror(_("archiver: %s: Error dequoting file name\n"), in_file);
 		return EXIT_FAILURE;
 	}
 
 	if (mode == 'c') {
 		if (out_file) {
 			char *cmd[] = {"zstd", "-zo", out_file, deq_file, NULL};
-			if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 				exit_status = EXIT_FAILURE;
 		} else {
 			char *cmd[] = {"zstd", "-z", deq_file, NULL};
 
-			if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 				exit_status = EXIT_FAILURE;
 		}
 
@@ -738,14 +753,14 @@ zstandard(char *in_file, char *out_file, char mode, char op)
 		char option[3] = "";
 
 		switch (op) {
-		case 'e': strcpy(option, "-d"); break;
-		case 't': strcpy(option, "-t"); break;
-		case 'i': strcpy(option, "-l"); break;
+		case 'e': xstrsncpy(option, "-d", sizeof(option)); break;
+		case 't': xstrsncpy(option, "-t", sizeof(option)); break;
+		case 'i': xstrsncpy(option, "-l", sizeof(option)); break;
 		default: break;
 		}
 
 		char *cmd[] = {"zstd", option, deq_file, NULL};
-		exit_status = launch_execve(cmd, FOREGROUND, E_NOFLAG);
+		exit_status = launch_execv(cmd, FOREGROUND, E_NOFLAG);
 		free(deq_file);
 
 		if (exit_status != EXIT_SUCCESS)
@@ -771,19 +786,19 @@ zstandard(char *in_file, char *out_file, char mode, char op)
 		switch (*operation) {
 		case 'e': {
 			char *cmd[] = {"zstd", "-d", deq_file, NULL};
-			if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 				exit_status = EXIT_FAILURE;
 		} break;
 
 		case 't': {
 			char *cmd[] = {"zstd", "-t", deq_file, NULL};
-			if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 				exit_status = EXIT_FAILURE;
 		} break;
 
 		case 'i': {
 			char *cmd[] = {"zstd", "-l", deq_file, NULL};
-			if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+			if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 				exit_status = EXIT_FAILURE;
 		} break;
 
@@ -837,8 +852,9 @@ compress_others(char **args, char *name)
 	char **tcmd = (char **)xnmalloc(3 + i + 1, sizeof(char *));
 	tcmd[0] = savestring("atool", 5);
 	tcmd[1] = savestring("-a", 2);
-	tcmd[2] = (char *)xnmalloc(strlen(name) + (!ext_ok ? 7 : 0) + 1, sizeof(char *));
-	sprintf(tcmd[2], "%s%s", name, !ext_ok ? ".tar.gz" : ""); /* NOLINT */
+	size_t len = strlen(name) + (!ext_ok ? 7 : 0) + 1;
+	tcmd[2] = (char *)xnmalloc(len, sizeof(char *));
+	snprintf(tcmd[2], len, "%s%s", name, !ext_ok ? ".tar.gz" : ""); /* NOLINT */
 	n += 3;
 
 	for (i = 1; args[i]; i++) {
@@ -851,7 +867,7 @@ compress_others(char **args, char *name)
 	}
 	tcmd[n] = (char *)NULL;
 
-	int ret = launch_execve(tcmd, FOREGROUND, E_NOFLAG);
+	int ret = launch_execv(tcmd, FOREGROUND, E_NOFLAG);
 
 	for (i = 0; tcmd[i]; i++)
 		free(tcmd[i]);
@@ -901,13 +917,12 @@ check_not_compressed(char **args)
 		char *deq = (char *)NULL;
 		if (strchr(args[i], '\\')) {
 			deq = dequote_str(args[i], 0);
-			strcpy(args[i], deq); /* NOLINT */
+			xstrsncpy(args[i], deq, strlen(deq) + 1);
 			free(deq);
 		}
 
 		if (is_compressed(args[i], 1) != 0) {
-			fprintf(stderr, _("archiver: %s: Not an archive/compressed file\n"),
-					args[i]);
+			xerror(_("archiver: %s: Not an archive/compressed file\n"), args[i]);
 			return 1;
 		}
 	}
@@ -915,20 +930,30 @@ check_not_compressed(char **args)
 	return 0;
 }
 
-/* Check if we have at least one Zstandard file */
+/* Check whether we have at least one Zstandard file */
 static size_t
 check_zstandard(char **args)
 {
 	size_t zst = 0, i;
 
 	for (i = 1; args[i]; i++) {
-		if (args[i][strlen(args[i]) - 1] != 't')
+#ifndef _NO_MAGIC
+		char *mime = xmagic(args[i], MIME_TYPE);
+		if (!mime || !*mime)
 			continue;
+		if (strcmp(mime, "application/zstd") == 0) {
+			zst = 1;
+			free(mime);
+			break;
+		}
+		free(mime);
+#else
 		char *ret = strrchr(args[i], '.');
 		if (ret && strcmp(ret, ".zst") == 0) {
 			zst = 1;
 			break;
 		}
+#endif /* !_NO_MAGIC */
 	}
 
 	return zst;
@@ -1006,11 +1031,10 @@ list_others(char **args)
 
 	size_t i;
 	for (i = 1; args[i]; i++) {
-		printf(_("%s%sFile%s: %s\n"), (i > 1) ? "\n" : "",
-		    BOLD, df_c, args[i]);
+		printf(_("%s%sFile%s: %s\n"), (i > 1) ? "\n" : "", BOLD, df_c, args[i]);
 
 		char *cmd[] = {"atool", "-l", args[i], NULL};
-		if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 			exit_status = EXIT_FAILURE;
 	}
 
@@ -1033,7 +1057,7 @@ extract_to_dir_others(char **args)
 
 		/* Construct and execute cmd */
 		char *cmd[] = {"atool", "-X", ext_path, args[i], NULL};
-		if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 			exit_status = EXIT_FAILURE;
 
 		free(ext_path);
@@ -1065,7 +1089,7 @@ extract_others(char **args)
 
 	/* Launch it */
 	int exit_status = EXIT_SUCCESS;
-	if (launch_execve(tcmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+	if (launch_execv(tcmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 		exit_status = EXIT_FAILURE;
 
 	for (i = 0; tcmd[i]; i++)
@@ -1129,7 +1153,7 @@ repack_others(char **args)
 	tcmd[n] = (char *)NULL;
 
 	int exit_status = EXIT_SUCCESS;
-	if (launch_execve(tcmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+	if (launch_execv(tcmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 		exit_status = EXIT_FAILURE;
 
 	for (i = 0; tcmd[i]; i++)
@@ -1144,7 +1168,7 @@ static int
 list_mounted_files(char *mountpoint)
 {
 	if (xchdir(mountpoint, SET_TITLE) == -1) {
-		fprintf(stderr, "archiver: %s: %s\n", mountpoint, strerror(errno));
+		xerror("archiver: %s: %s\n", mountpoint, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
@@ -1153,7 +1177,7 @@ list_mounted_files(char *mountpoint)
 	add_to_jumpdb(workspaces[cur_ws].path);
 
 	int exit_status = EXIT_SUCCESS;
-	if (autols) {
+	if (conf.autols) {
 		free_dirlist();
 		if (list_dir() != EXIT_SUCCESS)
 			exit_status = EXIT_FAILURE;
@@ -1178,7 +1202,7 @@ mount_others(char **args)
 
 		/* Construct and execute cmd */
 		char *cmd[] = {"archivemount", args[i], mountpoint, NULL};
-		if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
 			free(mountpoint);
 			continue;
 		}
@@ -1186,7 +1210,7 @@ mount_others(char **args)
 		/* List content of mountpoint if there is only one archive */
 		if (files_num > 1) {
 			printf(_("%s%s%s: Succesfully mounted on %s\n"),
-					BOLD, args[i], df_c, mountpoint);
+				BOLD, args[i], df_c, mountpoint);
 			free(mountpoint);
 			continue;
 		}
@@ -1249,7 +1273,7 @@ decompress_files(char **args)
  * zero on success and one on error. Depends on 'zstd' for Zdtandard
  * files, 'atool' and 'archivemount' for the remaining types. */
 int
-archiver(char **args, char mode)
+archiver(char **args, const char mode)
 {
 	if (!args[1])
 		return EXIT_FAILURE;
@@ -1262,6 +1286,6 @@ archiver(char **args, char mode)
 }
 
 #else
-void *__skip_me_archiving;
+void *_skip_me_archiving;
 #endif /* !_NO_ARCHIVING */
 
