@@ -1,8 +1,8 @@
 /* autocmds.h
  *
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -24,10 +24,13 @@
 #ifndef AUTOCMDS_H
 #define AUTOCMDS_H
 
-void parse_autocmd_line(char *cmd);
+__BEGIN_DECLS
+
+void parse_autocmd_line(char *);
 void reset_opts(void);
-int check_autocmds(void);
+int  check_autocmds(void);
 void revert_autocmd_opts(void);
-void free_autocmds(void);
+
+__END_DECLS
 
 #endif /* AUTOCMDS_H */
