@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,10 +25,12 @@
 #ifndef BOOKMARKS_H
 #define BOOKMARKS_H
 
-/* bookmarks.c */
-int bookmarks_function(char **cmd);
-int edit_bookmarks(char *cmd);
-int open_bookmark(void);
+__BEGIN_DECLS
+
+int  bookmarks_function(char **);
+int  open_bookmark(void);
 void free_bookmarks(void);
 
+__END_DECLS
+
 #endif /* BOOKMARKS_H */
