@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2021, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,23 +25,33 @@
 #ifndef CHECKS_H
 #define CHECKS_H
 
-#include <sys/stat.h>
+__BEGIN_DECLS
 
-/* checks.c */
-void file_cmd_check(void);
-void check_file_size(char *log_file, int max);
-char **check_for_alias(char **args);
-int check_regex(char *str);
-int check_immutable_bit(char *file);
-int is_bin_cmd(char *str);
-int is_internal(const char *cmd);
-int is_internal_c(const char *restrict cmd);
-int is_number(const char *restrict str);
-int is_acl(char *file);
-char *get_sudo_path(void);
+#ifndef _NO_FZF
+void check_completion_mode(void);
+#endif /* !_NO_FZF */
+
+int  check_file_access(const mode_t, const uid_t, const gid_t);
+char **check_for_alias(char **);
+int  check_glob_char(const char *, const int);
+int  check_immutable_bit(char *);
+int  check_regex(char *);
 void check_term(void);
-int check_file_access(const struct stat *file);
 void check_third_party_cmds(void);
-int is_url(char *url);
+void file_cmd_check(void);
+char *get_sudo_path(void);
+int  is_action_name(const char *);
+int  is_bin_cmd(char *);
+int  is_file_in_cwd(char *);
+int  is_force_param(const char *);
+int  is_internal(const char *);
+int  is_internal_c(const char *restrict);
+int  is_internal_f(const char *restrict);
+int  is_number(const char *restrict);
+int  is_acl(char *);
+int  is_url(const char *);
+void truncate_file(char *, const int, const int);
+
+__END_DECLS
 
 #endif /* CHECKS_H */
