@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,20 +25,27 @@
 #ifndef COLORS_H
 #define COLORS_H
 
-#include <sys/stat.h>
+__BEGIN_DECLS
 
-int set_colors(const char *colorscheme, const int env);
 void color_codes(void);
+void colors_list(char *, const int, const int, const int);
+int  cschemes_function(char **);
 #ifndef CLIFM_SUCKLESS
 size_t get_colorschemes(void);
 #endif /* CLIFM_SUCKLESS */
-int cschemes_function(char **args);
-void colors_list(char *ent, const int i, const int pad, const int new_line);
-char *get_ext_color(char *ext);
-char *get_dir_color(const char *filename, const mode_t mode);
-char *get_file_color(const char *filename, const struct stat *attr);
+char *get_dir_color(const char *, const mode_t, const nlink_t, const int);
+char *get_ext_color(const char *);
+char *get_file_color(const char *, const struct stat *);
+char *get_regfile_color(const char *, const struct stat *, int *);
+int  import_color_scheme(const char *);
+void update_warning_prompt_text_color(void);
+void remove_bold_attr(char *);
+char *remove_trash_ext(char **);
 void reset_filetype_colors(void);
 void reset_iface_colors(void);
-void unset_suggestions_color(void);
+int  set_colors(const char *, const int);
+void set_default_colors(void);
+
+__END_DECLS
 
 #endif /* COLORS_H */
