@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,16 +25,21 @@
 #ifndef CONFIG_H
 #define CONFIG_H
 
-int edit_function(char **comm);
-int create_config(char *file);
-int create_kbinds_file(void);
-void init_config(void);
-int create_bm_file(void);
-int create_mime_file(char *file, int new_prof);
-int reload_config(void);
+__BEGIN_DECLS
+
+int  create_bm_file(void);
+int  create_main_config_file(char *);
+int  create_kbinds_file(void);
+int  create_mime_file(char *, const int);
 void create_tmp_files(void);
-void set_sel_file(void);
+int  edit_function(char **);
+void init_config(void);
+int  reload_config(void);
+void set_div_line(char *);
 void set_env(void);
-void set_div_line(const char *line);
+void set_finder_paths(void);
+void set_sel_file(void);
+
+__END_DECLS
 
 #endif /* CONFIG_H */
