@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -30,11 +30,9 @@ typedef char *rl_cpvfunc_t;
 #include <ereadline/readline/readline.h>
 #else
 #include <readline/readline.h>
-#endif
+#endif /* __OpenBSD__ */
 
-#include "strings.h"
 #include "checks.h"
-#include "aux.h"
 
 /* Macros for single and double quotes */
 #define _SINGLE 0
@@ -59,18 +57,24 @@ change_word_color(const char *_last_word, const int offset, const char *color)
 	fputs("\x1b[?25h", stdout);
 } */
 
-/* Get the appropriate color for C and print the color (returning a null
- * pointer) if SET_COLOR is set to 1; otherwise, just return a pointer
- * to the corresponding color. This function is used to colorize input,
- * history entries, and accepted suggestions */
+/* Get the appropriate color for the character at position POS in the string
+ * STR and print the color if SET_COLOR is set to 1 (in which case NULL is
+ * returned); otherwise, just return a pointer to the corresponding color.
+ * This function is used to colorize input, history entries, and accepted
+ * suggestions */
 char *
 rl_highlight(char *str, const size_t pos, const int flag)
 {
 	char *cl = (char *)NULL;
-	/* PREV is -1 when there is no previous char (STR[POS] is the first) */
-	char prev = pos ? str[pos - 1] : 0;
+	/* PREV is 0 when there is no previous char (STR[POS] is the first one) */
+	char prev = pos > 0 ? str[pos - 1] : 0;
 	char c = *(str + pos);
 
+	if (wrong_cmd == 1 && cur_color == wp_c && rl_end == 0) {
+		fputs(tx_c, stdout); fflush(stdout);
+		rl_redisplay();
+	}
+
 	if ((rl_end == 0 && c == BS) || prev == '\\') {
 		if (prev == '\\')
 			goto END;
@@ -82,15 +86,21 @@ rl_highlight(char *str, const size_t pos, const int flag)
 		goto END;
 
 	char *sp = strchr(rl_line_buffer, ' ');
-	if (cur_color == hw_c && !sp)
+	if (cur_color == wp_c && !sp)
 		goto END;
 
-	if (!sp)
-		wrong_cmd_line = 0;
-
-	if (cur_color != hq_c && c >= '0' && c <= '9') {
-		if (prev == ' ' || cur_color == hn_c || rl_end == 1) {
-			cl = hn_c;
+/*	if (*rl_line_buffer != ';' && *rl_line_buffer != ':'
+	&& cur_color != hq_c && c >= '0' && c <= '9') {
+		if (prev == ' ' || prev == 0 || cur_color == hn_c || rl_end == 1) {
+			char *a = strchr(str + pos, ' ');
+			if (a) {
+				*a = '\0';
+				if (is_number(str + pos))
+					cl = hn_c;
+				*a = ' ';
+			} else {
+				cl = hn_c;
+			}
 			goto END;
 		} else {
 			char cc = c;
@@ -102,75 +112,95 @@ rl_highlight(char *str, const size_t pos, const int flag)
 				goto END;
 			}
 		}
-	}
+	} */
 
-	size_t qn[2] = {0};
-	size_t m = 0;
-	for (; m < (size_t)rl_point; m++) {
-		if (rl_line_buffer[m] == '\'') {
-			if (qn[_DOUBLE] == 1 || (m && rl_line_buffer[m - 1] == '\\'))
+	size_t quote[2] = {0};
+	size_t i;
+	for (i = 0; i < (size_t)rl_point; i++) {
+		if (rl_line_buffer[i] == '\'') {
+			if (quote[_DOUBLE] == 1
+			|| (i > 0 && rl_line_buffer[i - 1] == '\\'))
 				continue;
-			qn[_SINGLE]++;
-			if (qn[_SINGLE] > 2)
-				qn[_SINGLE] = 1;
+			quote[_SINGLE]++;
+			if (quote[_SINGLE] > 2)
+				quote[_SINGLE] = 1;
 		} else {
-			if (rl_line_buffer[m] == '"') {
-				if (qn[_SINGLE] == 1 || (m && rl_line_buffer[m - 1] == '\\'))
+			if (rl_line_buffer[i] == '"') {
+				if (quote[_SINGLE] == 1
+				|| (i > 0 && rl_line_buffer[i - 1] == '\\'))
 					continue;
-				qn[_DOUBLE]++;
-				if (qn[_DOUBLE] > 2)
-					qn[_DOUBLE] = 1;
+				quote[_DOUBLE]++;
+				if (quote[_DOUBLE] > 2)
+					quote[_DOUBLE] = 1;
 			}
 		}
 	}
 
 	if (prev != 0) {
-		switch(prev) {
+		switch (prev) {
 		case ')': /* fallthrough */
 		case ']': /* fallthrough */
 		case '}': cl = tx_c; break;
 		case '\'':
-			if (cur_color == hq_c && qn[_SINGLE] == 2)
+			if (cur_color == hq_c && quote[_SINGLE] == 2)
 				cl = tx_c;
 			break;
 		case '"':
-			if (cur_color == hq_c && qn[_DOUBLE] == 2)
+			if (cur_color == hq_c && quote[_DOUBLE] == 2)
 				cl = tx_c;
 			break;
 		default: break;
 		}
 	}
 
-	switch(c) {
+	switch (c) {
+	case '0': /* fallthrough */
+	case '1': /* fallthrough */
+	case '2': /* fallthrough */
+	case '3': /* fallthrough */
+	case '4': /* fallthrough */
+	case '5': /* fallthrough */
+	case '6': /* fallthrough */
+	case '7': /* fallthrough */
+	case '8': /* fallthrough */
+	case '9':
+		if (cur_color != hq_c)
+			cl = hn_c;
+		break;
 	case ' ':
 		if (cur_color != hq_c && cur_color != hc_c)
 			cl = tx_c;
 		break;
-	case '/': cl = (cur_color != hq_c) ? hd_c : (char *)NULL; break;
+	case '/': cl = (cur_color != hq_c) ? hd_c : cl; break;
 	case '\'': /* fallthrough */
 	case '"': cl = hq_c; break;
 	case '\\': /* fallthrough */
 	case ENTER: cl = tx_c; break;
 	case '~': /* fallthrough */
-	case '*': cl = (cur_color != hq_c) ? he_c : (char *)NULL; break;
+	case '*': cl = (cur_color != hq_c) ? he_c : cl; break;
 	case '=': /* fallthrough */
 	case '(': /* fallthrough */
 	case ')': /* fallthrough */
 	case '[': /* fallthrough */
 	case ']': /* fallthrough */
 	case '{': /* fallthrough */
-	case '}': cl = (cur_color != hq_c) ? hb_c : (char *)NULL; break;
+	case '}': cl = (cur_color != hq_c) ? hb_c : cl; break;
 	case '|': /* fallthrough */
 	case '&': /* fallthrough */
-	case ';': cl = (cur_color != hq_c) ? hs_c : (char *)NULL; break;
+	case ';': cl = (cur_color != hq_c) ? hs_c : cl; break;
 	case '<': /* fallthrough */
-	case '>': cl = (cur_color != hq_c) ? hr_c : (char *)NULL; break;
-	case '$': cl = (cur_color != hq_c) ? hv_c : (char *)NULL; break;
+	case '>': cl = (cur_color != hq_c) ? hr_c : cl; break;
+	case '$': cl = (cur_color != hq_c) ? hv_c : cl; break;
 	case '-':
 		if (prev == ' ' || prev == 0)
 			cl = (cur_color != hq_c) ? hp_c : (char *)NULL;
 		break;
-	case '#': cl = (cur_color != hq_c) ? hc_c : (char *)NULL; break;
+	case '#':
+		if (prev == ' ' || prev == 0)
+			cl = (cur_color != hq_c) ? hc_c : (char *)NULL;
+		else
+			cl = tx_c;
+		break;
 	default:
 		if (cur_color != hq_c && cur_color != hc_c
 		&& cur_color != hv_c && cur_color != hp_c)
@@ -179,9 +209,9 @@ rl_highlight(char *str, const size_t pos, const int flag)
 	}
 
 	if (cur_color == hq_c) {
-		if (qn[_SINGLE] == 1)
+		if (quote[_SINGLE] == 1)
 			cl = (char *)NULL;
-		else if (qn[_DOUBLE] == 1)
+		else if (quote[_DOUBLE] == 1)
 			cl = (char *)NULL;
 	}
 
@@ -204,22 +234,22 @@ void
 recolorize_line(void)
 {
 	/* Hide the cursor to minimize flickering */
-	fputs("\x1b[?25l", stdout);
+	HIDE_CURSOR;
 
 	/* Set text color to default */
-	if (cur_color != tx_c && cur_color != hw_c && cur_color != hn_c) {
+	if (cur_color != tx_c && cur_color != wp_c && cur_color != hn_c) {
 		cur_color = tx_c;
 		fputs(tx_c, stdout);
 	}
 
-	int bk = rl_point;
-	if (rl_point && rl_point != rl_end)
+	int bk_point = rl_point;
+	if (rl_point > 0 && rl_point != rl_end)
 		rl_point--;
 
 	/* Get the current color up to the current cursor position */
-	size_t i = 0;
+	size_t i;
 	char *cl = (char *)NULL;
-	for (; i < (size_t)rl_point; i++) {
+	for (i = 0; i < (size_t)rl_point; i++) {
 		cl = rl_highlight(rl_line_buffer, i, INFORM_COLOR);
 		if (cl)
 			cur_color = cl;
@@ -229,31 +259,36 @@ recolorize_line(void)
 		fputs(cl, stdout);
 
 	if (rl_point == 0 && rl_end == 0) {
-		fputs("\x1b[?25h", stdout);
+		UNHIDE_CURSOR;
 		return;
 	}
 
-	int point = rl_point;
-	int copy_start = point ? point - 1 : 0;
-	int start = point;
-	char *ss = rl_copy_text(copy_start, rl_end);
+	int end_bk = rl_end;
+	int start = rl_point > 0 ? rl_point - 1 : 0;
+	char *ss = rl_copy_text(start, rl_end);
 	rl_delete_text(start, rl_end);
 	rl_point = rl_end = start;
-	/* Loop through each char from cursor position onward and colorize it */
-	i = point ? 1 : 0;
+
+	if (start == 0 && end_bk > 1)
+		/* First char of a non-empty recolored line (recovering from wrong cmd) */
+		rl_redisplay();
+
+	i = 0;
+
 	size_t l = 0;
 
 	if (!ss || !*ss)
 		goto EXIT;
 
+	// Loop through each char from cursor position onward and colorize it
 	char t[PATH_MAX];
 	for (;ss[i]; i++) {
 		rl_highlight(ss, i, SET_COLOR);
 		/* Redisplay the current char with the appropriate color */
-		if (ss[i] < 0) {
+		if ((signed char)ss[i] < 0) {
 			t[l] = ss[i];
 			l++;
-			if (ss[i + 1] >= 0) {
+			if ((signed char)ss[i + 1] >= 0) {
 				t[l] = '\0';
 				l = 0;
 				rl_insert_text(t);
@@ -268,8 +303,7 @@ recolorize_line(void)
 	}
 
 EXIT:
-	/* Unhide the cursor */
-	fputs("\x1b[?25h", stdout);
 	free(ss);
-	rl_point = bk;
+	rl_point = bk_point;
+	UNHIDE_CURSOR;
 }
