@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,7 +25,11 @@
 #ifndef HIGHLIGHT_H
 #define HIGHLIGHT_H
 
-char *rl_highlight(const char *str, const size_t pos, const int flag);
+__BEGIN_DECLS
+
+char *rl_highlight(const char *, const size_t, const int);
 void recolorize_line(void);
 
+__END_DECLS
+
 #endif /* HIGHLIGHT_H */
