@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,14 +25,33 @@
 #ifndef HISTORY_H
 #define HISTORY_H
 
-int save_dirhist(void);
-void add_to_cmdhist(const char *cmd);
-int record_cmd(char *input);
-int get_history(void);
-int history_function(char **comm);
-int run_history_cmd(const char *cmd);
-void add_to_dirhist(const char *dir_path);
-int log_function(char **comm);
-void log_msg(char *_msg, int print);
+/* Macros for history_function() */
+#define NO_HIST_TIME 0
+#define HIST_TIME    1
+
+/* Macros for clear_logs() and print_logs() */
+#define MSG_LOGS 1
+#define CMD_LOGS 0
+
+__BEGIN_DECLS
+
+void add_to_cmdhist(char *);
+void add_to_dirhist(const char *);
+int  clear_logs(const int);
+int  get_history(void);
+int  history_function(char **);
+int  log_cmd(void);
+void log_msg(char *, const int, const int, const int);
+int  print_logs(const int);
+int  record_cmd(char *);
+int  run_history_cmd(const char *);
+
+////// TEMPORAL CODE
+#ifndef _NO_SPLIT_LOG
+void split_old_log_file(void);
+#endif /* !_NO_SPLIT_LOG */
+////////////////////
+
+__END_DECLS
 
 #endif /* HISTORY_H */
