@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,10 +25,12 @@
 #ifndef JUMP_H
 #define JUMP_H
 
-int add_to_jumpdb(const char *dir);
+__BEGIN_DECLS
+
+int  add_to_jumpdb(char *);
 void save_jumpdb(void);
-int dirjump(char **args, int mode);
-int run_autojump(char **cmd);
-/* int edit_jumpdb(void); */
+int  dirjump(char **, int);
+
+__END_DECLS
 
 #endif /* JUMP_H */
