@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,7 +25,17 @@
 #ifndef LISTING_H
 #define LISTING_H
 
+__BEGIN_DECLS
+
 void free_dirlist(void);
-int list_dir(void);
+int  list_dir(void);
+void reload_dirlist(void);
+void refresh_screen(void);
+
+#ifndef _NO_ICONS
+void init_icons_hashes(void);
+#endif /* !_NO_ICONS */
+
+__END_DECLS
 
 #endif /* LISTING_H */
