@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -28,18 +28,30 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
-#include <sys/stat.h>
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
+
+#if defined(__linux__) || defined(__CYGWIN__)
+# include <mntent.h>
+# define HAVE_PROC_MOUNTS
+# define DISK_LABELS_PATH "/dev/disk/by-label"
+# ifndef _PATH_MOUNTED
+#  define _PATH_MOUNTED "/etc/mtab"
+# endif
+#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) \
+|| defined(__DragonFly__)
 # include <sys/mount.h>
 # include <sys/sysctl.h>
 #elif defined(__APPLE__)
 # include <sys/param.h>
 # include <sys/ucred.h>
 # include <sys/mount.h>
-#endif
-#include <dirent.h>
+#elif defined(__sun)
+# include <sys/mnttab.h> /* getmntent() */
+# ifndef MNTTAB
+#  define MNTTAB "/etc/mnttab"
+# endif /* MNTTAB */
+#endif /* __linux__ || __CYGWIN__ */
 
-//#include <libudev.h>
+#include <dirent.h>
 
 #include "aux.h"
 #include "readline.h"
@@ -47,90 +59,24 @@
 #include "exec.h"
 #include "listing.h"
 #include "jump.h"
-#include "checks.h"
+#include "misc.h"
 #include "history.h"
 
-#ifdef __linux__
-# define DISK_LABELS_PATH "/dev/disk/by-label"
-#endif
+#ifdef __sun
+# include "checks.h" /* check_file_access() */
+#endif /* __sun */
 
 /* Information about devices */
 struct mnt_t {
-	char *mnt; /* Mountpoint */
-	char *dev; /* Device name (ex: /dev/sda1) */
+	char *mnt;   /* Mountpoint */
+	char *dev;   /* Device name (ex: /dev/sda1) */
 	char *label; /* Device label */
 };
 
-struct mnt_t *media = (struct mnt_t *)NULL;
-size_t mp_n = 0;
+static struct mnt_t *media = (struct mnt_t *)NULL;
+static size_t mp_n = 0;
 
-/*
-#ifdef __linux__
-static struct udev_device*
-get_child(struct udev* udev, struct udev_device* parent, const char* subsystem)
-{
-	struct udev_device* child = NULL;
-	struct udev_enumerate *enumerate = udev_enumerate_new(udev);
-
-	udev_enumerate_add_match_parent(enumerate, parent);
-	udev_enumerate_add_match_subsystem(enumerate, subsystem);
-	udev_enumerate_scan_devices(enumerate);
-
-	struct udev_list_entry *devices = udev_enumerate_get_list_entry(enumerate);
-	struct udev_list_entry *entry;
-
-	udev_list_entry_foreach(entry, devices) {
-		const char *path = udev_list_entry_get_name(entry);
-		child = udev_device_new_from_syspath(udev, path);
-		break;
-	}
-
-	udev_enumerate_unref(enumerate);
-	return child;
-}
-
-// Return an array of block devices partitions
-static char **
-get_block_devices(void)
-{
-	struct udev* udev = udev_new();
-	struct udev_enumerate* enumerate = udev_enumerate_new(udev);
-
-	udev_enumerate_add_match_property(enumerate, "DEVTYPE", "partition");
-	udev_enumerate_scan_devices(enumerate);
-
-	struct udev_list_entry *devices = udev_enumerate_get_list_entry(enumerate);
-	struct udev_list_entry *entry;
-
-	char **mps = (char **)NULL;
-	size_t n = 0;
-
-	udev_list_entry_foreach(entry, devices) {
-	const char* path = udev_list_entry_get_name(entry);
-	struct udev_device* scsi = udev_device_new_from_syspath(udev, path);
-
-	struct udev_device* block = get_child(udev, scsi, "block");
-
-	const char *dev = udev_device_get_devnode(block);
-	mps = (char **)xrealloc(mps, (n + 2) * sizeof(char *));
-	mps[n++] = savestring(dev, strlen(dev));
-
-	if (block)
-		udev_device_unref(block);
-
-	udev_device_unref(scsi);
-	}
-	mps[n] = (char *)NULL;
-
-	udev_enumerate_unref(enumerate);
-	udev_unref(udev);
-
-	return mps;
-}
-#endif // __linux__
-*/
-
-#ifdef __linux__
+#ifdef HAVE_PROC_MOUNTS
 static char **
 get_block_devices(void)
 {
@@ -140,20 +86,21 @@ get_block_devices(void)
 		return (char **)NULL;
 
 	char **bd = (char **)NULL;
-	size_t i = 0, n = 0;
-	for (; (int)i < block_n; i++) {
-#ifndef _DIRENT_HAVE_D_TYPE
+	size_t i, n = 0;
+
+	for (i = 0; (int)i < block_n; i++) {
+# ifndef _DIRENT_HAVE_D_TYPE
 		char bpath[PATH_MAX];
-		snprintf(bpath, PATH_MAX, "/dev/%s", blockdev[i]->d_name);
+		snprintf(bpath, sizeof(bpath), "/dev/%s", blockdev[i]->d_name);
 		struct stat a;
 		if (stat(bpath, &a) == -1) {
 			free(blockdev[i]);
 			continue;
 		}
 		if (!S_ISBLK(a.st_mode)) {
-#else
+# else
 		if (blockdev[i]->d_type != DT_BLK) {
-#endif /* !_DIRENT_HAVE_D_TYPE */
+# endif /* !_DIRENT_HAVE_D_TYPE */
 			free(blockdev[i]);
 			continue;
 		}
@@ -169,14 +116,14 @@ get_block_devices(void)
 
 		/* Get only partition names, normally ending with a number */
 		size_t blen = strlen(name);
-		if (name[blen - 1] < '1' || name[blen - 1] > '9') {
+		if (blen > 0 && (name[blen - 1] < '1' || name[blen - 1] > '9')) {
 			free(blockdev[i]);
 			continue;
 		}
 
 		bd = (char **)xrealloc(bd, (n + 2) * sizeof(char *));
 		bd[n] = (char *)xnmalloc(blen + 6, sizeof(char *));
-		sprintf(bd[n], "/dev/%s", name);
+		snprintf(bd[n], blen + 6, "/dev/%s", name);
 		n++;
 		bd[n] = (char *)NULL;
 		free(blockdev[i]);
@@ -187,16 +134,16 @@ get_block_devices(void)
 }
 
 static int
-unmount_dev(size_t i, const int n)
+unmount_dev(const size_t i, const int n)
 {
 	if (xargs.mount_cmd == UNSET) {
-		fprintf(stderr, _("%s: No mount application found. Install either "
+		xerror(_("%s: No mount application found. Install either "
 			"udevil or udisks2\n"), PROGRAM_NAME);
 		return EXIT_FAILURE;
 	}
 
 	if ((unsigned int)n + (unsigned int)1 < (unsigned int)1 || n + 1 > (int)i) {
-		fprintf(stderr, _("%s: %d: Invalid ELN\n"), PROGRAM_NAME, n + 1);
+		xerror(_("%s: %d: Invalid ELN\n"), PROGRAM_NAME, n + 1);
 		return EXIT_FAILURE;
 	}
 
@@ -213,12 +160,13 @@ unmount_dev(size_t i, const int n)
 	}
 
 	char *cmd[] = {xargs.mount_cmd == MNT_UDISKS2 ? "udisksctl" : "udevil",
-					"unmount", "-b", media[n].dev, NULL};
-	if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+			"unmount", "-b", media[n].dev, NULL};
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 		exit_status = EXIT_FAILURE;
 
 	if (exit_status != EXIT_FAILURE && xargs.mount_cmd == MNT_UDEVIL)
 		printf(_("%s: Unmounted %s\n"), PROGRAM_NAME, media[n].dev);
+
 	return exit_status;
 }
 
@@ -232,8 +180,9 @@ get_dev_label(void)
 		return (char *)NULL;
 
 	char *label = (char *)NULL;
-	int i = 0;
-	for (; i < ln; i++) {
+	int i;
+
+	for (i = 0; i < ln; i++) {
 		if (label) {
 			free(labels[i]);
 			continue;
@@ -241,7 +190,7 @@ get_dev_label(void)
 
 		char *name = labels[i]->d_name;
 		char lpath[PATH_MAX];
-		snprintf(lpath, PATH_MAX, "%s/%s", DISK_LABELS_PATH, name);
+		snprintf(lpath, sizeof(lpath), "%s/%s", DISK_LABELS_PATH, name);
 		char *rpath = realpath(lpath, NULL);
 		if (!rpath) {
 			free(labels[i]);
@@ -257,7 +206,7 @@ get_dev_label(void)
 				char pp = 0;
 				pp = (char)(from_hex(*(p + 2)) << 4 | from_hex(*(p + 3)));
 				*p = pp;
-				strcpy(p + 1, p + 4);
+				xstrsncpy(p + 1, p + 4, strlen(p + 4) + 1);
 			}
 			label = savestring(name, strlen(name));
 		}
@@ -272,24 +221,24 @@ get_dev_label(void)
 static void
 list_unmounted_devs(void)
 {
-	size_t k = mp_n;
-	char **unm_devs = (char **)NULL;
-	unm_devs = get_block_devices();
-
+	int i;
+	size_t mp_n_bk = mp_n;
+	char **unm_devs = get_block_devices();
 	if (!unm_devs)
 		return;
 
 	printf(_("\n%sUnmounted devices%s\n\n"), BOLD, df_c);
-	int i = 0;
-	for (; unm_devs[i]; i++) {
+
+	for (i = 0; unm_devs[i]; i++) {
 		int skip = 0;
-		size_t j = 0;
-		// Skip already mounted devices
-		for (; j < k; j++) {
+		size_t j;
+		/* Skip already mounted devices */
+		for (j = 0; j < mp_n_bk; j++) {
 			if (strcmp(media[j].dev, unm_devs[i]) == 0)
 				skip = 1;
 		}
-		if (skip) {
+
+		if (skip == 1) {
 			free(unm_devs[i]);
 			continue;
 		}
@@ -303,9 +252,10 @@ list_unmounted_devs(void)
 
 		if (media[mp_n].label)
 			printf("%s%zu %s%s [%s%s%s]\n", el_c, mp_n + 1, df_c,
-					media[mp_n].dev, mi_c, media[mp_n].label, df_c);
+				media[mp_n].dev, mi_c, media[mp_n].label, df_c);
 		else
 			printf("%s%zu %s%s\n", el_c, mp_n + 1, df_c, media[mp_n].dev);
+
 		mp_n++;
 		free(unm_devs[i]);
 	}
@@ -315,83 +265,42 @@ list_unmounted_devs(void)
 	media[mp_n].mnt = (char *)NULL;
 	media[mp_n].label = (char *)NULL;
 }
-#endif /* __linux__ */
 
 static int
-list_mounted_devs(int mode)
+list_mounted_devs(const int mode)
 {
-	FILE *mp_fp = fopen("/proc/mounts", "r");
-	if (!mp_fp) {
-		fprintf(stderr, "%s: mp: fopen: /proc/mounts: %s\n",
-				PROGRAM_NAME, strerror(errno));
+	struct mntent *ent;
+
+	FILE *fp = setmntent(_PATH_MOUNTED, "r");
+	if (!fp) {
+		xerror("mp: setmntent: %s: %s\n", _PATH_MOUNTED, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
-	size_t line_size = 0;
-	char *line = (char *)NULL;
+	while ((ent = getmntent(fp)) != NULL) {
+		if (strncmp(ent->mnt_fsname, "/dev/", 5) != 0)
+			continue;
 
-	while (getline(&line, &line_size, mp_fp) > 0) {
-		/* Do not list all mountpoints, but only those corresponding
-		 * to a block device (/dev) */
-		if (strncmp(line, "/dev/", 5) == 0) {
-			char *str = (char *)NULL;
-			size_t counter = 0;
-
-			/* use strtok() to split LINE into tokens using space as
-			 * IFS */
-			str = strtok(line, " ");
-			size_t dev_len = strlen(str);
-
-			media = (struct mnt_t *)xrealloc(media,
-				    (mp_n + 2) * sizeof(struct mnt_t));
-			media[mp_n].dev = savestring(str, dev_len);
-			media[mp_n].label = (char *)NULL;
-			/* Print only the first two fileds of each /proc/mounts
-			 * line */
-			while (str && counter < 2) {
-				if (counter == 1) { /* 1 == second field */
-					/* /proc/mounts encode special chars as octal.
-					 * Let's decode it */
-					char *p = strchr(str, '\\');
-					if (p && *(p + 1)) {
-						char *q = p;
-						p++;
-						char pp = 0;
-						p += 3;
-						pp = *p;
-						*p = '\0';
-						int d = read_octal(q + 1);
-						*p = pp;
-						*q = (char)d;
-						strcpy(q + 1, q + 4);
-					}
-
-					if (mode == MEDIA_LIST) {
-						printf("%s%zu%s %s%s%s [%s]\n", el_c, mp_n + 1,
-							df_c, (access(str, R_OK | X_OK) == 0) ? di_c : nd_c,
-							str, df_c, media[mp_n].dev);
-					} else {
-						printf("%s%zu%s %s [%s%s%s]\n", el_c, mp_n + 1,
-							df_c, media[mp_n].dev,
-							(access(str, R_OK | X_OK) == 0) ? di_c
-							: nd_c, str, df_c);
-					}
-
-					/* Store the second field (mountpoint) into an
-					 * array */
-					media[mp_n].mnt = savestring(str, strlen(str));
-					mp_n++;
-				}
-
-				str = strtok(NULL, " ,");
-				counter++;
-			}
+		if (mode == MEDIA_LIST) {
+			printf("%s%zu%s %s%s%s [%s]\n", el_c, mp_n + 1,
+				df_c, (access(ent->mnt_dir, R_OK | X_OK) == 0) ? di_c : nd_c,
+				ent->mnt_dir, df_c, ent->mnt_fsname);
+		} else {
+			printf("%s%zu%s %s [%s%s%s]\n", el_c, mp_n + 1,
+				df_c, ent->mnt_fsname,
+				(access(ent->mnt_dir, R_OK | X_OK) == 0) ? di_c
+				: nd_c, ent->mnt_dir, df_c);
 		}
+
+		media = (struct mnt_t *)xrealloc(media, (mp_n + 2) * sizeof(struct mnt_t));
+		media[mp_n].mnt = savestring(ent->mnt_dir, strlen(ent->mnt_dir));
+		media[mp_n].dev = savestring(ent->mnt_fsname, strlen(ent->mnt_fsname));
+		media[mp_n].label = (char *)NULL;
+
+		mp_n++;
 	}
 
-	free(line);
-	line = (char *)NULL;
-	fclose(mp_fp);
+	endmntent(fp);
 
 	media[mp_n].dev = (char *)NULL;
 	media[mp_n].mnt = (char *)NULL;
@@ -400,18 +309,19 @@ list_mounted_devs(int mode)
 	return EXIT_SUCCESS;
 }
 
-/* Mount device and store mountpoint */
+/* Mount device and store mountpoint (at media[N]). */
 static int
-mount_dev(int n)
+mount_dev(const int n)
 {
 	if (xargs.mount_cmd == UNSET) {
-		fprintf(stderr, _("%s: No mount application found. Install either "
+		xerror(_("%s: No mount application found. Install either "
 			"udevil or udisks2\n"), PROGRAM_NAME);
 		return EXIT_FAILURE;
 	}
 
 	char file[PATH_MAX];
-	snprintf(file, PATH_MAX, "%s/clifm.XXXXXX", P_tmpdir);
+	snprintf(file, sizeof(file), "%s/%s", xargs.stealth_mode == 1
+		? P_tmpdir : tmp_dir, TMP_FILENAME);
 
 	int fd = mkstemp(file);
 	if (fd == -1)
@@ -423,16 +333,16 @@ mount_dev(int n)
 
 	if (xargs.mount_cmd == MNT_UDISKS2) {
 		char *cmd[] = {"udisksctl", "mount", "-b", media[n].dev, NULL};
-		launch_execve(cmd, FOREGROUND, E_NOFLAG);
+		launch_execv(cmd, FOREGROUND, E_NOFLAG);
 	} else {
 		char *cmd[] = {"udevil", "mount", media[n].dev, NULL};
-		launch_execve(cmd, FOREGROUND, E_NOFLAG);
+		launch_execv(cmd, FOREGROUND, E_NOFLAG);
 	}
 
 	dup2(stdout_bk, STDOUT_FILENO); /* Restore original stdout */
 	close(stdout_bk);
 
-	FILE *fp = open_fstream_r(file, &fd);
+	FILE *fp = open_fread(file, &fd);
 	if (!fp) {
 		unlink(file);
 		return EXIT_FAILURE;
@@ -441,30 +351,34 @@ mount_dev(int n)
 	char out_line[PATH_MAX];
 	if (fgets(out_line, (int)sizeof(out_line), fp) == NULL) {
 		/* Error is printed by the mount command itself */
-		close_fstream(fp, fd);
+		fclose(fp);
 		unlink(file);
 		return EXIT_FAILURE;
 	}
 
-	close_fstream(fp, fd);
+	fclose(fp);
 	unlink(file);
 
-	/* Recover the mountpoint used by the mounting command */
+	/* Recover the mountpoint used by the mounting command. */
 	char *p = strstr(out_line, " at ");
-	if (!p || *(p + 4) != '/') {
-		fprintf(stderr, _("%s: Error retrieving mountpoint\n"), PROGRAM_NAME);
+	if (!p || !p[1] || !p[2] || !p[3] || p[4] != '/') {
+		xerror(_("%s: Error retrieving mountpoint\n"), PROGRAM_NAME);
 		return EXIT_FAILURE;
 	}
 	p += 4;
 
-	size_t plen = strlen(p);
-	if (p[plen - 1] == '\n')
-		p[plen - 1] = '\0';
+	/* Using strnlen() here avoids a Redhat hardened compilation warning. */
+	size_t plen = strnlen(p, sizeof(out_line) - 4);
+	if (plen > 0 && p[plen - 1] == '\n') {
+		plen--;
+		p[plen] = '\0';
+	}
 
-	media[n].mnt = savestring(p, strlen(p));
+	media[n].mnt = savestring(p, plen);
 
 	return EXIT_SUCCESS;
 }
+#endif /* HAVE_PROC_MOUNTS */
 
 static void
 free_media(void)
@@ -481,165 +395,105 @@ free_media(void)
 
 /* Get device information via external application */
 static int
-print_dev_info(int n)
+print_dev_info(const int n)
 {
-	if (!media[n].dev)
+	if (!media[n].dev || xargs.mount_cmd == UNSET)
 		return EXIT_FAILURE;
 
 	int exit_status = EXIT_SUCCESS;
 	if (xargs.mount_cmd == MNT_UDEVIL) {
 		char *cmd[] = {"udevil", "info", media[n].dev, NULL};
-		if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 			exit_status = EXIT_FAILURE;
 	} else {
 		char *cmd[] = {"udisksctl", "info", "-b", media[n].dev, NULL};
-		if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 			exit_status = EXIT_FAILURE;
 	}
 
 	return exit_status;
 }
 
-/* If MODE is MEDIA_MOUNT (used by the 'media' command) list mounted and
- * unmounted devices allowing the user to mount or unmount any of them.
- * If MODE is rather MEDIA_LIST (used by the 'mp' command), just list
- * available mountpoints and allow the user to cd into the selected one */
-int
-media_menu(int mode)
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) \
+|| defined(__DragonFly__) || defined(__APPLE__)
+static size_t
+# ifdef __NetBSD__
+list_mountpoints_bsd(struct statvfs *fslist)
+# else
+list_mountpoints_bsd(struct statfs *fslist)
+# endif /* __NetBSD__ */
 {
-#if defined(__HAIKU__)
-	fprintf(stderr, _("%s: %s: This feature is not available on Haiku\n"),
-			PROGRAM_NAME, mode == MEDIA_LIST ? _("Mountpoints") : _("Media"));
-	return EXIT_FAILURE;
-#endif
+	size_t i, j = 0;
 
-#ifndef __linux__
-	if (mode == MEDIA_MOUNT) {
-		fprintf(stderr, _("%s: media: Function only available on Linux "
-				"systems\n"), PROGRAM_NAME);
-		return EXIT_FAILURE;
-	}
-#endif
-
-	if (mode == MEDIA_MOUNT && xargs.mount_cmd == UNSET) {
-		fprintf(stderr, _("%s: No mount command found. Install either "
-				"udevil or udisks2\n"), PROGRAM_NAME);
-		return EXIT_FAILURE;
-	}
-
-#ifdef __linux__
-	printf("%s%s%s\n\n", BOLD, mode == MEDIA_LIST ? _("Mountpoints")
-			: _("Mounted devices"), df_c);
-#else
-	printf(_("%sMountpoints%s\n\n"), BOLD, df_c);
-#endif
-
-	media = (struct mnt_t *)xnmalloc(1, sizeof(struct mnt_t));
-	mp_n = 0;
-	int exit_status = EXIT_SUCCESS;
-
-#ifdef __linux__
-	if (list_mounted_devs(mode) == EXIT_FAILURE) {
-		free(media);
-		media = (struct mnt_t *)NULL;
-		return EXIT_FAILURE;
-	}
-
-	size_t k = mp_n;
-
-	if (mode == MEDIA_MOUNT)
-		list_unmounted_devs();
-
-#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
-	struct statfs *fslist;
-	mp_n = (size_t)getmntinfo(&fslist, MNT_NOWAIT);
-#elif defined(__NetBSD__)
-	struct statvfs *fslist;
-	mp_n = (size_t)getmntinfo(&fslist, MNT_NOWAIT);
-#endif /* __linux__ */
-
-	/* This should never happen: There should always be a mountpoint,
-	 * at least "/" */
-	// cppcheck-suppress knownConditionTrueFalse
-	if (mp_n == 0) {
-#ifdef __linux__
-		printf(_("%s: There are no available %s\n"), mode == MEDIA_LIST ? "mp"
-			: "media", mode == MEDIA_LIST ? _("mountpoints") : _("devices"));
-#else
-		fputs(_("mp: There are no available mountpoints\n"), stdout);
-#endif
-		return EXIT_SUCCESS;
-	}
-
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) \
-|| defined(__APPLE__)
-	int i, j;
-	for (i = j = 0; i < (int)mp_n; i++) {
+	for (i = 0; i < mp_n; i++) {
 		/* Do not list all mountpoints, but only those corresponding
 		 * to a block device (/dev) */
-		if (strncmp(fslist[i].f_mntfromname, "/dev/", 5) == 0) {
-			printf("%s%d%s %s%s%s (%s)\n", el_c, j + 1, df_c,
-			    (access(fslist[i].f_mntonname, R_OK | X_OK) == 0)
-			    ? di_c : nd_c, fslist[i].f_mntonname,
-			    df_c, fslist[i].f_mntfromname);
-			/* Store the mountpoint into the mounpoints struct */
-			media = (struct mnt_t *)xrealloc(media,
-				    (j + 2) * sizeof(struct mnt_t));
-			media[j].mnt = savestring(fslist[i].f_mntonname,
-					strlen(fslist[i].f_mntonname));
-			media[j].label = (char *)NULL;
-			media[j].dev = (char *)NULL;
-			j++;
-		}
+		if (strncmp(fslist[i].f_mntfromname, "/dev/", 5) != 0)
+			continue;
+
+		printf("%s%zu%s %s%s%s (%s)\n", el_c, j + 1, df_c,
+		    (access(fslist[i].f_mntonname, R_OK | X_OK) == 0)
+		    ? di_c : nd_c, fslist[i].f_mntonname,
+		    df_c, fslist[i].f_mntfromname);
+
+		/* Store the mountpoint into the mounpoints struct */
+		media = (struct mnt_t *)xrealloc(media, (j + 2) * sizeof(struct mnt_t));
+		media[j].mnt = savestring(fslist[i].f_mntonname,
+			strlen(fslist[i].f_mntonname));
+		media[j].label = (char *)NULL;
+		media[j].dev = (char *)NULL;
+		j++;
 	}
 
 	media[j].dev = (char *)NULL;
 	media[j].mnt = (char *)NULL;
 	media[j].label = (char *)NULL;
-	/* Update filesystem counter as it would be used to free() the
-	 * mountpoints entries later (below) */
-	mp_n = (size_t)j;
-#endif
 
-	putchar('\n');
+	return j;
+}
+#endif /* BSD || APPLE */
+
+static int
+get_mnt_input(const int mode, int *info)
+{
 	int n = -1;
-	/* Ask the user and mount/unmount or chdir into the selected
-	 * device/mountpoint */
 	puts(_("Enter 'q' to quit"));
-	puts(_("Enter 'iELN' for device information. Ex: i4"));
-
-	int info = 0;
+	if (xargs.mount_cmd != UNSET)
+		puts(_("Enter 'iELN' for device information. Ex: i4"));
 
 	char *input = (char *)NULL;
 	while (!input) {
-#ifdef __linux__
+#ifdef HAVE_PROC_MOUNTS
 		if (mode == MEDIA_LIST)
 			input = rl_no_hist(_("Choose a mountpoint: "));
 		else
 			input = rl_no_hist(_("Choose a mountpoint/device: "));
 #else
+		UNUSED(mode);
 		input = rl_no_hist(_("Choose a mountpoint: "));
-#endif
-		if (!input)
-			continue;
-
-		if (!*input) {
+#endif /* HAVE_PROC_MOUNTS */
+		if (!input || !*input) {
 			free(input);
 			input = (char *)NULL;
 			continue;
 		}
 
-		if (*input == 'q' && *(input + 1) == '\0')
-			goto EXIT;
+		if (*input == 'q' && *(input + 1) == '\0') {
+			if (conf.autols == 1)
+				reload_dirlist();
+			free(input);
+			return (-1);
+		}
 
 		char *p = input;
 		if (*p == 'i') {
-			info = 1;
+			*info = 1;
 			++p;
 		}
+
 		int atoi_num = atoi(p);
 		if (atoi_num <= 0 || atoi_num > (int)mp_n) {
-			fprintf(stderr, "%s: %s: Invalid ELN\n", PROGRAM_NAME, input);
+			xerror("%s: %s: Invalid ELN\n", PROGRAM_NAME, input);
 			free(input);
 			input = (char *)NULL;
 			continue;
@@ -648,23 +502,160 @@ media_menu(int mode)
 		n = atoi_num - 1;
 	}
 
+	free(input);
+	return n;
+}
+
+static int
+print_mnt_info(const int n)
+{
+	int exit_status = print_dev_info(n);
+
+	if (exit_status == EXIT_SUCCESS) {
+		printf("\nPress any key to continue... ");
+		xgetchar();
+		putchar('\n');
+	}
+
+	free_media();
+	return exit_status;
+}
+
+#ifdef __sun
+/* Get and list available mountpoints (dirs only).
+ * Returns the number of mounpoints found. */
+static size_t
+xgetmntinfo_sun(void)
+{
+	FILE *fp = fopen(MNTTAB, "r");
+	if (!fp)
+		return 0;
+
+	size_t n = 0;
+	struct mnttab ent;
+
+	while (getmntent(fp, &ent) != -1) {
+		char *mp = ent.mnt_mountp;
+		if (!mp || !*mp)
+			continue;
+
+		struct stat a;
+		if (stat(mp, &a) == -1 || !S_ISDIR(a.st_mode))
+			continue;
+
+		int perm = check_file_access(a.st_mode, a.st_uid, a.st_gid);
+		printf("%s%zu%s %s%s%s [%s]\n", el_c, n + 1, df_c,
+			perm == 1 ? di_c : nd_c, mp, df_c, ent.mnt_special);
+
+		media = (struct mnt_t *)xrealloc(media, (n + 2) * sizeof(struct mnt_t));
+		media[n].mnt = savestring(mp, strlen(mp));
+		media[n].label = (char *)NULL;
+		media[n].dev = (char *)NULL;
+		n++;
+	}
+
+	fclose(fp);
+
+	media[n].mnt = (char *)NULL;
+	media[n].label = (char *)NULL;
+	media[n].dev = (char *)NULL;
+
+	return n;
+}
+#endif /* __sun */
+
+/* If MODE is MEDIA_MOUNT (used by the 'media' command) list mounted and
+ * unmounted devices allowing the user to mount or unmount any of them.
+ * If MODE is rather MEDIA_LIST (used by the 'mp' command), just list
+ * available mountpoints and allow the user to cd into the selected one */
+int
+media_menu(const int mode)
+{
+#if defined(__HAIKU__)
+	xerror(_("%s: This feature is not available on Haiku\n"),
+		mode == MEDIA_LIST ? _("mountpoints") : _("media"));
+	return EXIT_FAILURE;
+#endif /* __HAIKU__ */
+
+#ifndef HAVE_PROC_MOUNTS
+	if (mode == MEDIA_MOUNT) {
+		xerror("%s\n", _("media: Function only available on Linux systems"));
+		return EXIT_FAILURE;
+	}
+#endif /* HAVE_PROC_MOUNTS */
+
+	if (mode == MEDIA_MOUNT && xargs.mount_cmd == UNSET) {
+		xerror("%s\n", _("media: No mount application found. Install either "
+			"udevil or udisks2"));
+		return EXIT_FAILURE;
+	}
+
+#ifdef HAVE_PROC_MOUNTS
+	printf("%s%s%s\n\n", BOLD, mode == MEDIA_LIST ? _("Mountpoints")
+			: _("Mounted devices"), df_c);
+#else
+	printf(_("%sMountpoints%s\n\n"), BOLD, df_c);
+#endif /* HAVE_PROC_MOUNTS */
+
+	media = (struct mnt_t *)xnmalloc(1, sizeof(struct mnt_t));
+	mp_n = 0;
+
+#ifdef HAVE_PROC_MOUNTS
+	if (list_mounted_devs(mode) == EXIT_FAILURE) {
+		free(media);
+		media = (struct mnt_t *)NULL;
+		return EXIT_FAILURE;
+	}
+
+	size_t mp_n_bk = mp_n;
+
+	if (mode == MEDIA_MOUNT)
+		list_unmounted_devs();
+
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__) \
+|| defined(__DragonFly__)
+	struct statfs *fslist;
+	mp_n = (size_t)getmntinfo(&fslist, MNT_NOWAIT);
+#elif defined(__NetBSD__)
+	struct statvfs *fslist;
+	mp_n = (size_t)getmntinfo(&fslist, MNT_NOWAIT);
+#elif defined(__sun)
+	mp_n = xgetmntinfo_sun();
+#endif /* HAVE_PROC_MOUNTS */
+
+	/* This should never happen: There should always be a mountpoint,
+	 * at least "/" */
+	// cppcheck-suppress knownConditionTrueFalse
+	if (mp_n == 0) {
+#ifdef HAVE_PROC_MOUNTS
+		printf(_("%s: There are no available %s\n"), mode == MEDIA_LIST ? "mp"
+			: "media", mode == MEDIA_LIST ? _("mountpoints") : _("devices"));
+#else
+		fputs(_("mp: There are no available mountpoints\n"), stdout);
+#endif /* HAVE_PROC_MOUNTS */
+		return EXIT_SUCCESS;
+	}
+
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) \
+|| defined(__APPLE__) || defined(__DragonFly__)
+	mp_n = list_mountpoints_bsd(fslist);
+#endif /* BSD || APPLE */
+
+	putchar('\n');
+	int info = 0;
+	int n = get_mnt_input(mode, &info);
+
+	int exit_status = EXIT_SUCCESS;
 	if (n == -1)
 		goto EXIT;
 
 	if (info == 1) {
-		exit_status = print_dev_info(n);
-		if (exit_status == EXIT_SUCCESS) {
-			printf("\nPress any key to continue... ");
-			xgetchar();
-			putchar('\n');
-		}
-		free(input);
-		free_media();
+		exit_status = print_mnt_info(n);
 		media_menu(mode);
 		return exit_status;
 	}
 
-#ifdef __linux__
+#ifdef HAVE_PROC_MOUNTS
 	if (mode == MEDIA_MOUNT) {
 		if (!media[n].mnt) {
 			/* The device is unmounted: mount it */
@@ -674,16 +665,16 @@ media_menu(int mode)
 			}
 		} else {
 			/* The device is mounted: unmount it */
-			int ret = unmount_dev(k, n);
+			int ret = unmount_dev(mp_n_bk, n);
 			if (ret == EXIT_FAILURE)
 				exit_status = EXIT_FAILURE;
 			goto EXIT;
 		}
 	}
-#endif /* __linux__ */
+#endif /* HAVE_PROC_MOUNTS */
 
 	if (xchdir(media[n].mnt, SET_TITLE) != EXIT_SUCCESS) {
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, media[n].mnt, strerror(errno));
+		xerror("%s: %s: %s\n", PROGRAM_NAME, media[n].mnt, strerror(errno));
 		exit_status = EXIT_FAILURE;
 		goto EXIT;
 	}
@@ -691,17 +682,13 @@ media_menu(int mode)
 	free(workspaces[cur_ws].path);
 	workspaces[cur_ws].path = savestring(media[n].mnt, strlen(media[n].mnt));
 
-	if (autols) {
-		free_dirlist();
-		if (list_dir() != EXIT_SUCCESS)
-			exit_status = EXIT_FAILURE;
-	}
+	if (conf.autols == 1)
+		reload_dirlist();
 
 	add_to_dirhist(workspaces[cur_ws].path);
 	add_to_jumpdb(workspaces[cur_ws].path);
 
 EXIT:
-	free(input);
 	free_media();
 	return exit_status;
 }
