@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,6 +25,10 @@
 #ifndef MEDIA_H
 #define MEDIA_H
 
-int media_menu(int mode);
+__BEGIN_DECLS
+
+int media_menu(const int);
+
+__END_DECLS
 
 #endif /* MEDIA_H */
