@@ -1,8 +1,8 @@
 /* name_cleaner.c -- functions to clean up file names
  * 
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -21,33 +21,34 @@
  * MA 02110-1301, USA.
 */
 
-/* The algorithm to translate Unicode characters into ASCII is based on
- * https://github.com/dharple/detox */
+/* check_width, unpack_start, and unpack_cont macros, just
+ * as get_utf_8_width and get_uft8_dec_value functions are taken from
+ * https://github.com/dharple/detox/blob/main/src/clean_utf_8.c, licensed
+ * under BSD-3-Clause.
+ * All changes are licensed under GPL-2.0-or-later. */
 
 #ifndef _NO_BLEACH
 
 #include "helpers.h"
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <ctype.h>
 #include <time.h>
 #include <errno.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 
 #include "aux.h"
-#include "exec.h"
 #include "file_operations.h"
 #include "history.h"
-#if defined(__HAIKU__) || defined(__APPLE__)
-# include "listing.h"
-#endif
+#ifdef NO_FS_EVENTS_MONITOR
+# include "listing.h" /* reload_dirlist() */
+#endif /* NO_FS_EVENTS_MONITOR */
+#include "messages.h"
 #include "misc.h"
 #include "cleaner_table.h"
 #include "readline.h"
+#include "selection.h"
 
 #define FUNC_NAME "bleach"
 #define DEFAULT_TRANSLATION  '_'
@@ -75,21 +76,23 @@
 #define UTF_8_ENCODED_2_BYTES_MASK 0xE0
 #define UTF_8_ENCODED_2_BYTES      0xC0
 
+#define BLEACH_TMP_HEADER "# CliFM - Bleach\n\
+# Edit replacement file names as you wish, save, and close the editor.\n\
+# You will be asked for confirmation at exit.\n\n"
+
 #define check_width(chr, size) if ((chr & UTF_8_ENCODED_ ## size ## _BYTES_MASK) == UTF_8_ENCODED_ ## size ## _BYTES) { return size; }
 #define unpack_start(chr, size) ((unsigned char) chr & ~UTF_8_ENCODED_ ## size ## _BYTES_MASK)
 #define unpack_cont(chr) ((unsigned char) chr & ~UTF_8_ENCODED_MASK)
 
-int edited_names = 1;
-
 struct  bleach_t {
 	char *original;
 	char *replacement;
 };
 
 static int
-get_utf_8_width(char c)
+get_utf_8_width(const char c)
 {
-	if ((c & 0xc0) == 0xc0) {
+	if ((c & UTF_8_ENCODED_MASK) == UTF_8_ENCODED_START) { /* UTF-8 leading byte */
 		check_width(c, 2);
 		check_width(c, 3);
 		check_width(c, 4);
@@ -97,7 +100,7 @@ get_utf_8_width(char c)
 		check_width(c, 6);
     }
 
-	if ((c & 0xc0) == 0x80) {
+	if ((c & UTF_8_ENCODED_MASK) == UTF_8_ENCODED_CONT) { /* UTF-8 continuation byte */
 		return -1;
 	}
 
@@ -109,7 +112,7 @@ get_utf_8_width(char c)
  * {[()]} are replaced by a dash (-)
  * Everything else is replaced by an underscore (_) */
 static int
-translate_unsafe_char(unsigned char c)
+translate_unsafe_char(const unsigned char c)
 {
 	unsigned char t = 0;
 	if ((c >= '0' && c <= '9') || (c >= 'a' && c <= 'z')
@@ -124,7 +127,6 @@ translate_unsafe_char(unsigned char c)
 	return t;
 }
 
-
 static int
 get_uft8_dec_value(size_t *i, char *str)
 {
@@ -148,15 +150,11 @@ get_uft8_dec_value(size_t *i, char *str)
 		(*i)++;
 
 		if (str[*i] == '\0') {
-/*			fprintf(stderr, "CliFM: warning: UTF-8 sequence ended unexpectedly "
-					"(null)\n"); */
 			failed = 1;
 			break;
 		}
 
-		if ((str[*i] & 0xc0) != 0x80) {
-/*			fprintf(stderr, "CliFM: warning: UTF-8 sequence ended unexpectedly "
-					"(missing continuation byte)\n"); */
+		if ((str[*i] & UTF_8_ENCODED_MASK) != UTF_8_ENCODED_CONT) { /* Not a UTF-8 continuation byte */
 			failed = 1;
 			break;
 		}
@@ -167,9 +165,7 @@ get_uft8_dec_value(size_t *i, char *str)
 		expected_chars--;
 	}
 
-	if (failed)
-		return (-2);
-	return new_value;
+	return failed == 1 ? (-2) : new_value;
 }
 
 /* Clean up NAME either by removing those (extended-ASCII/Unicode) characters
@@ -211,11 +207,11 @@ clean_file_name(char *restrict name)
 		/* ASCII chars */
 		if (n == 38) { /* & */
 			if (q == p || *(q - 1) != DEFAULT_TRANSLATION) {
-				strcat(q, "_and_");
+				xstrncat(q, strlen(q), "_and_", NAME_MAX + 1);
 				q += 5;
 				cur_len += 5;
 			} else {
-				sprintf((q == p) ? q : q - 1, "_and_");
+				snprintf(q - 1, NAME_MAX, "_and_");
 				q += 4;
 				cur_len += 4;
 			}
@@ -233,9 +229,8 @@ clean_file_name(char *restrict name)
 					*q = (char)ret;
 					q++;
 					cur_len++;
-				}
-				else {
-					if (q != p && *(q - 1) == DEFAULT_TRANSLATION) {
+				} else {
+					if (*(q - 1) == DEFAULT_TRANSLATION) {
 						q--;
 						*q = (char)ret;
 					}
@@ -286,13 +281,13 @@ clean_file_name(char *restrict name)
 
 		size_t tlen = strlen(t);
 		if (q == p || *(q - 1) != DEFAULT_TRANSLATION) {
-			strcat(q, t);
+			xstrncat(q, strlen(q), t, NAME_MAX + 1);
 			q += tlen;
 			cur_len += tlen;
 		} else {
-			sprintf((q == p) ? q : q - 1, "%s", t);
-			q += tlen - 1;
-			cur_len += tlen - 1;
+			snprintf(q - 1, NAME_MAX, "%s", t);
+			q += (tlen > 0 ? tlen - 1 : 0);
+			cur_len += (tlen > 0 ? tlen - 1 : 0);
 		}
 	}
 
@@ -305,9 +300,9 @@ clean_file_name(char *restrict name)
 
 	if (!*p) { /* Empty file name */
 		time_t rawtime = time(NULL);
-		struct tm tm;
-		localtime_r(&rawtime, &tm);
-		char *suffix = gen_date_suffix(tm);
+		struct tm t;
+		char *suffix = localtime_r(&rawtime, &t)
+			? gen_date_suffix(t) : (char *)NULL;
 		if (!suffix) {
 			free(p);
 			return (char *)NULL;
@@ -343,56 +338,62 @@ clean_file_name(char *restrict name)
  * the original list of files is returned instead. In case of error
  * returns NULL */
 static struct bleach_t *
-edit_replacements(struct bleach_t *bfiles, size_t *n)
+edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 {
 	if (!bfiles || !bfiles[0].original)
 		return (struct bleach_t *)NULL;
 
-	edited_names = 1;
-	log_function(NULL);
+	*edited_names = 1;
 
 	char f[PATH_MAX];
 	if (xargs.stealth_mode == 1)
-		snprintf(f, PATH_MAX - 1, "%s/%s", P_tmpdir, TMP_FILENAME);
+		snprintf(f, sizeof(f), "%s/%s", P_tmpdir, TMP_FILENAME);
 	else
-		snprintf(f, PATH_MAX - 1, "%s/%s", tmp_dir, TMP_FILENAME);
+		snprintf(f, sizeof(f), "%s/%s", tmp_dir, TMP_FILENAME);
 
 	int fd = mkstemp(f);
 	if (fd == -1) {
-		_err('e', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME, f, strerror(errno));
+		_err('e', PRINT_PROMPT, "bleach: mkstemp: %s: %s\n", f, strerror(errno));
 		return (struct bleach_t *)NULL;
 	}
 
 	FILE *fp = (FILE *)NULL;
 
-#ifdef __HAIKU__
+#if defined(__HAIKU__) || defined(__sun)
 	fp = fopen(f, "w");
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "%s: %s: %s\n", PROGRAM_NAME, f, strerror(errno));
+		_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
 		return (struct bleach_t *)NULL;
 	}
-#endif
+#endif /* __HAIKU__ || __sun */
+
+#if !defined(__HAIKU__) && !defined(__sun)
+	dprintf(fd, BLEACH_TMP_HEADER);
+#else
+	fprintf(fp, BLEACH_TMP_HEADER);
+#endif /* !__HAIKU__ && !__sun */
 
 	size_t i;
 	/* Copy all files to be renamed to the temp file */
 	for (i = 0; i < *n; i++) {
-#ifndef __HAIKU__
+#if !defined(__HAIKU__) && !defined(__sun)
 		dprintf(fd, "original: %s\nreplacement: %s\n\n",
+			bfiles[i].original, bfiles[i].replacement);
 #else
 		fprintf(fp, "original: %s\nreplacement: %s\n\n",
-#endif
 			bfiles[i].original, bfiles[i].replacement);
+#endif /* !__HAIKU__ && !__sun */
 	}
 	size_t total_files = i;
 
-#ifdef __HAIKU__
+#if defined(__HAIKU__) || defined(__sun)
 	fclose(fp);
-#endif
+#endif /* __HAIKU__ || __sun */
 	close(fd);
 
-	fp = open_fstream_r(f, &fd);
+	fp = open_fread(f, &fd);
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "%s: '%s': %s\n", FUNC_NAME, f, strerror(errno));
+		_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
 		return (struct bleach_t *)NULL;
 	}
 
@@ -405,19 +406,17 @@ edit_replacements(struct bleach_t *bfiles, size_t *n)
 	int exit_status = open_file(f);
 	open_in_foreground = 0;
 	if (exit_status != EXIT_SUCCESS) {
-		fprintf(stderr, _("%s: %s\n"), FUNC_NAME, strerror(errno));
-		if (unlinkat(fd, f, 0) == -1) {
-			_err('e', PRINT_PROMPT, "%s: '%s': %s\n", PROGRAM_NAME,
-			    f, strerror(errno));
-		}
-		close_fstream(fp, fd);
+		xerror("bleach: %s: %s\n", f, strerror(errno));
+		if (unlinkat(fd, f, 0) == -1)
+			_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		fclose(fp);
 		return (struct bleach_t *)NULL;
 	}
 
-	close_fstream(fp, fd);
-	fp = open_fstream_r(f, &fd);
+	fclose(fp);
+	fp = open_fread(f, &fd);
 	if (!fp) {
-		_err('e', PRINT_PROMPT, "%s: '%s': %s\n", FUNC_NAME, f, strerror(errno));
+		_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
 		return (struct bleach_t *)NULL;
 	}
 
@@ -425,14 +424,13 @@ edit_replacements(struct bleach_t *bfiles, size_t *n)
 	 * match, nothing was modified */
 	fstat(fd, &attr);
 	if (mtime_bfr == (time_t)attr.st_mtime) {
-		if (unlinkat(fd, f, 0) == -1) {
-			_err('e', PRINT_PROMPT, "%s: '%s': %s\n", PROGRAM_NAME,
-			    f, strerror(errno));
-		}
-		close_fstream(fp, fd);
-		edited_names = 0;
+		if (unlinkat(fd, f, 0) == -1)
+			_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+		fclose(fp);
+		*edited_names = 0;
 		return bfiles; /* Return the original list of files */
 	}
+
 	/* Free the original list of files */
 	for (i = 0; i < *n; i++) {
 		free(bfiles[i].original);
@@ -504,7 +502,11 @@ edit_replacements(struct bleach_t *bfiles, size_t *n)
 
 	*n = j;
 	free(line);
-	close_fstream(fp, fd);
+
+	if (unlinkat(fd, f, 0) == -1)
+		_err('e', PRINT_PROMPT, "bleach: %s: %s\n", f, strerror(errno));
+	fclose(fp);
+
 	return bfiles;
 }
 
@@ -514,21 +516,28 @@ edit_replacements(struct bleach_t *bfiles, size_t *n)
 int
 bleach_files(char **names)
 {
+	if (!names || !names[1] || IS_HELP(names[1])) {
+		puts(_(BLEACH_USAGE));
+		return EXIT_SUCCESS;
+	}
+
 	struct bleach_t *bfiles = (struct bleach_t *)NULL;
 
 	size_t f = 0, i = 1;
 	for (; names[i]; i++) {
 		char *dstr = dequote_str(names[i], 0);
 		if (!dstr) {
-			fprintf(stderr, _("%s: %s: Error dequoting file name\n"),
-				FUNC_NAME, names[i]);
+			xerror(_("bleach: %s: Error dequoting file name\n"), names[i]);
 			continue;
 		}
-		strcpy(names[i], dstr);
+
+		xstrsncpy(names[i], dstr, strlen(dstr) + 1);
 		free(dstr);
 		size_t nlen = strlen(names[i]);
-		if (names[i][nlen - 1] == '/')
-			names[i][--nlen] = '\0';
+		if (names[i][nlen - 1] == '/') {
+			names[i][nlen - 1] = '\0';
+			nlen--;
+		}
 
 		char *sl = strrchr(names[i], '/');
 		char *p = clean_file_name((sl && *(sl + 1)) ? sl + 1 : names[i]);
@@ -536,7 +545,8 @@ bleach_files(char **names)
 			continue;
 
 		/* Nothing to clean. Skip this one */
-		if (*names[i] == *p && strcmp(names[i], p) == 0) {
+		char *n = (sl && *(sl + 1)) ? sl + 1 : names[i];
+		if (*n == *p && strcmp(n, p) == 0) {
 			free(p);
 			continue;
 		}
@@ -545,19 +555,20 @@ bleach_files(char **names)
 		bfiles[f].original = savestring(names[i], nlen);
 		if (sl) {
 			*sl = '\0';
-			bfiles[f].replacement = (char *)xnmalloc(nlen + strlen(p) + 2, sizeof(char));
-			sprintf(bfiles[f].replacement, "%s/%s", names[i], p);
+			size_t len = nlen + strlen(p) + 2;
+			bfiles[f].replacement = (char *)xnmalloc(len, sizeof(char));
+			snprintf(bfiles[f].replacement, len, "%s/%s", names[i], p);
 			*sl = '/';
 		} else {
 			bfiles[f].replacement = savestring(p, strlen(p));
 		}
-		printf("%s %s->%s %s\n", bfiles[f].original, mi_c, df_c,
-				bfiles[f].replacement);
+		printf("%s %s->%s %s\n",
+			bfiles[f].original, mi_c, df_c, bfiles[f].replacement);
 		f++;
 		free(p);
 	}
 
-	if (f == 0) {
+	if (f == 0 || !bfiles) {
 		printf(_("%s: Nothing to do\n"), FUNC_NAME);
 		return EXIT_SUCCESS;
 	}
@@ -567,35 +578,38 @@ bleach_files(char **names)
 
 CONFIRM:
 	putchar('\n');
+	int _edit = 0, edited_names = 0;
 
 	while (!input) {
-		input = rl_no_hist(_("Is this OK? [y/N/(e)dit] "));
+		input = rl_no_hist(_("Is this OK? [y/n/(e)dit] "));
 
-		if (input && (*(input + 1) || !strchr("yYnNeE", *input))) {
+		if (!input)
+			continue;
+
+		if (input[1] || !strchr("yYnNeEq", *input)) {
 			free(input);
 			input = (char *)NULL;
 			continue;
 		}
 
-		if (!input)
-			break;
-
-		switch(*input) {
-			case 'y': /* fallthrough */
-			case 'Y': rename = 1; break;
-			case 'e':
-				bfiles = edit_replacements(bfiles, &f);
-				if (!bfiles)
-					break;
-				if (edited_names && f > 0) {
-					free(input);
-					input = (char *)NULL;
-					goto CONFIRM;
-				} else {
-					rename = 1;
-				}
+		switch (*input) {
+		case 'y': /* fallthrough */
+		case 'Y': rename = 1; break;
+		case 'e':
+			_edit = 1;
+			bfiles = edit_replacements(bfiles, &f, &edited_names);
+			if (!bfiles)
 				break;
-			default: break;
+
+			if (edited_names == 1 && f > 0) {
+				free(input);
+				input = (char *)NULL;
+				goto CONFIRM;
+			} else {
+				rename = 1;
+			}
+			break;
+		default: break;
 		}
 	}
 
@@ -603,8 +617,7 @@ CONFIRM:
 
 	if (f == 0) {
 		/* Just in case either the original or the replacement file name
-		 * was removed from the list by the user, leaving only one of the
-		 * two */
+		 * was removed from the list by the user, leaving only one of the two */
 		free(bfiles[0].original);
 		free(bfiles[0].replacement);
 		free(bfiles);
@@ -612,59 +625,67 @@ CONFIRM:
 		return EXIT_SUCCESS;
 	}
 
-	/* If renaming all selected files, deselect them */
-	if (rename && is_sel)
-		clear_selbox();
+	/* The user entered 'e' to edit the file, but nothing was modified
+	 * Ask for confirmation in case the user just wanted to see what would
+	 * be done */
+	if (_edit == 1) {
+		printf(_("%zu %s will be bleached\n"), f, f > 1 ? _("files") : _("file"));
+		if (rl_get_y_or_n(_("Continue? [y/n] ")) != 1) {
+			for (i = 0; i < f; i++) {
+				free(bfiles[i].original);
+				free(bfiles[i].replacement);
+			}
+			free(bfiles);
+			return EXIT_SUCCESS;
+		}
+	}
 
-	int total_rename = rename ? (int)f : 0;
+	/* If renaming all selected files, deselect them */
+	if (rename == 1 && is_sel)
+		deselect_all();
+
+	int total_rename = rename == 1 ? (int)f : 0;
 	size_t rep_suffix = 1;
 	int exit_status = EXIT_SUCCESS;
+
 	for (i = 0; i < f; i++) {
-		char *o = bfiles[i].original;
-		char *r = bfiles[i].replacement;
+		char *o = bfiles[i].original ? bfiles[i].original : (char *)NULL;
+		char *r = bfiles[i].replacement ? bfiles[i].replacement : (char *)NULL;
 		if (o && *o && r && *r && rename) {
 			/* Make sure the replacement file name does not exist. If
-			 * it does, append REP_SUFFIX and try again */
+			 * it does, append REP_SUFFIX and try again. */
 			struct stat a;
 			while (lstat(r, &a) == 0) {
 				char tmp[PATH_MAX];
-				xstrsncpy(tmp, r, PATH_MAX - 1);
-				r = (char *)xrealloc(r,	PATH_MAX * sizeof(char));
-				sprintf(r, "%s-%zu", tmp, rep_suffix);
+				xstrsncpy(tmp, r, sizeof(tmp));
+				r = (char *)xrealloc(r,	(PATH_MAX + 32) * sizeof(char));
+				snprintf(r, PATH_MAX + 31, "%s-%zu", tmp, rep_suffix);
 				rep_suffix++;
 			}
 
-			if (renameat(AT_FDCWD, o, AT_FDCWD, r) == -1) {
-				fprintf(stderr, "renameat: '%s': %s\n", o, strerror(errno));
+			if (renameat(XAT_FDCWD, o, XAT_FDCWD, r) == -1) {
+				xerror("bleach: renameat: %s: %s\n", o, strerror(errno));
 				total_rename--;
 				exit_status = EXIT_FAILURE;
 			}
 		}
+
 		free(o);
 		free(r);
 	}
+
 	free(bfiles);
 
 	if (exit_status == EXIT_FAILURE || total_rename == 0) {
 		printf(_("%s: %d file(s) bleached\n"), FUNC_NAME, total_rename);
 	} else {
-		_err(0, PRINT_PROMPT, ("%s: %d file(s) bleached\n"),
+		_err(ERR_NO_LOG, PRINT_PROMPT, _("%s: %d file(s) bleached\n"),
 			FUNC_NAME, total_rename);
 	}
 
-#if defined(__HAIKU__)// || defined(__APPLE__)
-	if (exit_status == EXIT_FAILURE)
-		return EXIT_FAILURE;
-	if (autols) {
-		free_dirlist();
-		return list_dir();
-	}
-	return EXIT_SUCCESS;
-#else
 	return exit_status;
-#endif /* __HAIKU__ || __APPLE__ */
 }
 
 #else
-void *__skip_me_bleach;
+void *_skip_me_bleach;
 #endif /* !_NO_BLEACH */
