@@ -2,7 +2,7 @@
  *
  * This file is part of CliFM
  * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -24,6 +24,10 @@
 #ifndef NAME_CLEANER_H
 #define NAME_CLEANER_H
 
-int bleach_files(char **names);
+__BEGIN_DECLS
+
+int bleach_files(char **);
+
+__END_DECLS
 
 #endif /* NAME_CLEANER_H */
