@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,14 +25,19 @@
 #ifndef NAVIGATION_H
 #define NAVIGATION_H
 
-int back_function(char **comm);
-int forth_function(char **comm);
-int xchdir(char *dir, const int set_title);
-int cd_function(char *new_path, const int print_error);
-char *fastback(char *str);
-int handle_workspaces(char *str);
-void print_dirhist(void);
-int backdir(char *str);
-char **get_bd_matches(const char *str, int *n, int mode);
+__BEGIN_DECLS
+
+int  back_function(char **);
+int  backdir(char *);
+int  cd_function(char *, const int);
+char *fastback(char *);
+int  forth_function(char **);
+char **get_bd_matches(const char *, int *, const int);
+int  handle_workspaces(char **);
+void print_dirhist(char *);
+int  pwd_function(const char *);
+int  xchdir(char *, const int);
+
+__END_DECLS
 
 #endif /* NAVIGATION_H */
