@@ -1,9 +1,9 @@
-/* prompt.c -- functions controlling the appearance and behaviour of the prompt */
+/* prompt.c -- functions controlling the appearance and behavior of the prompt */
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -22,38 +22,41 @@
  * MA 02110-1301, USA.
 */
 
+/* The decode_prompt function is taken from Bash (1.14.7), licensed under
+ * GPL-2.0-or-later, and modified to fit our needs. */
+
 #include "helpers.h"
 
 #include <stdio.h>
 #include <string.h>
-#if !defined(__HAIKU__) && !defined(__OpenBSD__)
+#if !defined(__HAIKU__) && !defined(__OpenBSD__) && !defined(__ANDROID__)
 # include <wordexp.h>
-#endif
+#endif /* !__HAIKU__ && !__OpenBSD__ && !__ANDROID__ */
 #include <readline/readline.h>
+#include <errno.h>
 
 #include "aux.h"
+#include "colors.h" /* update_warning_prompt_text_color() */
 #include "exec.h"
+#include "file_operations.h"
 #include "history.h"
 #include "init.h"
 #include "listing.h"
+#include "messages.h"
 #include "misc.h"
 #include "navigation.h"
 #include "prompt.h"
 #include "sanitize.h"
 
-#ifndef _NO_TRASH
-# include "trash.h"
-#else
-# include <time.h>
-#endif
+#ifndef _NO_SUGGESTIONS
+# include "suggestions.h"
+#endif /* !_NO_SUGGESTIONS */
 
 #define CTLESC '\001'
 #define CTLNUL '\177'
 
-#define __WS_STR_LEN sizeof(int) + 6 + (MAX_COLOR + 2) * 2
-
 #define ROOT_IND "\001\x1b[1;31m\002R\001\x1b[0m\002"
-#define ROOT_IND_NO_COLOR "\001\x1b[1m\002R\001\x1b[0m\002"
+#define ROOT_IND_NO_COLOR "R"
 #define ROOT_IND_SIZE 17
 #define STEALTH_IND "S\001\x1b[0m\002"
 #define STEALTH_IND_SIZE MAX_COLOR + 7 + 1
@@ -83,6 +86,16 @@ emergency prompt"
 #define STATS_EXTENDED 16 /* Extended attributes (acl) */
 #define STATS_UNKNOWN  17
 #define STATS_UNSTAT   18
+#ifdef __sun
+# define STATS_DOOR    19
+#endif /* __sun */
+
+#define NOTIF_SEL     0
+#define NOTIF_TRASH   1
+#define NOTIF_WARNING 2
+#define NOTIF_ERROR   3
+#define NOTIF_NOTICE  4
+#define NOTIF_ROOT    5
 
 /* Size of the indicator for msgs, trash, and sel */
 #define N_IND MAX_COLOR + 1 + sizeof(size_t) + 6 + 1 + 13
@@ -94,9 +107,10 @@ gen_time(const int c)
 	char *temp = (char *)NULL;
 	time_t rawtime = time(NULL);
 	struct tm tm;
-	localtime_r(&rawtime, &tm);
 
-	if (c == 't') {
+	if (!localtime_r(&rawtime, &tm)) {
+		temp = savestring("?", 1);
+	} else if (c == 't') {
 		char time[9] = "";
 		strftime(time, sizeof(time), "%H:%M:%S", &tm);
 		temp = savestring(time, sizeof(time));
@@ -122,7 +136,7 @@ gen_time(const int c)
 }
 
 static inline char *
-get_dir_basename(char *_path)
+get_dir_basename(const char *_path)
 {
 	char *temp = (char *)NULL,
 		 *ret = (char *)NULL;
@@ -140,11 +154,11 @@ get_dir_basename(char *_path)
 }
 
 static inline char *
-reduce_path(char *_path)
+reduce_path(const char *_path)
 {
 	char *temp = (char *)NULL;
 
-	if (strlen(_path) > (size_t)max_path) {
+	if (strlen(_path) > (size_t)conf.max_path) {
 		char *ret = strrchr(_path, '/');
 		if (!ret)
 			temp = savestring(_path, strlen(_path));
@@ -158,26 +172,24 @@ reduce_path(char *_path)
 }
 
 static inline char *
-gen_pwd(int c)
+gen_pwd(const int c)
 {
-	char *temp = (char *)NULL;
-	/* Reduce HOME to "~" */
+	char *temp = (char *)NULL, *tmp_path = (char *)NULL;
 	int free_tmp_path = 0;
-	char *tmp_path = (char *)NULL;
 
-	if (strncmp(workspaces[cur_ws].path, user.home, user.home_len) == 0)
+	if (user.home
+	&& strncmp(workspaces[cur_ws].path, user.home, user.home_len) == 0)
 		tmp_path = home_tilde(workspaces[cur_ws].path, &free_tmp_path);
 
 	if (!tmp_path)
 		tmp_path = workspaces[cur_ws].path;
 
-	if (c == 'W') {
+	if (c == 'W')
 		temp = get_dir_basename(tmp_path);
-	} else if (c == 'p') {
+	else if (c == 'p')
 		temp = reduce_path(tmp_path);
-	} else { /* If c == 'w' */
+	else /* If c == 'w' */
 		temp = savestring(tmp_path, strlen(tmp_path));
-	}
 
 	if (free_tmp_path == 1)
 		free(tmp_path);
@@ -188,34 +200,45 @@ gen_pwd(int c)
 static inline char *
 gen_workspace(void)
 {
-	char s[__WS_STR_LEN];
+		/* An int (or workspace name) + workspaces color + NUL byte */
+	char s[NAME_MAX + sizeof(ws1_c) + 1];
 	char *cl = (char *)NULL;
 
-	switch(cur_ws + 1) {
-	case 1: cl = ws1_c; break;
-	case 2: cl = ws2_c; break;
-	case 3: cl = ws3_c; break;
-	case 4: cl = ws4_c; break;
-	case 5: cl = ws5_c; break;
-	case 6: cl = ws6_c; break;
-	case 7: cl = ws7_c; break;
-	case 8: cl = ws8_c; break;
-	default: break;
+	if (conf.colorize == 1) {
+		switch (cur_ws + 1) {
+		case 1: cl = *ws1_c ? ws1_c : DEF_WS1_C; break;
+		case 2: cl = *ws2_c ? ws2_c : DEF_WS2_C; break;
+		case 3: cl = *ws3_c ? ws3_c : DEF_WS3_C; break;
+		case 4: cl = *ws4_c ? ws4_c : DEF_WS4_C; break;
+		case 5: cl = *ws5_c ? ws5_c : DEF_WS5_C; break;
+		case 6: cl = *ws6_c ? ws6_c : DEF_WS6_C; break;
+		case 7: cl = *ws7_c ? ws7_c : DEF_WS7_C; break;
+		case 8: cl = *ws8_c ? ws8_c : DEF_WS8_C; break;
+		default: break;
+		}
+	} else {
+		cl = df_c;
 	}
 
-	snprintf(s, __WS_STR_LEN, "%s%d\001%s\002", cl, cur_ws + 1, df_c);
-	return savestring(s, strlen(s));
+	if (workspaces[cur_ws].name)
+		snprintf(s, sizeof(s), "%s%s", cl, workspaces[cur_ws].name);
+	else
+		snprintf(s, sizeof(s), "%s%d", cl, cur_ws + 1);
+
+	/* Using strnlen() here avoids a Redhat hardened compilation warning. */
+	return savestring(s, strnlen(s, sizeof(s)));
 }
 
 static inline char *
 gen_exit_status(void)
 {
 	size_t code_len = (size_t)DIGINUM(exit_code);
+	size_t len = code_len + 3 + (MAX_COLOR * 2);
 
-	char *temp = (char *)xnmalloc(code_len + 12 + MAX_COLOR, sizeof(char));
-	sprintf(temp, "%s%d\001%s\002",
-			(exit_code == 0) ? (colorize ? xs_c : "")
-			: (colorize ? xf_c : ""), exit_code, df_c);
+	char *temp = (char *)xnmalloc(len, sizeof(char));
+	snprintf(temp, len, "%s%d\001%s\002",
+		(exit_code == 0) ? (conf.colorize == 1 ? xs_c : "")
+		: (conf.colorize == 1 ? xf_c : ""), exit_code, df_c);
 
 	return temp;
 }
@@ -239,7 +262,7 @@ gen_octal(char **line, int *c)
 	char octal_string[4];
 	int n;
 
-	xstrsncpy(octal_string, *line, 3);
+	xstrsncpy(octal_string, *line, sizeof(octal_string));
 	octal_string[3] = '\0';
 
 	n = read_octal(octal_string);
@@ -293,7 +316,8 @@ gen_user_name(void)
 static inline char *
 gen_hostname(const int c)
 {
-	char *temp = savestring(hostname, strlen(hostname));
+	/* Using strnlen() here avoids a Redhat hardened compilation warning. */
+	char *temp = savestring(hostname, strnlen(hostname, sizeof(hostname)));
 	if (c != 'h')
 		return temp;
 
@@ -308,13 +332,9 @@ static inline char *
 gen_user_flag(void)
 {
 	char *temp = (char *)xnmalloc(2, sizeof(char));
-
-	if ((flags & ROOT_USR))
-		*temp = '#';
-	else
-		*temp = '$';
-
+	*temp = user.uid == 0 ? '#' : '$';
 	temp[1] = '\0';
+
 	return temp;
 }
 
@@ -322,7 +342,8 @@ static inline char *
 gen_mode(void)
 {
 	char *temp = (char *)xnmalloc(2, sizeof(char));
-	if (light_mode) {
+
+	if (conf.light_mode == 1) {
 		*temp = 'L';
 		temp[1] = '\0';
 	} else {
@@ -353,8 +374,7 @@ static inline char *
 gen_non_print_sequence(const int c)
 {
 	char *temp = (char *)xnmalloc(2, sizeof(char));
-	*temp = (c == '[') ? RL_PROMPT_START_IGNORE
-			: RL_PROMPT_END_IGNORE;
+	*temp = (c == '[') ? RL_PROMPT_START_IGNORE : RL_PROMPT_END_IGNORE;
 	temp[1] = '\0';
 
 	return temp;
@@ -363,15 +383,10 @@ gen_non_print_sequence(const int c)
 static inline char *
 gen_shell_name(void)
 {
-	char *p = (char *)NULL,
-		 *shell_name = strrchr(user.shell, '/');
+	if (user.shell && user.shell_basename)
+		return savestring(user.shell_basename, strlen(user.shell_basename));
 
-	if (shell_name && *(shell_name + 1))
-		p = shell_name + 1;
-	else
-		p = user.shell;
-
-	return savestring(p, strlen(p));
+	return savestring("unknown", 7);
 }
 
 static inline void
@@ -393,10 +408,11 @@ add_string(char **tmp, const int c, char **line, char **res, size_t *len)
 		*res = (char *)xrealloc(*res, (l + 1) * sizeof(char));
 	}
 
-	strcat(*res, *tmp);
+	xstrncat(*res, strlen(*res), *tmp, l + 1);
 	free(*tmp);
 }
 
+#if !defined(__HAIKU__) && !defined(__OpenBSD__) && !defined(__ANDROID__)
 static inline void
 reset_ifs(const char *value)
 {
@@ -406,15 +422,15 @@ reset_ifs(const char *value)
 		unsetenv("IFS");
 }
 
-#if !defined(__HAIKU__) && !defined(__OpenBSD__)
 static inline void
 substitute_cmd(char **line, char **res, size_t *len)
 {
 	int tmp = strcntchr(*line, ')');
 	if (tmp == -1) return; /* No ending bracket */
 
-	char *tmp_str = (char *)xnmalloc(strlen(*line) + 2, sizeof(char));
-	sprintf(tmp_str, "$%s", *line);
+	size_t tmp_len = strlen(*line) + 2;
+	char *tmp_str = (char *)xnmalloc(tmp_len, sizeof(char));
+	snprintf(tmp_str, tmp_len, "$%s", *line);
 
 	tmp_str[tmp + 2] = '\0';
 	*line += tmp + 1;
@@ -440,14 +456,14 @@ substitute_cmd(char **line, char **res, size_t *len)
 			} else {
 				*res = (char *)xrealloc(*res, (*len + 2) * sizeof(char));
 			}
-			strcat(*res, wordbuf.we_wordv[j]);
+			xstrncat(*res, strlen(*res), wordbuf.we_wordv[j], *len + 2);
 		}
 	}
 
 	wordfree(&wordbuf);
 	return;
 }
-#endif /* !__HAIKU__ && !__OpenBSD__*/
+#endif /* !__HAIKU__ && !__OpenBSD__ && !__ANDROID__ */
 
 static inline char *
 gen_emergency_prompt(void)
@@ -455,35 +471,38 @@ gen_emergency_prompt(void)
 	static int f = 0;
 	if (f == 0) {
 		f = 1;
-		fprintf(stderr, _("%s: %s\n"), PROGRAM_NAME, EMERGENCY_PROMPT_MSG);
+		xerror("%s: %s\n", PROGRAM_NAME, EMERGENCY_PROMPT_MSG);
 	}
 	char *_prompt = savestring(EMERGENCY_PROMPT, EMERGENCY_PROMPT_LEN);
 	return _prompt;
 }
 
 static inline char *
-gen_stats_str(int flag)
+gen_stats_str(const int flag)
 {
 	size_t val = 0;
 
-	switch(flag) {
-	case STATS_DIR: val = stats.dir; break;
-	case STATS_REG: val = stats.reg; break;
-	case STATS_EXE: val = stats.exec; break;
-	case STATS_HIDDEN: val = stats.hidden; break;
-	case STATS_SUID: val = stats.suid; break;
-	case STATS_SGID: val = stats.sgid; break;
-	case STATS_FIFO: val = stats.fifo; break;
-	case STATS_SOCK: val = stats.socket; break;
+	switch (flag) {
 	case STATS_BLK: val = stats.block_dev; break;
-	case STATS_CHR: val = stats.char_dev; break;
-	case STATS_CAP: val = stats.caps; break;
-	case STATS_LNK: val = stats.link; break;
 	case STATS_BROKEN_L: val = stats.broken_link; break;
+	case STATS_CAP: val = stats.caps; break;
+	case STATS_CHR: val = stats.char_dev; break;
+	case STATS_DIR: val = stats.dir; break;
+#ifdef __sun
+	case STATS_DOOR: val = stats.door; break;
+#endif /* __sun */
+	case STATS_EXE: val = stats.exec; break;
+	case STATS_EXTENDED: val = stats.extended; break;
+	case STATS_FIFO: val = stats.fifo; break;
+	case STATS_HIDDEN: val = stats.hidden; break;
+	case STATS_LNK: val = stats.link; break;
 	case STATS_MULTI_L: val = stats.multi_link; break;
 	case STATS_OTHER_W: val = stats.other_writable; break;
+	case STATS_REG: val = stats.reg; break;
+	case STATS_SUID: val = stats.suid; break;
+	case STATS_SGID: val = stats.sgid; break;
+	case STATS_SOCK: val = stats.socket; break;
 	case STATS_STICKY: val = stats.sticky; break;
-	case STATS_EXTENDED: val = stats.extended; break;
 	case STATS_UNKNOWN: val = stats.unknown; break;
 	case STATS_UNSTAT: val = stats.unstat; break;
 	default: break;
@@ -491,8 +510,8 @@ gen_stats_str(int flag)
 
 	char *p = (char *)NULL;
 	if (val != 0) {
-		p = (char *)xnmalloc(sizeof(size_t) + 1, sizeof(char));
-		sprintf(p, "%zu", val);
+		p = (char *)xnmalloc(32, sizeof(char));
+		snprintf(p, 32, "%zu", val);
 	} else {
 		p = (char *)xnmalloc(2, sizeof(char));
 		*p = '-';
@@ -502,19 +521,54 @@ gen_stats_str(int flag)
 	return p;
 }
 
-/*
-static inline void
-write_result(char **res, size_t *len, const int c)
+static inline char *
+gen_notification(const int flag)
 {
-	*res = (char *)xrealloc(*res, (*len + 2) * sizeof(char));
-	*res[*len] = (char)c;
-	(*len)++;
-	*res[*len] = '\0';
-} */
+	char *p;
+	size_t len = 32;
+
+/*	if (user.uid == 0) {
+		p = (char *)xnmalloc(2, sizeof(char));
+		len = 2;
+	} else { */
+//	len = DIGINUM(UINT_MAX) + 2;
+	p = (char *)xnmalloc(len, sizeof(char));
+//	}
+	*p = '\0';
+
+	switch (flag) {
+	case NOTIF_ERROR:
+		if (msgs.error > 0)
+			snprintf(p, len, "E%zu", msgs.error);
+		break;
+	case NOTIF_NOTICE:
+		if (msgs.notice > 0)
+			snprintf(p, len, "N%zu", msgs.notice);
+		break;
+	case NOTIF_WARNING:
+		if (msgs.warning > 0)
+			snprintf(p, len, "W%zu", msgs.warning);
+		break;
+	case NOTIF_ROOT:
+		if (user.uid == 0)
+			{ *p = 'R'; p[1] = '\0'; }
+		break;
+	case NOTIF_SEL:
+		if (sel_n > 0)
+			snprintf(p, len, "*%zu", sel_n);
+		break;
+	case NOTIF_TRASH:
+		if (trash_n > 2)
+			snprintf(p, len, "T%zu", trash_n - 2);
+		break;
+	default: break;
+	}
+
+	return p;
+}
 
 /* Decode the prompt string (encoded_prompt global variable) taken from
- * the configuration file. Based on the decode_prompt_string function
- * found in an old bash release (1.14.7). */
+ * the configuration file. */
 char *
 decode_prompt(char *line)
 {
@@ -532,25 +586,35 @@ decode_prompt(char *line)
 			c = *line;
 			switch (c) {
 			/* Files statistics */
-			case 'D': temp = gen_stats_str(STATS_DIR); goto ADD_STRING;
-			case 'R': temp = gen_stats_str(STATS_REG); goto ADD_STRING;
-			case '.': temp = gen_stats_str(STATS_HIDDEN); goto ADD_STRING;
-			case 'X': temp = gen_stats_str(STATS_EXE); goto ADD_STRING;
-			case 'U': temp = gen_stats_str(STATS_SUID); goto ADD_STRING;
-			case 'G': temp = gen_stats_str(STATS_SGID); goto ADD_STRING;
-			case 'F': temp = gen_stats_str(STATS_FIFO); goto ADD_STRING;
-			case 'K': temp = gen_stats_str(STATS_SOCK); goto ADD_STRING;
 			case 'B': temp = gen_stats_str(STATS_BLK); goto ADD_STRING;
 			case 'C': temp = gen_stats_str(STATS_CHR); goto ADD_STRING;
-			case 'x': temp = gen_stats_str(STATS_CAP); goto ADD_STRING;
-			case 'L': temp = gen_stats_str(STATS_LNK); goto ADD_STRING;
-			case 'o': temp = gen_stats_str(STATS_BROKEN_L); goto ADD_STRING;
-			case 'M': temp = gen_stats_str(STATS_MULTI_L); goto ADD_STRING;
+			case 'D': temp = gen_stats_str(STATS_DIR); goto ADD_STRING;
 			case 'E': temp = gen_stats_str(STATS_EXTENDED); goto ADD_STRING;
+			case 'F': temp = gen_stats_str(STATS_FIFO); goto ADD_STRING;
+			case 'G': temp = gen_stats_str(STATS_SGID); goto ADD_STRING;
+			case 'K': temp = gen_stats_str(STATS_SOCK); goto ADD_STRING;
+			case 'L': temp = gen_stats_str(STATS_LNK); goto ADD_STRING;
+			case 'M': temp = gen_stats_str(STATS_MULTI_L); goto ADD_STRING;
+			case 'o': temp = gen_stats_str(STATS_BROKEN_L); goto ADD_STRING;
 			case 'O': temp = gen_stats_str(STATS_OTHER_W); goto ADD_STRING;
-			case '*': temp = gen_stats_str(STATS_STICKY); goto ADD_STRING;
+			case 'R': temp = gen_stats_str(STATS_REG); goto ADD_STRING;
+			case 'U': temp = gen_stats_str(STATS_SUID); goto ADD_STRING;
+			case 'x': temp = gen_stats_str(STATS_CAP); goto ADD_STRING;
+			case 'X': temp = gen_stats_str(STATS_EXE); goto ADD_STRING;
+			case '.': temp = gen_stats_str(STATS_HIDDEN); goto ADD_STRING;
+			case '"': temp = gen_stats_str(STATS_STICKY); goto ADD_STRING;
 			case '?': temp = gen_stats_str(STATS_UNKNOWN); goto ADD_STRING;
 			case '!': temp = gen_stats_str(STATS_UNSTAT); goto ADD_STRING;
+#ifdef __sun
+			case '>': temp = gen_stats_str(STATS_DOOR); goto ADD_STRING;
+#endif /* __sun */
+
+			case '*': temp = gen_notification(NOTIF_SEL); goto ADD_STRING;
+			case '%': temp = gen_notification(NOTIF_TRASH); goto ADD_STRING;
+			case '#': temp = gen_notification(NOTIF_ROOT); goto ADD_STRING;
+			case ')': temp = gen_notification(NOTIF_WARNING); goto ADD_STRING;
+			case '(': temp = gen_notification(NOTIF_ERROR); goto ADD_STRING;
+			case '=': temp = gen_notification(NOTIF_NOTICE); goto ADD_STRING;
 
 			case 'z': /* Exit status of last executed command */
 				temp = gen_exit_status(); goto ADD_STRING;
@@ -569,7 +633,8 @@ decode_prompt(char *line)
 				temp = gen_octal(&line, &c); goto ADD_STRING;
 
 			case 'c': /* Program name */
-				temp = savestring(PNL, strlen(PNL)); goto ADD_STRING;
+				temp = savestring(PROGRAM_NAME, strlen(PROGRAM_NAME));
+				goto ADD_STRING;
 
 			case 'P': /* Current profile name */
 				temp = gen_profile(); goto ADD_STRING;
@@ -630,21 +695,20 @@ ADD_STRING:
 		}
 
 		/* If not escape code, check for command substitution, and if not,
-		 * just add whatever char is there */
+		 * just add whatever char is there. */
 		else {
 			/* Remove non-escaped quotes */
 			if (c == '\'' || c == '"')
 				continue;
 
-#if !defined(__HAIKU__) && !defined(__OpenBSD__)
+#if !defined(__HAIKU__) && !defined(__OpenBSD__) && !defined(__ANDROID__)
 			/* Command substitution */
 			if (c == '$' && *line == '(') {
 				substitute_cmd(&line, &result, &result_len);
 				continue;
 			}
-#endif /* __HAIKU__ && __OpenBSD__ */
+#endif /* !__HAIKU__ && !__OpenBSD__ && !__ANDROID__ */
 
-/*			write_result(&result, &result_len, c); */
 			size_t new_len = result_len + 2
 							+ (wrong_cmd ? (MAX_COLOR + 6) : 0);
 			result = (char *)xrealloc(result, new_len * sizeof(char));
@@ -654,11 +718,8 @@ ADD_STRING:
 		}
 	}
 
-/*	if (wrong_cmd == 1)
-		sprintf(result + result_len, "\001\x1b[0m\002%s", wp_c); */
-
 	/* Remove trailing new line char, if any */
-	if (result && result[result_len - 1] == '\n')
+	if (result && result_len > 0 && result[result_len - 1] == '\n')
 		result[result_len - 1] = '\0';
 
 	/* Emergency prompt, just in case something went wrong */
@@ -698,21 +759,29 @@ trim_final_slashes(void)
 static inline void
 print_welcome_msg(void)
 {
-	if (welcome_message) {
-		printf("%s%s > %s\n%s%s\n", wc_c, PROGRAM_NAME, _(PROGRAM_DESC),
-				df_c, _(HELP_MESSAGE));
-		welcome_message = 0;
-	}
+	static int wm = UNSET;
+
+	if (wm == 0 || conf.welcome_message == 0)
+		return;
+
+	if (conf.welcome_message_str)
+		printf("%s%s%s\n", wc_c, conf.welcome_message_str, df_c);
+	else
+		printf("%s%s\n%s", wc_c, DEF_WELCOME_MESSAGE_STR, df_c);
+
+	printf("%s\n", _(HELP_MESSAGE));
+
+	wm = 0;
 }
 
 static inline void
 _print_tips(void)
 {
-	if (tips == 0)
+	if (conf.tips == 0)
 		return;
 
 	static int first_run = 1;
-	if (first_run) {
+	if (first_run == 1) {
 		print_tips(0);
 		first_run = 0;
 	}
@@ -721,67 +790,79 @@ _print_tips(void)
 static inline void
 run_prompt_cmds(void)
 {
-	if (ext_cmd_ok == 0 || prompt_cmds_n == 0)
+	if (conf.ext_cmd_ok == 0 || prompt_cmds_n == 0)
 		return;
 
+	int tflags = flags;
+	flags &= ~DELAYED_REFRESH;
 	size_t i;
 	for (i = 0; i < prompt_cmds_n; i++) {
 		if (xargs.secure_cmds == 0
 		|| sanitize_cmd(prompt_cmds[i], SNT_PROMPT) == EXIT_SUCCESS)
-			launch_execle(prompt_cmds[i]);
+			launch_execl(prompt_cmds[i]);
 	}
+	flags = tflags;
 }
 
 #ifndef _NO_TRASH
 static inline void
 update_trash_indicator(void)
 {
-	if (trash_ok) {
-		trash_n = count_dir(trash_files_dir, NO_CPOP);
-		if (trash_n <= 2)
+	if (trash_ok == 1) {
+		int n = count_dir(trash_files_dir, NO_CPOP);
+		if (n <= 2)
 			trash_n = 0;
+		else
+			trash_n = (size_t)n;
 	}
 }
-#endif
+#endif /* !_NO_TRASH */
 
 static inline void
 setenv_prompt(void)
 {
-	if (prompt_style != CUSTOM_PROMPT_STYLE)
+	if (prompt_notif == 1)
 		return;
 
 	/* Set environment variables with CliFM state information
 	 * (sel files, trash, stealth mode, messages) to be handled by
 	 * the prompt itself */
 	char t[32];
-	sprintf(t, "%d", (int)sel_n);
+	snprintf(t, sizeof(t), "%d", (int)sel_n);
 	setenv("CLIFM_STAT_SEL", t, 1);
 #ifndef _NO_TRASH
-	sprintf(t, "%d", trash_n > 2 ? (int)trash_n - 2 : 0);
+	snprintf(t, sizeof(t), "%d", trash_n > 2 ? (int)trash_n - 2 : 0);
 	setenv("CLIFM_STAT_TRASH", t, 1);
-#endif
-	sprintf(t, "%d", (msgs_n && pmsg) ? (int)msgs_n : 0);
-	setenv("CLIFM_STAT_MSG", t, 1);
-	sprintf(t, "%d", cur_ws + 1);
+#endif /* !_NO_TRASH */
+	snprintf(t, sizeof(t), "%d", (int)msgs.error);
+	setenv("CLIFM_STAT_ERROR_MSGS", t, 1);
+	snprintf(t, sizeof(t), "%d", (int)msgs.warning);
+	setenv("CLIFM_STAT_WARNING_MSGS", t, 1);
+	snprintf(t, sizeof(t), "%d", (int)msgs.notice);
+	setenv("CLIFM_STAT_NOTICE_MSGS", t, 1);
+
+	snprintf(t, sizeof(t), "%d", cur_ws + 1);
 	setenv("CLIFM_STAT_WS", t, 1);
-	sprintf(t, "%d", exit_code);
+	snprintf(t, sizeof(t), "%d", exit_code);
 	setenv("CLIFM_STAT_EXIT", t, 1);
-	setenv("CLIFM_STAT_ROOT", (flags & ROOT_USR) ? "1" : "0", 1);
+	setenv("CLIFM_STAT_ROOT", user.uid == 0 ? "1" : "0", 1);
 	setenv("CLIFM_STAT_STEALTH", (xargs.stealth_mode == 1) ? "1" : "0", 1);
 }
 
 static inline size_t
-set_prompt_length(size_t decoded_prompt_len)
+set_prompt_length(const size_t decoded_prompt_len)
 {
 	size_t len = 0;
 
-	if (prompt_style == DEF_PROMPT_STYLE) {
+	if (prompt_notif == 1) {
 		len = (size_t)(decoded_prompt_len
 		+ (xargs.stealth_mode == 1 ? STEALTH_IND_SIZE : 0)
-		+ ((flags & ROOT_USR) ? ROOT_IND_SIZE : 0)
-		+ (sel_n ? N_IND : 0)
-		+ (trash_n ? N_IND : 0)
-		+ ((msgs_n && pmsg) ? N_IND : 0)
+		+ (user.uid == 0 ? ROOT_IND_SIZE : 0)
+		+ ((sel_n > 0) ? N_IND : 0)
+		+ ((trash_n > 0) ? N_IND : 0)
+		+ ((msgs.error > 0) ? N_IND : 0)
+		+ ((msgs.warning > 0) ? N_IND : 0)
+		+ ((msgs.notice > 0) ? N_IND : 0)
 		+ 6 + sizeof(tx_c) + 1 + 2);
 	} else {
 		len = (size_t)(decoded_prompt_len + 6 + sizeof(tx_c) + 1);
@@ -793,55 +874,42 @@ set_prompt_length(size_t decoded_prompt_len)
 static inline char *
 construct_prompt(const char *decoded_prompt)
 {
-	/* Construct indicators: MSGS, SEL, and TRASH */
+	/* Construct indicators: MSGS (ERR, WARN, and NOTICE), SEL, and TRASH */
+	char err_ind[N_IND], warn_ind[N_IND],
+		notice_ind[N_IND], trash_ind[N_IND], sel_ind[N_IND];
+	*err_ind = *warn_ind = *notice_ind = *trash_ind = *sel_ind = '\0';
 
-	/* Messages are categorized in three groups: errors, warnings, and
-	 * notices. The kind of message should be specified by the function
-	 * printing the message itself via a global enum: pmsg, with the
-	 * following values: NOMSG, ERROR, WARNING, and NOTICE. */
-	char msg_ind[N_IND];
-	*msg_ind = '\0';
-
-	if (msgs_n) {
-		/* Errors take precedence over warnings, and warnings over
-		 * notices. That is to say, if there is an error message AND a
-		 * warning message, the prompt will always display the error
-		 * message sign: a red 'E'. */
-		switch (pmsg) {
-		case NOMSG:	break;
-		case ERROR:	snprintf(msg_ind, N_IND, "%sE%zu%s", em_c, msgs_n, RL_NC); break;
-		case WARNING: snprintf(msg_ind, N_IND, "%sW%zu%s", wm_c, msgs_n, RL_NC); break;
-		case NOTICE: snprintf(msg_ind, N_IND, "%sN%zu%s", nm_c, msgs_n, RL_NC); break;
-		default: break;
-		}
+	if (prompt_notif == 1) {
+		if (msgs.error > 0)
+			snprintf(err_ind, N_IND, "%sE%zu%s", em_c, msgs.error, RL_NC);
+		if (msgs.warning > 0)
+			snprintf(warn_ind, N_IND, "%sW%zu%s", wm_c, msgs.warning, RL_NC);
+		if (msgs.notice > 0)
+			snprintf(notice_ind, N_IND, "%sN%zu%s", nm_c, msgs.notice, RL_NC);
+		if (trash_n > 2)
+			snprintf(trash_ind, N_IND, "%sT%zu%s",
+				ti_c, (size_t)trash_n - 2, RL_NC);
+		if (sel_n > 0)
+			snprintf(sel_ind, N_IND, "%s*%zu%s", li_c, sel_n, RL_NC);
 	}
 
-	char trash_ind[N_IND];
-	*trash_ind = '\0';
-	if (trash_n > 2)
-		snprintf(trash_ind, N_IND, "%sT%zu%s", ti_c, (size_t)trash_n - 2, RL_NC);
-
-	char sel_ind[N_IND];
-	*sel_ind = '\0';
-	if (sel_n > 0)
-		snprintf(sel_ind, N_IND, "%s*%zu%s", li_c, sel_n, RL_NC);
-
 	size_t prompt_len = set_prompt_length(strlen(decoded_prompt));
 	char *the_prompt = (char *)xnmalloc(prompt_len, sizeof(char));
 
-	if (prompt_style == DEF_PROMPT_STYLE) {
+	if (prompt_notif == 1) {
 		snprintf(the_prompt, prompt_len,
-			"%s%s%s%s%s%s%s%s\001%s\002",
-			(flags & ROOT_USR) ? (colorize ? ROOT_IND : ROOT_IND_NO_COLOR) : "",
-			(msgs_n && pmsg) ? msg_ind : "",
+			"%s%s%s%s%s%s%s%s%s%s\001%s\002",
+			(user.uid == 0) ? (conf.colorize == 1 ? ROOT_IND : ROOT_IND_NO_COLOR) : "",
+			(msgs.error > 0) ? err_ind : "",
+			(msgs.warning > 0) ? warn_ind : "",
+			(msgs.notice > 0) ? notice_ind : "",
 			(xargs.stealth_mode == 1) ? si_c : "",
 			(xargs.stealth_mode == 1) ? STEALTH_IND : "",
-			(trash_n) ? trash_ind : "",
-			(sel_n) ? sel_ind : "",
+			(trash_n > 0) ? trash_ind : "",
+			(sel_n > 0) ? sel_ind : "",
 			decoded_prompt, RL_NC, tx_c);
 	} else {
-		snprintf(the_prompt, prompt_len, "%s%s\001%s\002", decoded_prompt,
-			RL_NC, tx_c);
+		snprintf(the_prompt, prompt_len, "%s%s\001%s\002", decoded_prompt, RL_NC, tx_c);
 	}
 
 	return the_prompt;
@@ -855,6 +923,12 @@ initialize_prompt_data(void)
 	print_welcome_msg();
 	_print_tips();
 
+	/* If autols is disabled, and since terminal dimensions are gathered
+	 * in list_dir() via get_term_size(), let's get terminal dimensions
+	 * here. We need them to print suggestions */
+	if (conf.autols == 0 && conf.suggestions == 1)
+		get_term_size();
+
 	/* Set foreground color to default */
 	fputs(df_c, stdout);
 	fflush(stdout);
@@ -862,15 +936,22 @@ initialize_prompt_data(void)
 	run_prompt_cmds();
 #ifndef _NO_TRASH
 	update_trash_indicator();
-#endif
+#endif /* !_NO_TRASH */
 	get_sel_files();
 	setenv_prompt();
 
 	args_n = 0;
 	curhistindex = current_hist_n;
+#ifndef _NO_SUGGESTIONS
+	if (wrong_cmd == 1) {
+		rl_delete_text(0, rl_end);
+		rl_point = rl_end = 0;
+		recover_from_wrong_cmd();
+	}
+#endif /* !_NO_SUGGESTIONS */
 
 	/* Print error messages */
-	if (print_msg && msgs_n) {
+	if (print_msg == 1 && msgs_n > 0) {
 		fputs(messages[msgs_n - 1], stderr);
 		print_msg = 0; /* Print messages only once */
 	}
@@ -879,16 +960,13 @@ initialize_prompt_data(void)
 static inline void
 log_and_record(char *input)
 {
-	/* Keep a literal copy of the last entered command to compose the
-	 * commands log, if needed and enabled */
-	if (logs_enabled) {
+	if (conf.log_cmds == 1) {
 		free(last_cmd);
 		last_cmd = savestring(input, strlen(input));
+		log_cmd();
 	}
 
-	/* Do not record empty lines, exit, history commands, consecutively
-	 * equal inputs, or lines starting with space */
-	if (record_cmd(input))
+	if (record_cmd(input) == 1)
 		add_to_cmdhist(input);
 }
 /*
@@ -899,12 +977,12 @@ print_right_prompt(void)
 	get_cursor_position(0, 1);
 	char *p = decode_prompt(right_prompt);
 	printf("%*s", term_cols, p);
-	printf("\x1b[%d;%dH", currow, curcol);
+	printf("\x1b[%d;%dH", curline, curcol);
 	free(p);
 } */
 
-/* Print the prompt and return the string entered by the user (to be
- * parsed later by parse_input_str()) */
+/* Print the prompt and return the string entered by the user, to be
+ * parsed later by parse_input_str() */
 char *
 prompt(void)
 {
@@ -912,29 +990,212 @@ prompt(void)
 
 	/* Generate the prompt string using the prompt line in the config
 	 * file (stored in encoded_prompt at startup) */
-	char *decoded_prompt = decode_prompt(encoded_prompt);
-	char *the_prompt = construct_prompt(decoded_prompt);
+	char *decoded_prompt = decode_prompt(conf.encoded_prompt);
+	char *the_prompt = construct_prompt(decoded_prompt
+		? decoded_prompt : EMERGENCY_PROMPT);
 	free(decoded_prompt);
 
 	/* Tell my_rl_getc() (readline.c) to recalculate the length
-	 * of the last prompt line, needed to calculate FZF offset. This
-	 * length might vary if the prompt contains dynamic values */
+	 * of the last prompt line, needed to calculate the finder's offset
+	 * and the current cursor column. This length might vary if the
+	 * prompt contains dynamic values */
 	prompt_offset = UNSET;
 
 /*	if (right_prompt && *right_prompt)
 		print_right_prompt(); */
 
+	UNHIDE_CURSOR;
+
 	/* Print the prompt and get user input */
 	char *input = (char *)NULL;
 	input = readline(the_prompt);
 	free(the_prompt);
 
-	if (!input || !*input) {
+	if (!input || !*input || rl_end == 0) {
 		free(input);
+		if (conf.autols == 1 && ((flags & DELAYED_REFRESH)
+		|| xargs.refresh_on_empty_line == 1)) {
+			flags &= ~DELAYED_REFRESH;
+			refresh_screen();
+		} else {
+			flags &= ~DELAYED_REFRESH;
+		}
 		return (char *)NULL;
 	}
+	flags &= ~DELAYED_REFRESH;
 
 	log_and_record(input);
 
 	return input;
 }
+
+static int
+list_prompts(void)
+{
+	if (prompts_n == 0) {
+		puts(_("prompt: No extra prompts found. Using the default prompt"));
+		return EXIT_SUCCESS;
+	}
+
+	size_t i;
+	for (i = 0; i < prompts_n; i++) {
+		if (!prompts[i].name)
+			continue;
+		if (*cur_prompt_name == *prompts[i].name
+		&& strcmp(cur_prompt_name, prompts[i].name) == 0)
+			printf("%s>%s %s\n", mi_c, df_c, prompts[i].name);
+		else
+			printf("  %s\n", prompts[i].name);
+	}
+
+	return EXIT_SUCCESS;
+}
+
+static int
+switch_prompt(const size_t n)
+{
+	if (prompts[n].regular) {
+		free(conf.encoded_prompt);
+		conf.encoded_prompt = savestring(prompts[n].regular, strlen(prompts[n].regular));
+	}
+
+	if (prompts[n].warning) {
+		free(conf.wprompt_str);
+		conf.wprompt_str = savestring(prompts[n].warning, strlen(prompts[n].warning));
+	}
+
+	prompt_notif = prompts[n].notifications;
+
+	if (xargs.warning_prompt == 0)
+		return EXIT_SUCCESS;
+
+	conf.warning_prompt = prompts[n].warning_prompt_enabled;
+
+	update_warning_prompt_text_color();
+
+	return EXIT_SUCCESS;
+}
+
+static int
+set_prompt(char *name)
+{
+	if (!name || !*name)
+		return EXIT_FAILURE;
+
+	if (prompts_n == 0) {
+		xerror("%s\n", _("prompt: No extra prompts defined. Using the "
+			"default prompt"));
+		return EXIT_FAILURE;
+	}
+
+	char *p = dequote_str(name, 0);
+	if (!p) {
+		xerror(_("prompt: %s: Error dequoting string\n"), name);
+		return EXIT_FAILURE;
+	}
+
+	int i = (int)prompts_n;
+	while(--i >= 0) {
+		if (*p != *prompts[i].name || strcmp(p, prompts[i].name) != 0)
+			continue;
+		free(p);
+		xstrsncpy(cur_prompt_name, prompts[i].name, sizeof(cur_prompt_name));
+		return switch_prompt((size_t)i);
+	}
+
+	xerror(_("prompt: %s: No such prompt\n"), p);
+	free(p);
+	return EXIT_FAILURE;
+}
+
+static int
+set_default_prompt(void)
+{
+	free(conf.encoded_prompt);
+	conf.encoded_prompt = savestring(DEFAULT_PROMPT, strlen(DEFAULT_PROMPT));
+	free(conf.wprompt_str);
+	conf.wprompt_str = savestring(DEF_WPROMPT_STR, strlen(DEF_WPROMPT_STR));
+	*cur_prompt_name = '\0';
+	prompt_notif = DEF_PROMPT_NOTIF;
+	return EXIT_SUCCESS;
+}
+
+static int
+edit_prompts_file(char *app)
+{
+	if (xargs.stealth_mode == 1) {
+		printf("%s: prompt: %s\n", PROGRAM_NAME, STEALTH_DISABLED);
+		return EXIT_SUCCESS;
+	}
+
+	if (!prompts_file || !*prompts_file) {
+		xerror("%s\n", _("prompt: No prompts file found"));
+		return EXIT_FAILURE;
+	}
+
+	struct stat a;
+	if (stat(prompts_file, &a) == -1) {
+		xerror("prompt: %s: %s\n", prompts_file, strerror(errno));
+		return EXIT_FAILURE;
+	}
+
+	time_t old_time = a.st_mtime;
+
+	int ret = EXIT_FAILURE;
+	if (app && *app) {
+		char *cmd[] = {app, prompts_file, NULL};
+		ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
+	} else {
+		open_in_foreground = 1;
+		ret = open_file(prompts_file);
+		open_in_foreground = 0;
+	}
+
+	if (ret != EXIT_SUCCESS)
+		return ret;
+
+	stat(prompts_file, &a);
+	if (conf.autols == 1)
+		reload_dirlist();
+	if (old_time == a.st_mtime)
+		return EXIT_SUCCESS;
+
+	ret = load_prompts();
+	print_reload_msg(_("File modified. Prompts reloaded\n"));
+	if (*cur_prompt_name)
+		set_prompt(cur_prompt_name);
+	return ret;
+}
+
+int
+prompt_function(char **args)
+{
+	if (!args[0] || !*args[0] || (*args[0] == 'l'
+	&& strcmp(args[0], "list") == 0))
+		return list_prompts();
+
+	if (IS_HELP(args[0])) {
+		puts(PROMPT_USAGE);
+		return EXIT_SUCCESS;
+	}
+
+	if (*args[0] == 'u' && strcmp(args[0], "unset") == 0)
+		return set_default_prompt();
+
+	if (*args[0] == 'e' && strcmp(args[0], "edit") == 0)
+		return edit_prompts_file(args[1]);
+
+	if (*args[0] == 'r' && strcmp(args[0], "reload") == 0) {
+		int ret = load_prompts();
+		if (ret == EXIT_SUCCESS) {
+			printf(_("%s: Prompts successfully reloaded\n"), PROGRAM_NAME);
+			return EXIT_SUCCESS;
+		}
+		return ret;
+	}
+
+	if (*args[0] == 's' && strcmp(args[0], "set") == 0)
+		return set_prompt(args[1]);
+
+	return set_prompt(args[0]);
+}
