@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,7 +25,12 @@
 #ifndef PROMPT_H
 #define PROMPT_H
 
+__BEGIN_DECLS
+
 char *prompt(void);
-char *decode_prompt(char *line);
+char *decode_prompt(char *);
+int  prompt_function(char **);
+
+__END_DECLS
 
 #endif /* PROMPT_H */
