@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,10 +25,17 @@
 #ifndef __READLINE_H
 #define __READLINE_H
 
-int initialize_readline(void);
-char **my_rl_completion(const char *text, int start, int end);
-char *rl_no_hist(const char *prompt);
-int is_quote_char(const char c);
-int tab_complete(int mode);
+__BEGIN_DECLS
+
+int  alt_rl_prompt(const char *, const char *);
+int  initialize_readline(void);
+int  is_quote_char(const char);
+char **my_rl_completion(const char *, int, int);
+char *my_rl_path_completion(const char *, int);
+int  rl_get_y_or_n(const char *);
+char *rl_no_hist(const char *);
+int  tab_complete(int);
+
+__END_DECLS
 
 #endif /* __READLINE_H */
