@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -27,14 +27,13 @@
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
-#include <sys/stat.h>
 
 #include "aux.h"
 #include "exec.h"
+#include "init.h"
 #include "listing.h"
 #include "navigation.h"
 #include "history.h"
-#include "mime.h"
 #include "misc.h"
 #include "jump.h"
 #include "messages.h"
@@ -44,8 +43,9 @@
 static int
 remotes_list(void)
 {
-	if (!remotes_n) {
-		printf(_("%s: No remotes defined\n"), PROGRAM_NAME);
+	if (remotes_n == 0) {
+		printf(_("%s: No remotes defined. Run 'net edit' to add one.\n"),
+			PROGRAM_NAME);
 		return EXIT_SUCCESS;
 	}
 
@@ -53,7 +53,7 @@ remotes_list(void)
 	for (i = 0; i < remotes_n; i++) {
 		if (!remotes[i].name)
 			continue;
-		printf(_("Name: %s\n"), remotes[i].name);
+		printf(_("Name: %s%s%s\n"), BOLD, remotes[i].name, df_c);
 		if (remotes[i].desc)
 			printf(_(" Comment: %s\n"), remotes[i].desc);
 		if (remotes[i].mountpoint)
@@ -63,11 +63,11 @@ remotes_list(void)
 		if (remotes[i].unmount_cmd)
 			printf(_(" Unmount command: %s\n"), remotes[i].unmount_cmd);
 		printf(_(" Auto-unmount: %s\n"), (remotes[i].auto_unmount == 0)
-				? _("false") : _("true"));
+			? _("false") : _("true"));
 		printf(_(" Auto-mount: %s\n"), (remotes[i].auto_mount == 0)
-				? _("false") : _("true"));
-		printf(_(" Mounted: %s\n"), (remotes[i].mounted == 0) ? _("No")
-				: _("Yes"));
+			? _("false") : _("true"));
+		printf(_(" Mounted: %s%s%s\n"), BOLD, (remotes[i].mounted == 0)
+			? _("No") : _("Yes"), df_c);
 		if (i < remotes_n - 1)
 			puts("");
 	}
@@ -81,11 +81,10 @@ dequote_remote_name(char *name)
 	if (strchr(name, '\\')) {
 		deq = dequote_str(name, 0);
 		if (deq) {
-			strcpy(name, deq);
+			xstrsncpy(name, deq, strlen(deq) + 1);
 			free(deq);
 		} else {
-			fprintf(stderr, "%s: %s: Error dequoting resource name\n",
-					PROGRAM_NAME, name);
+			xerror("net: %s: Error dequoting resource name\n", name);
 			return EXIT_FAILURE;
 		}
 	}
@@ -94,7 +93,7 @@ dequote_remote_name(char *name)
 }
 
 /* Get the index of the remote named NAME from the remotes list. Returns
- * this index incase of success and -1 in case of error */
+ * this index in case of success or -1 in case of error */
 static int
 get_remote(char *name)
 {
@@ -114,14 +113,13 @@ get_remote(char *name)
 		}
 	}
 
-	if (!found) {
-		fprintf(stderr, _("%s: %s: No such remote\n"), PROGRAM_NAME, name);
+	if (found == 0) {
+		xerror(_("net: %s: No such remote\n"), name);
 		return (-1);
 	}
 
 	if (!remotes[i].mountpoint) {
-		fprintf(stderr, _("%s: No mountpoint specified for '%s'\n"), PROGRAM_NAME,
-				remotes[i].name);
+		xerror(_("net: No mountpoint specified for '%s'\n"), remotes[i].name);
 		return (-1);
 	}
 
@@ -129,13 +127,12 @@ get_remote(char *name)
 }
 
 static inline int
-_create_mountpoint(int i)
+_create_mountpoint(const int i)
 {
 	char *cmd[] = {"mkdir", "-p", remotes[i].mountpoint, NULL};
 
-	if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
-		fprintf(stderr, _("%s: %s: %s\n"), PROGRAM_NAME,
-				remotes[i].mountpoint, strerror(errno));
+	if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS) {
+		xerror("net: %s: %s\n", remotes[i].mountpoint, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
@@ -143,11 +140,11 @@ _create_mountpoint(int i)
 }
 
 static inline int
-cd_to_mountpoint(int i)
+cd_to_mountpoint(const int i)
 {
 	free(workspaces[cur_ws].path);
 	workspaces[cur_ws].path = savestring(remotes[i].mountpoint,
-						strlen(remotes[i].mountpoint));
+		strlen(remotes[i].mountpoint));
 	add_to_jumpdb(workspaces[cur_ws].path);
 	add_to_dirhist(workspaces[cur_ws].path);
 
@@ -159,18 +156,16 @@ cd_to_mountpoint(int i)
 }
 
 static inline int
-print_cd_error(int i)
+print_cd_error(const int i)
 {
-	fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME,
-			remotes[i].mountpoint, strerror(errno));
+	xerror("net: %s: %s\n", remotes[i].mountpoint, strerror(errno));
 	return EXIT_FAILURE;
 }
 
 static inline int
-print_no_mount_cmd_error(int i)
+print_no_mount_cmd_error(const int i)
 {
-	fprintf(stderr, _("%s: No mount command specified for '%s'\n"),
-		PROGRAM_NAME, remotes[i].name);
+	xerror(_("net: No mount command specified for '%s'\n"), remotes[i].name);
 	return EXIT_FAILURE;
 }
 
@@ -194,20 +189,20 @@ remotes_mount(char *name)
 	&& _create_mountpoint(i) == EXIT_FAILURE)
 		return EXIT_FAILURE;
 
-	/* Make sure mountpoint is not populated */
+	/* Make sure mountpoint is not populated and run the mount command */
 	if (count_dir(remotes[i].mountpoint, CPOP) <= 2
-	&& launch_execle(remotes[i].mount_cmd) != EXIT_SUCCESS)
+	&& launch_execl(remotes[i].mount_cmd) != EXIT_SUCCESS)
 		return EXIT_FAILURE;
 
 	if (xchdir(remotes[i].mountpoint, SET_TITLE) == -1)
 		return print_cd_error(i);
 
 	int exit_status = EXIT_SUCCESS;
-	if (autols) {
+	if (conf.autols == 1) {
 		exit_status = cd_to_mountpoint(i);
 	} else {
 		printf(_("%s: %s: Remote mounted on %s\n"), PROGRAM_NAME,
-				remotes[i].name, remotes[i].mountpoint);
+			remotes[i].name, remotes[i].mountpoint);
 	}
 
 	remotes[i].mounted = 1;
@@ -222,19 +217,17 @@ remotes_unmount(char *name)
 		return EXIT_FAILURE;
 
 	if (remotes[i].mounted == 0) {
-		fprintf(stderr, _("%s: %s: Not mounted\n"), PROGRAM_NAME, remotes[i].name);
+		xerror(_("net: %s: Not mounted\n"), remotes[i].name);
 		return EXIT_FAILURE;
 	}
 
 	if (!remotes[i].mountpoint) {
-		fprintf(stderr, _("%s: Error getting mountpoint for '%s'\n"), PROGRAM_NAME,
-				remotes[i].name);
+		xerror(_("net: Error getting mountpoint for '%s'\n"), remotes[i].name);
 		return EXIT_FAILURE;
 	}
 
 	if (!remotes[i].unmount_cmd) {
-		fprintf(stderr, _("%s: No unmount command found for '%s'\n"),
-			PROGRAM_NAME, remotes[i].name);
+		xerror(_("net: No unmount command for '%s'\n"), remotes[i].name);
 		return EXIT_FAILURE;
 	}
 
@@ -245,15 +238,15 @@ remotes_unmount(char *name)
 	/* Get out of mountpoint before unmounting */
 	size_t mlen = strlen(remotes[i].mountpoint);
 	if (strncmp(remotes[i].mountpoint, workspaces[cur_ws].path, mlen) == 0) {
-		if (remotes[i].mountpoint[mlen - 1] == '/') {
+		if (mlen > 0 && remotes[i].mountpoint[mlen - 1] == '/') {
 			mlen--;
 			remotes[i].mountpoint[mlen] = '\0';
 		}
 
 		char *p = strrchr(remotes[i].mountpoint, '/');
 		if (!p) {
-			fprintf(stderr, _("%s: %s: Error getting parent directory\n"),
-					PROGRAM_NAME, remotes[i].mountpoint);
+			xerror(_("net: %s: Error getting parent directory\n"),
+				remotes[i].mountpoint);
 			return EXIT_FAILURE;
 		}
 
@@ -261,18 +254,20 @@ remotes_unmount(char *name)
 		errno = 0;
 		if (xchdir(remotes[i].mountpoint, SET_TITLE) == EXIT_FAILURE) {
 			*p = '/';
-			fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME,
-					remotes[i].mountpoint, strerror(errno));
+			xerror("net: %s: %s\n", remotes[i].mountpoint, strerror(errno));
 			return EXIT_FAILURE;
 		}
 
 		free(workspaces[cur_ws].path);
 		workspaces[cur_ws].path = savestring(remotes[i].mountpoint,
-						strlen(remotes[i].mountpoint));
+			strlen(remotes[i].mountpoint));
 		*p = '/';
+
+		if (conf.autols == 1)
+			reload_dirlist();
 	}
 
-	if (launch_execle(remotes[i].unmount_cmd) != EXIT_SUCCESS)
+	if (launch_execl(remotes[i].unmount_cmd) != EXIT_SUCCESS)
 		return EXIT_FAILURE;
 
 	remotes[i].mounted = 0;
@@ -287,17 +282,16 @@ remotes_edit(char *app)
 
 	struct stat attr;
 	if (stat(remotes_file, &attr) == -1) {
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, remotes_file, strerror(errno));
+		xerror("net: %s: %s\n", remotes_file, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
 	time_t mtime_bfr = (time_t)attr.st_mtime;
 
 	int ret = EXIT_SUCCESS;
-	if (app) {
+	if (app && *app) {
 		char *cmd[] = {app, remotes_file, NULL};
-		if (launch_execve(cmd, FOREGROUND, E_NOSTDERR) != EXIT_SUCCESS)
-			ret = EXIT_FAILURE;
+		ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
 	} else {
 		open_in_foreground = 1;
 		ret = open_file(remotes_file);
@@ -305,16 +299,17 @@ remotes_edit(char *app)
 	}
 
 	if (ret != EXIT_SUCCESS)
-		return EXIT_FAILURE;
+		return ret;
 
 	if (stat(remotes_file, &attr) == -1) {
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, remotes_file, strerror(errno));
-		return EXIT_FAILURE;
+		xerror("net: %s: %s\n", remotes_file, strerror(errno));
+		return errno;
 	}
 
 	if (mtime_bfr != (time_t)attr.st_mtime) {
 		free_remotes(0);
 		load_remotes();
+		print_reload_msg(_("File modified. Remotes reloaded\n"));
 	}
 
 	return EXIT_SUCCESS;
@@ -324,20 +319,18 @@ int
 remotes_function(char **args)
 {
 	if (xargs.stealth_mode == 1) {
-		fprintf(stderr, "%s: net: %s\n", PROGRAM_NAME, STEALTH_DISABLED);
-		return EXIT_FAILURE;
-	}
-
-	if (!args[1])
-		return remotes_list();
-
-	if (IS_HELP(args[1])) {
-		puts(_(NET_USAGE));
+		printf("%s: net: %s\n", PROGRAM_NAME, STEALTH_DISABLED);
 		return EXIT_SUCCESS;
 	}
 
+	if (!args[1] || (*args[1] == 'l' && strcmp(args[1], "list") == 0))
+		return remotes_list();
+
+	if (IS_HELP(args[1]))
+		{ puts(_(NET_USAGE)); return EXIT_SUCCESS; }
+
 	if (*args[1] == 'e' && strcmp(args[1], "edit") == 0)
-		return remotes_edit(args[2] ? args[2] : NULL);
+		return remotes_edit(args[2]);
 
 	if (*args[1] == 'u' && (!*(args[1] + 1) || strcmp(args[1], "unmount") == 0)) {
 		if (!args[2]) {
@@ -361,11 +354,12 @@ remotes_function(char **args)
 int
 automount_remotes(void)
 {
-	if (!remotes_n)
+	if (remotes_n == 0)
 		return EXIT_SUCCESS;
 
-	int i = (int)remotes_n,
-		exit_status = EXIT_SUCCESS;
+	int i = (int)remotes_n;
+	int exit_status = EXIT_SUCCESS;
+
 	while (--i >= 0) {
 		if (remotes[i].name && remotes[i].auto_mount == 1
 		&& remotes[i].mountpoint && remotes[i].mount_cmd) {
@@ -377,17 +371,23 @@ automount_remotes(void)
 			struct stat attr;
 			if (stat(remotes[i].mountpoint, &attr) == -1) {
 				char *cmd[] = {"mkdir", "-p", remotes[i].mountpoint, NULL};
-				if (launch_execve(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
+				if (launch_execv(cmd, FOREGROUND, E_NOFLAG) != EXIT_SUCCESS)
 					continue;
 			} else {
 				if (count_dir(remotes[i].mountpoint, CPOP) > 2)
 					continue;
 			}
-			printf(_("%s: Mounting remote...\n"), remotes[i].name);
-			if (launch_execle(remotes[i].mount_cmd) != EXIT_SUCCESS)
+
+			int ret = 0;
+			printf(_("%s: net: %s: Mounting remote...\n"), PROGRAM_NAME,
+				remotes[i].name);
+			if ((ret = launch_execl(remotes[i].mount_cmd)) != EXIT_SUCCESS) {
+				_err('w', PRINT_PROMPT, _("net: %s: Mount command failed with "
+					"error code %d\n"), remotes[i].name, ret);
 				exit_status = EXIT_FAILURE;
-			else
+			} else {
 				remotes[i].mounted = 1;
+			}
 		}
 	}
 
@@ -397,11 +397,11 @@ automount_remotes(void)
 int
 autounmount_remotes(void)
 {
-	if (!remotes_n)
+	if (remotes_n == 0)
 		return EXIT_SUCCESS;
 
-	int i = (int)remotes_n,
-		exit_status = EXIT_SUCCESS;
+	int i = (int)remotes_n, exit_status = EXIT_SUCCESS;
+
 	while (--i >= 0) {
 		if (remotes[i].name && remotes[i].auto_unmount == 1
 		&& remotes[i].mountpoint && remotes[i].unmount_cmd) {
@@ -420,11 +420,16 @@ autounmount_remotes(void)
 				dir_change = 1;
 			}
 
-			printf(_("%s: Unmounting remote...\n"), remotes[i].name);
-			if (launch_execle(remotes[i].unmount_cmd) != EXIT_SUCCESS)
+			int ret = 0;
+			printf(_("%s: net: %s: Unmounting remote...\n"), PROGRAM_NAME,
+				remotes[i].name);
+			if ((ret = launch_execl(remotes[i].unmount_cmd)) != EXIT_SUCCESS) {
+				xerror(_("%s: net: %s: Unmount command failed with "
+					"error code %d\n"), PROGRAM_NAME, remotes[i].name, ret);
 				exit_status = EXIT_FAILURE;
+			}
 
-			if (dir_change)
+			if (dir_change == 1)
 				xchdir(workspaces[cur_ws].path, NO_TITLE);
 		}
 	}
