@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,8 +25,12 @@
 #ifndef _REMOTES_H
 #define _REMOTES_H
 
-int remotes_function(char **args);
+__BEGIN_DECLS
+
+int remotes_function(char **);
 int automount_remotes(void);
 int autounmount_remotes(void);
 
+__END_DECLS
+
 #endif /* _REMOTES_H */
