@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,7 +25,10 @@
 #ifndef SEARCH_H
 #define SEARCH_H
 
-int search_glob(char **comm, int invert);
-int search_regex(char **comm, int invert, int case_sens);
+__BEGIN_DECLS
+
+int search_function(char **);
+
+__END_DECLS
 
 #endif /* SEARCH_H */
