@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,14 +25,20 @@
 #ifndef SELECTION_H
 #define SELECTION_H
 
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
-#include <sys/ioctl.h>
-#endif
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) \
+|| defined(__DragonFly__)
+# include <sys/ioctl.h>
+#endif /* BSD */
 
-/* selection.c */
-int deselect(char **comm);
-int sel_function(char **args);
+__BEGIN_DECLS
+
+int  deselect(char **);
+int  sel_function(char **);
+int  select_file(char *);
 void show_sel_files(void);
-int save_sel(void);
+int  save_sel(void);
+int  deselect_all(void);
+
+__END_DECLS
 
 #endif /* _SELECTION_H */
