@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,17 +25,17 @@
 #include "helpers.h"
 
 #include <dirent.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#ifdef __OpenBSD__
-#include <strings.h>
-#endif
+#if defined(__OpenBSD__)
+# include <strings.h>
+#endif /* __OpenBSD__ */
 
 #include "checks.h"
-#include "aux.h" /* For xatoi */
+#include "aux.h" /* xatoi */
 #include "listing.h"
+#include "messages.h"
 
 int
 skip_nonexec(const struct dirent *ent)
@@ -82,11 +82,13 @@ skip_files(const struct dirent *ent)
 		return 0;
 
 	/* Skip files matching FILTER */
-	if (_filter && regexec(&regex_exp, ent->d_name, 0, NULL, 0) == EXIT_SUCCESS)
+	// ADD FILTER TYPE CHECK!
+	if (filter.str
+	&& regexec(&regex_exp, ent->d_name, 0, NULL, 0) == EXIT_SUCCESS)
 		return 0;
 
 	/* If not hidden files */
-	if (!show_hidden && *ent->d_name == '.')
+	if (conf.show_hidden == 0 && *ent->d_name == '.')
 		return 0;
 
 	return 1;
@@ -100,7 +102,7 @@ skip_name_prefixes(char **name)
 	char *s = *name;
 
 	while (*s) {
-		if (!_ISDIGIT(*s) && !_ISALPHA(*s) && (*s < 'A' || *s > 'Z')) {
+		if (!IS_DIGIT(*s) && !IS_ALPHA(*s) && (*s < 'A' || *s > 'Z')) {
 			s++;
 			continue;
 		}
@@ -113,14 +115,31 @@ skip_name_prefixes(char **name)
 	*name = s;
 }
 
+/* Simple comparison routine for qsort()ing strings */
+int
+compare_strings(char **s1, char **s2)
+{
+#if defined(HAVE_STRCOLL)
+	return strcoll(*s2, *s2);
+#else
+	int ret;
+
+	ret = **s1 - **s2;
+	if (ret == 0)
+		ret = strcmp(*s1, *s2);
+
+	return ret;
+#endif /* HAVE_STRCOLL */
+}
+
 static int
 namecmp(char *s1, char *s2)
 {
 	skip_name_prefixes(&s1);
 	skip_name_prefixes(&s2);
 
-	/* If both string starts with number, sort them as numbers, not as strings */
-	if (_ISDIGIT(*s1) && _ISDIGIT(*s2)) {
+	/* If both strings start with number, sort them as numbers, not as strings */
+	if (IS_DIGIT(*s1) && IS_DIGIT(*s2)) {
 		char *p1, *p2;
 		long long n1 = strtoll(s1, &p1, 10);
 		long long n2 = strtoll(s2, &p2, 10);
@@ -135,7 +154,7 @@ namecmp(char *s1, char *s2)
 	if ((*s1 & 0xc0) != 0xc0 && (*s2 & 0xc0) != 0xc0) {
 	/* None of the strings starts with a unicode char: compare the first
 	 * byte of both strings */
-		if (!case_sensitive) {
+		if (!conf.case_sens_list) {
 			ac = (char)TOUPPER(*s1);
 			bc = (char)TOUPPER(*s2);
 		}
@@ -147,7 +166,7 @@ namecmp(char *s1, char *s2)
 			return 1;
 	}
 
-	if (!case_sensitive || (*s1 & 0xc0) == 0xc0 || (*s2 & 0xc0) == 0xc0)
+	if (!conf.case_sens_list || (*s1 & 0xc0) == 0xc0 || (*s2 & 0xc0) == 0xc0)
 		return strcoll(s1, s2);
 
 	return strcmp(s1, s2);
@@ -157,11 +176,12 @@ static inline int
 sort_by_size(struct fileinfo *pa, struct fileinfo *pb)
 {
 	off_t as = pa->size, bs = pb->size;
-	if (long_view && full_dir_size) {
-		if (pa->type == DT_DIR)
-			as = pa->size * 1024;
-		if (pb->type == DT_DIR)
-			bs = pb->size * 1024;
+	if (conf.long_view == 1 && conf.full_dir_size == 1) {
+		int base = xargs.si == 1 ? 1000 : 1024;
+		if (pa->dir == 1)
+			as = pa->size * base;
+		if (pb->dir == 1)
+			bs = pb->size * base;
 	}
 
 	if (as > bs)
@@ -276,15 +296,15 @@ entrycmp(const void *a, const void *b)
 {
 	struct fileinfo *pa = (struct fileinfo *)a;
 	struct fileinfo *pb = (struct fileinfo *)b;
-	int ret = 0, st = sort;
+	int ret = 0, st = conf.sort;
 
-	if (list_folders_first) {
+	if (conf.list_dirs_first == 1) {
 		ret = sort_dirs(pa->dir, pb->dir);
 		if (ret != 0)
 			return ret;
 	}
 
-	if (light_mode && (st == SOWN || st == SGRP))
+	if (conf.light_mode == 1 && (st == SOWN || st == SGRP))
 		st = SNAME;
 
 	switch (st) {
@@ -303,7 +323,7 @@ entrycmp(const void *a, const void *b)
 
 	if (!ret)
 		ret = namecmp(pa->name, pb->name);
-	if (!sort_reverse)
+	if (!conf.sort_reverse)
 		return ret;
 
 	return (ret - (ret * 2));
@@ -328,7 +348,7 @@ xalphasort(const struct dirent **a, const struct dirent **b)
 	else
 		ret = strcmp((*a)->d_name, (*b)->d_name);
 
-	if (!sort_reverse)
+	if (!conf.sort_reverse)
 		return ret;
 
 	/* If sort_reverse, return the opposite value */
@@ -345,78 +365,79 @@ alphasort_insensitive(const struct dirent **a, const struct dirent **b)
 	int ret = strcasecmp(((*a)->d_name[0] == '.') ? (*a)->d_name + 1
 	: (*a)->d_name, ((*b)->d_name[0] == '.') ? (*b)->d_name + 1 : (*b)->d_name);
 
-	if (!sort_reverse)
+	if (!conf.sort_reverse)
 		return ret;
 
 	return (ret - (ret * 2));
 }
 
 static inline void
-print_owner_group_sort(int mode)
+print_owner_group_sort(const int mode)
 {
-	if (light_mode) {
+	if (conf.light_mode == 1) {
 		printf(_("%s (not available: using 'name') %s\n"),
 			(mode == SOWN) ? "owner" : "group",
-			(sort_reverse == 1) ? "[rev]" : "");
+			(conf.sort_reverse == 1) ? "[rev]" : "");
 		return;
 	}
 
 	printf(_("%s %s\n"), (mode == SOWN) ? "owner" : "group",
-		(sort_reverse == 1) ? "[rev]" : "");
+		(conf.sort_reverse == 1) ? "[rev]" : "");
 }
 
 void
 print_sort_method(void)
 {
-	switch (sort) {
+	fputs(BOLD, stdout);
+	switch (conf.sort) {
 	case SNONE:	puts(_("none")); break;
 	case SNAME:
-		printf(_("name %s\n"), (sort_reverse) ? "[rev]" : ""); break;
+		printf(_("name %s\n"), (conf.sort_reverse) ? "[rev]" : ""); break;
 	case STSIZE:
-		printf(_("size %s\n"), (sort_reverse) ? "[rev]" : ""); break;
+		printf(_("size %s\n"), (conf.sort_reverse) ? "[rev]" : ""); break;
 	case SATIME:
-		printf(_("atime %s\n"), (sort_reverse) ? "[rev]" : "");	break;
+		printf(_("atime %s\n"), (conf.sort_reverse) ? "[rev]" : "");	break;
 	case SBTIME:
 #if defined(HAVE_ST_BIRTHTIME) || defined(__BSD_VISIBLE) || defined(_STATX)
-		printf(_("btime %s\n"), (sort_reverse) ? "[rev]" : ""); break;
+		printf(_("btime %s\n"), (conf.sort_reverse) ? "[rev]" : ""); break;
 #else
 		printf(_("btime (not available: using 'ctime') %s\n"),
-		    (sort_reverse) ? "[rev]" : ""); break;
-#endif
+		    (conf.sort_reverse) ? "[rev]" : ""); break;
+#endif /* HAVE_ST_BIRTHTIME || __BSD_VISIBLE || _STATX */
 	case SCTIME:
-		printf(_("ctime %s\n"), (sort_reverse) ? "[rev]" : ""); break;
+		printf(_("ctime %s\n"), (conf.sort_reverse) ? "[rev]" : ""); break;
 	case SMTIME:
-		printf(_("mtime %s\n"), (sort_reverse) ? "[rev]" : ""); break;
+		printf(_("mtime %s\n"), (conf.sort_reverse) ? "[rev]" : ""); break;
 	case SVER:
-		printf(_("version %s\n"), (sort_reverse) ? "[rev]" : ""); break;
+		printf(_("version %s\n"), (conf.sort_reverse) ? "[rev]" : ""); break;
 	case SEXT:
-		printf(_("extension %s\n"), (sort_reverse) ? "[rev]" : "");	break;
+		printf(_("extension %s\n"), (conf.sort_reverse) ? "[rev]" : "");	break;
 	case SINO:
-		printf(_("inode %s\n"), (sort_reverse) ? "[rev]" : "");	break;
+		printf(_("inode %s\n"), (conf.sort_reverse) ? "[rev]" : "");	break;
 	case SOWN: print_owner_group_sort(SOWN); break;
 	case SGRP: print_owner_group_sort(SGRP); break;
-	default: fputs("unknown sorting method\n", stdout); break;
+	default: fputs("unknown sorting order\n", stdout); break;
 	}
+	fputs(NC, stdout);
 }
 
 static inline void
 toggle_sort_reverse(void)
 {
-	if (sort_reverse)
-		sort_reverse = 0;
+	if (conf.sort_reverse)
+		conf.sort_reverse = 0;
 	else
-		sort_reverse = 1;
+		conf.sort_reverse = 1;
 }
 
 static inline int
 re_sort_files_list(void)
 {
-	if (autols == 0)
+	if (conf.autols == 0)
 		return EXIT_SUCCESS;
 
-	/* sort_switch just tells list_dir() to print a line
-	 * with the current sorting method at the end of the
-	 * files list */
+	/* sort_switch just tells list_dir() to print a line with the current
+	 * sorting order at the end of the files list */
 	sort_switch = 1;
 	free_dirlist();
 	int ret = list_dir();
@@ -425,45 +446,31 @@ re_sort_files_list(void)
 	return ret;
 }
 
-/* If ARG is a string, write the corresponding integer to ARG itself
- * Return zero if ARG corresponds to a valid sorting method, and one
- * otherwise */
+/* If ARG is a string, write the corresponding integer to ARG itself.
+ * Return zero if ARG corresponds to a valid sorting method or one
+ * otherwise. */
 static inline int
 _set_sort_by_name(char **arg)
 {
-	static struct sort_t sorts[] = {
-	    {"none", 0, 0},
-	    {"name", 1, 0},
-	    {"size", 2, 0},
-	    {"atime", 3, 0},
-	    {"btime", 4, 0},
-	    {"ctime", 5, 0},
-	    {"mtime", 6, 0},
-	    {"version", 7, 0},
-	    {"extension", 8, 0},
-	    {"inode", 9, 0},
-	    {"owner", 10, 0},
-	    {"group", 11, 0},
-	};
-
 	size_t i;
-	for (i = 0; i < sizeof(sorts) / sizeof(struct sort_t); i++) {
-		if (*(*arg) == *sorts[i].name && strcmp(*arg, sorts[i].name) == 0) {
-			sprintf(*arg, "%d", sorts[i].num);
+	for (i = 0; i <= SORT_TYPES; i++) {
+		if (*(*arg) == *_sorts[i].name && strcmp(*arg, _sorts[i].name) == 0) {
+			*arg = (char *)xrealloc(*arg, 32 * sizeof(char));
+			snprintf(*arg, 32, "%d", _sorts[i].num);
 			return EXIT_SUCCESS;
 		}
 	}
 
-	fprintf(stdout, _("%s: %s: No such sorting method\n"), PROGRAM_NAME, *arg);
+	fprintf(stdout, _("st: %s: No such sorting order\n"), *arg);
 	return EXIT_FAILURE;
 }
 
 int
 sort_function(char **arg)
 {
-	/* No argument: Just print current sorting method */
+	/* No argument: Just print current sorting order */
 	if (!arg[1]) {
-		fputs(_("Sorting method: "), stdout);
+		fputs(_("Sorting order: "), stdout);
 		print_sort_method();
 		return EXIT_SUCCESS;
 	}
@@ -483,7 +490,7 @@ sort_function(char **arg)
 	int n = atoi(arg[1]);
 
 	if (n >= 0 && n <= SORT_TYPES) {
-		sort = n;
+		conf.sort = n;
 
 		if (arg[2] && *arg[2] == 'r' && strcmp(arg[2], "rev") == 0)
 			toggle_sort_reverse();
