@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -27,12 +27,20 @@
 
 #include <dirent.h>
 
-int alphasort_insensitive(const struct dirent **a, const struct dirent **b);
-int xalphasort(const struct dirent **a, const struct dirent **b);
-int sort_function(char **arg);
-int entrycmp(const void *a, const void *b);
-int skip_nonexec(const struct dirent *ent);
-int skip_files(const struct dirent *ent);
+/* Needed by qsort(3) to use compare_string() as comparison function */
+typedef int QSFUNC(const void *, const void *);
+
+__BEGIN_DECLS
+
+int  alphasort_insensitive(const struct dirent **, const struct dirent **);
+int  compare_strings(char **, char **);
+int  xalphasort(const struct dirent **, const struct dirent **);
+int  sort_function(char **);
+int  entrycmp(const void *, const void *);
+int  skip_nonexec(const struct dirent *);
+int  skip_files(const struct dirent *);
 void print_sort_method(void);
 
+__END_DECLS
+
 #endif /* SORT_H */
