@@ -2,8 +2,8 @@
 
 /*
  * This file is part of CliFM
- * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ *
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -29,27 +29,27 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <sys/stat.h>
 #include <dirent.h>
 #include <unistd.h>
 #include <fcntl.h>
-#include <readline/readline.h>
+#include <readline/tilde.h>
 
 #include "aux.h"
+#include "colors.h" /* colors_list() */
 #include "misc.h"
 #include "exec.h"
+#include "init.h"
+#include "messages.h"
 #include "sort.h"
 
 /* A few printing functions */
 static int
-print_tag_creation_error(const char *link, mode_t mode)
+print_tag_creation_error(const char *link, const mode_t mode)
 {
-	if (S_ISLNK(mode)) {
-		fprintf(stderr, _("%s: %s: File already tagged\n"), PROGRAM_NAME, link);
-	} else {
-		fprintf(stderr, _("%s: %s: Cannot create tag: file already "
-			"exists\n"), PROGRAM_NAME, link);
-	}
+	if (S_ISLNK(mode))
+		xerror(_("tag: %s: File already tagged\n"), link);
+	else
+		xerror(_("tag: %s: Cannot create tag: file already exists\n"), link);
 
 	return EXIT_FAILURE;
 }
@@ -57,41 +57,41 @@ print_tag_creation_error(const char *link, mode_t mode)
 static int
 print_symlink_error(const char *name)
 {
-	fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, name, strerror(errno));
+	xerror("tag: %s: %s\n", name, strerror(errno));
 	return errno;
 }
 
 static int
 print_no_tags(void)
 {
-	printf(_("%s: No tags found, Use 'tag new' to create new tags\n"),
-		PROGRAM_NAME);
+	printf(_("%s: No tags found, Use 'tag new' to create new "
+		"tags\n"), PROGRAM_NAME);
 	return EXIT_SUCCESS;
 }
 
 static int
 print_no_such_tag(const char *name)
 {
-	fprintf(stderr, _("%s: %s: No such tag\n"), PROGRAM_NAME, name);
+	xerror(_("tag: %s: No such tag\n"), name);
 	return EXIT_FAILURE;
 }
 
 static int
-print_usage(int retval)
+print_usage(const int retval)
 {
 	puts(TAG_USAGE);
 	return retval;
 }
 
 #ifndef _DIRENT_HAVE_D_TPYE
-/* Check whether NAME is actually tagged as TAG
- * Returns 1 if true and zero otherwise */
+/* Check whether NAME is actually tagged as TAG.
+ * Returns 1 if true and zero otherwise. */
 static int
 check_tagged_file(const char *tag, const char *name)
 {
 	struct stat a;
 	char tmp[PATH_MAX];
-	snprintf(tmp, PATH_MAX, "%s/%s/%s", tags_dir, tag, name);
+	snprintf(tmp, sizeof(tmp), "%s/%s/%s", tags_dir, tag, name);
 
 	if (SELFORPARENT(name) || lstat(tmp, &a) == -1 || !S_ISLNK(a.st_mode))
 		return 0;
@@ -100,13 +100,13 @@ check_tagged_file(const char *tag, const char *name)
 }
 #endif /* _DIRENT_HAVE_D_TPYE */
 
-/* Print the tagged file NAME tagged as TAG */
+/* Print the tagged file NAME tagged as TAG. */
 static void
 print_tagged_file(char *name, const char *tag)
 {
 	char dir[PATH_MAX], tmp[PATH_MAX];
 	*tmp = '\0';
-	snprintf(dir, PATH_MAX, "%s/%s/%s", tags_dir, tag, name);
+	snprintf(dir, sizeof(dir), "%s/%s/%s", tags_dir, tag, name);
 	char *ret = realpath(dir, tmp);
 	if (!ret)
 		return;
@@ -124,47 +124,46 @@ print_tagged_file(char *name, const char *tag)
 	if (strchr(name, '\\')) {
 		char *d = dequote_str(name, 0);
 		if (d) {
-			strcpy(name, d);
+			xstrsncpy(name, d, strlen(d) + 1);
 			free(d);
 		}
 	}
 
-	char *p = strrchr(tmp, '/');
-	if (p && p != q)
-		*p = '\0';
-	printf("%s%s%s [%s]\n", mi_c, (p && *(p + 1)) ? p + 1 : tmp, df_c, tmp);
+	putchar(' ');
+	char *abbr_name = abbreviate_file_name(tmp);
+	colors_list(abbr_name ? abbr_name : tmp, 0, 0, 1);
+	if (abbr_name != tmp)
+		free(abbr_name);
 
 	if (free_name == 1)
 		free(q);
 }
 
-/* Print the list of all files tagged as NAME */
+/* Print the list of all files tagged as NAME. */
 static int
 list_files_in_tag(char *name)
 {
 	if (strchr(name, '\\')) {
 		char *p = dequote_str(name, 0);
 		if (p) {
-			strcpy(name, p);
+			xstrsncpy(name, p, strlen(p) + 1);
 			free(p);
 		}
 	}
+
 	char tmp[PATH_MAX];
-	snprintf(tmp, PATH_MAX, "%s/%s", tags_dir, name);
+	snprintf(tmp, sizeof(tmp), "%s/%s", tags_dir, name);
 
 	struct dirent **t = (struct dirent **)NULL;
-	int n = scandir(tmp, &t, NULL, case_sensitive ? xalphasort : alphasort_insensitive);
-	if (n == -1) {
-		fprintf(stderr, _("%s: %s: No such tag\n"), PROGRAM_NAME, name);
-		return EXIT_FAILURE;
-	}
+	int n = scandir(tmp, &t, NULL, conf.case_sens_list ? xalphasort : alphasort_insensitive);
+	if (n == -1)
+		return errno;
 
 	size_t i;
 	if (n <= 2) {
 		for (i = 0; i < (size_t)n; i++)
 			free(t[i]);
 		free(t);
-		printf(_("No file tagged as '%s'\n"), name);
 		return EXIT_SUCCESS;
 	}
 
@@ -173,7 +172,7 @@ list_files_in_tag(char *name)
 		if (check_tagged_file(name, t[i]->d_name) == 0) {
 #else
 		if (SELFORPARENT(t[i]->d_name) || t[i]->d_type != DT_LNK) {
-#endif
+#endif /* !_DIRENT_HAVE_D_TYPE */
 			free(t[i]);
 			continue;
 		}
@@ -185,8 +184,8 @@ list_files_in_tag(char *name)
 	return EXIT_SUCCESS;
 }
 
-/* Return the longest tag name
- * Used to pad the output of 'tag ls' command */
+/* Return the length of longest tag name.
+ * Used to pad the output of the 'tag list' command. */
 static size_t
 get_longest_tag(void)
 {
@@ -201,50 +200,159 @@ get_longest_tag(void)
 	return _longest;
 }
 
-/* Print the list of available tags, plus the amount of tagged files for
- * each tag */
+/* List all tags applied to the file whose device ID is DEV and inode number
+ * is INO. */
+static void
+list_tags_having_file(const dev_t dev, const ino_t ino)
+{
+	if (!tags_dir || !tags)
+		return;
+
+	size_t i;
+	for (i = 0; tags[i]; i++) {
+		char tmp[PATH_MAX];
+		snprintf(tmp, sizeof(tmp), "%s/%s", tags_dir, tags[i]);
+
+		DIR *dir = opendir(tmp);
+		if (dir == NULL)
+			continue;
+
+		struct dirent *ent;
+		while ((ent = readdir(dir))) {
+			char full_name[PATH_MAX + NAME_MAX + 2];
+			snprintf(full_name, sizeof(full_name), "%s/%s", tmp, ent->d_name);
+
+			struct stat a;
+			if (stat(full_name, &a) == -1)
+				continue;
+
+			if (a.st_dev == dev && a.st_ino == ino) {
+				printf(" %s%s%s\n", mi_c, tags[i], NC);
+				break;
+			}
+		}
+
+		closedir(dir);
+	}
+}
+
+/* Check whether NAME is a valid and existent tag name.
+ * Returns 1 if true or zero otherwise. */
+int
+is_tag(char *name)
+{
+	if (!name || !*name)
+		return 0;
+
+	if (strchr(name, '\\')) {
+		char *deq = dequote_str(name, 0);
+		if (deq) {
+			xstrsncpy(name, deq, strlen(deq) + 1);
+			free(deq);
+		}
+	}
+
+	int i = (int)tags_n;
+	while (--i >= 0) {
+		if (*name == *tags[i] && strcmp(name, tags[i]) == 0)
+			return 1;
+	}
+
+	return 0;
+}
+
+/* Print the list of available tags and all files tagged as each tag. */
+static int
+list_tags_full(void)
+{
+	if (tags_n == 0) {
+		puts(_("tag: No tags"));
+		return EXIT_SUCCESS;
+	}
+
+	size_t i;
+	int exit_status = EXIT_SUCCESS;
+
+	for (i = 0; tags[i]; i++) {
+		printf(_("Files tagged as %s%s%s:\n"), conf.colorize == 1 ? BOLD : "'",
+			tags[i], conf.colorize == 1 ? NC : "'");
+		if (list_files_in_tag(tags[i]) != EXIT_SUCCESS)
+			exit_status = EXIT_FAILURE;
+
+		if (tags[i + 1])
+			putchar('\n');
+	}
+
+	return exit_status;
+}
+
 static int
 list_tags(char **args)
 {
 	if (tags_n == 0)
 		return print_no_tags();
 
-	if (args[1] && args[2]) {
-		int exit_status = EXIT_SUCCESS;
-		size_t i;
-		for (i = 2; args[i]; i++) {
-			if (args[3])
-				printf("%s:\n", args[i]);
+	size_t i;
+	int exit_status = EXIT_SUCCESS;
+
+	if (!args || !args[0] || !args[1] || !args[2]) {
+		/* 'tag list': list all tags */
+		int pad = (int)get_longest_tag();
+
+		for (i = 0; tags[i]; i++) {
+			char p[PATH_MAX];
+			snprintf(p, sizeof(p), "%s/%s", tags_dir, tags[i]);
+			int n = count_dir(p, NO_CPOP);
+			if (n > 2)
+				printf("%-*s [%s%d%s]\n", pad, tags[i], mi_c, n - 2, df_c);
+			else
+				printf("%-*s  -\n", pad, tags[i]);
+		}
+
+		return EXIT_SUCCESS;
+	}
+
+	for (i = 2; args[i]; i++) {
+		if (!is_tag(args[i])) {
+			/* 'tag list FILENAME' */
+			char *p = dequote_str(args[i], 0);
+
+			struct stat a;
+			if (lstat(p ? p : args[i], &a) == -1) {
+				exit_status = errno;
+				xerror("%s: %s\n", p ? p : args[i], strerror(errno));
+				free(p);
+				continue;
+			}
+
+			printf(_("%s%s%s is tagged as:\n"), conf.colorize == 1 ? BOLD : "'",
+				p ? p : args[i], conf.colorize == 1 ? NC : "'");
+			free(p);
+			list_tags_having_file(a.st_dev, a.st_ino);
+
+		} else {
+			/* 'tag list TAG' */
+			printf(_("Files tagged as %s%s%s:\n"), conf.colorize == 1 ? BOLD : "'",
+				args[i], conf.colorize == 1 ? BOLD : "'");
 			if (list_files_in_tag(args[i]) == EXIT_FAILURE)
 				exit_status = EXIT_FAILURE;
 		}
-		return exit_status;
+
+		if (args[i + 1])
+			putchar('\n');
 	}
 
-	int pad = (int)get_longest_tag();
-
-	size_t i;
-	for (i = 0; tags[i]; i++) {
-		char p[PATH_MAX];
-		snprintf(p, PATH_MAX, "%s/%s", tags_dir, tags[i]);
-		int n = count_dir(p, NO_CPOP);
-		if (n > 2)
-			printf("%-*s [%s%d%s]\n", pad, tags[i], mi_c, n - 2, df_c);
-		else
-			printf("%-*s  -\n", pad, tags[i]);
-	}
-
-	return EXIT_SUCCESS;
+	return exit_status;
 }
 
-static inline void
+static void
 reload_tags(void)
 {
 	free_tags();
 	load_tags();
 }
 
-/* Create tags according to ARGS + 2 */
+/* Create tags according to ARGS + 2. */
 static int
 create_tags(char **args)
 {
@@ -253,7 +361,7 @@ create_tags(char **args)
 
 	size_t i;
 	int exit_status = EXIT_SUCCESS;
-	
+
 	for (i = 2; args[i]; i++) {
 		char *p = strchr(args[i], '\\'), dir[PATH_MAX];
 		if (p) {
@@ -264,17 +372,18 @@ create_tags(char **args)
 			}
 		}
 
-		snprintf(dir, PATH_MAX, "%s/%s", tags_dir, args[i]);
+		snprintf(dir, sizeof(dir), "%s/%s", tags_dir, args[i]);
 
 		struct stat a;
 		if (lstat(dir, &a) != -1) {
-			fprintf(stderr, _("%s: %s: Tag already exists\n"), PROGRAM_NAME, args[i]);
+			xerror(_("tag: %s: Tag already exists\n"), args[i]);
 			exit_status = EXIT_FAILURE;
 			continue;
 		}
 
 		if (xmkdir(dir, S_IRWXU) == EXIT_FAILURE) {
-			fprintf(stderr, _("%s: %s: Error creating tag\n"), PROGRAM_NAME, args[i]);
+			xerror(_("tag: %s: Error creating tag: %s\n"),
+				args[i], strerror(errno));
 			exit_status = EXIT_FAILURE;
 			continue;
 		}
@@ -286,7 +395,7 @@ create_tags(char **args)
 	return exit_status;
 }
 
-/* Remove those tags specified in ARGS + 2 */
+/* Remove those tags specified in ARGS + 2. */
 static int
 remove_tags(char **args)
 {
@@ -306,15 +415,15 @@ remove_tags(char **args)
 		}
 
 		char dir[PATH_MAX];
-		snprintf(dir, PATH_MAX, "%s/%s", tags_dir, args[i]);
+		snprintf(dir, sizeof(dir), "%s/%s", tags_dir, args[i]);
 
 		struct stat a;
 		if (stat(dir, &a) == -1 || !S_ISDIR(a.st_mode))
 			return print_no_such_tag(args[i]);
 
-		char *cmd[] = {"rm", "-r", dir, NULL};
-		if (launch_execve(cmd, FOREGROUND, E_NOFLAG) == EXIT_SUCCESS) {
-			printf(_("%s: %s: Successfully removed tag\n"), PROGRAM_NAME, args[i]);
+		char *cmd[] = {"rm", "-r", "--", dir, NULL};
+		if (launch_execv(cmd, FOREGROUND, E_NOFLAG) == EXIT_SUCCESS) {
+			printf(_("%s: Successfully removed tag\n"), args[i]);
 			reload_tags();
 		} else {
 			exit_status = EXIT_FAILURE;
@@ -324,87 +433,50 @@ remove_tags(char **args)
 	return exit_status;
 }
 
-/* Check whether NAME is a valid and existent tag name
- * Returns 1 if true or zero otherwise */
-int
-is_tag(char *name)
-{
-	if (!name || !*name)
-		return 0;
-
-	if (strchr(name, '\\')) {
-		char *deq = dequote_str(name, 0);
-		if (deq) {
-			strcpy(name, deq);
-			free(deq);
-		}
-	}
-
-	int i = (int)tags_n;
-	while(--i >= 0) {
-		if (*name == *tags[i] && strcmp(name, tags[i]) == 0)
-			return 1;
-	}
-
-	return 0;
-}
-
-char *
-replace_slashes(char *str)
-{
-	if (*str == '/')
-		str++;
-
-	char *p = savestring(str, strlen(str));
-	char *q = p;
-
-	while (*q) {
-		if (*q == '/' && (q == p || *(q - 1) != '\\'))
-			*q = ':';
-		q++;
-	}
-
-	return p;
-}
-
-/* Tag the file named NAME as TAG */
+/* Tag the file named NAME as TAG. */
 static int
 tag_file(char *name, char *tag)
 {
+	struct stat a;
+	if (lstat(name, &a) == -1) {
+		xerror("tag: %s: %s\n", name, strerror(errno));
+		return EXIT_FAILURE;
+	}
+
 	int new_tag = 0;
 	char *p = (char *)NULL;
 	if (strchr(tag, '\\'))
 		p = dequote_str(tag, 0);
 	char dir[PATH_MAX];
-	snprintf(dir, PATH_MAX, "%s/%s", tags_dir, p ? p : tag);
+	snprintf(dir, sizeof(dir), "%s/%s", tags_dir, p ? p : tag);
 
-	struct stat a;
 	if (stat(dir, &a) == -1) {
 		if (xmkdir(dir, S_IRWXU) != EXIT_SUCCESS) {
-			fprintf(stderr, _("%s: %s: Cannot create tag\n"), PROGRAM_NAME, p ? p : tag);
+			xerror(_("tag: %s: Cannot create tag: %s\n"),
+				p ? p : tag, strerror(errno));
 			free(p);
 			return EXIT_FAILURE;
 		}
-		new_tag = 1;	
+		new_tag = 1;
+	}
+
+	if (new_tag == 1) {
+		printf(_("Created new tag %s%s%s\n"),
+			conf.colorize ? BOLD : "", p ? p : tag, df_c);
+		reload_tags();
 	}
 	free(p);
 
-	if (new_tag) {
-		printf(_("Created new tag %s%s%s\n"), colorize ? BOLD : "", tag, df_c);
-		reload_tags();
-	}
-
 	char name_path[PATH_MAX];
 	*name_path = '\0';
-	if (*name != '/')
-		snprintf(name_path, PATH_MAX, "%s/%s", workspaces[cur_ws].path, name);
+	if (*name != '/') {
+		snprintf(name_path, sizeof(name_path), "%s/%s",
+			workspaces[cur_ws].path, name);
+	}
 
 	char link[PATH_MAX + NAME_MAX], *q = (char *)NULL;
-	char *link_path = replace_slashes(*name_path ? name_path : name);
+	char *link_path = replace_slashes(*name_path ? name_path : name, ':');
 
-//	if (*name == '/')
-//		q = strrchr(name, '/');
-//	snprintf(link, sizeof(link), "%s/%s", dir, (q && *(++q)) ? q : link_path);
 	snprintf(link, sizeof(link), "%s/%s", dir, link_path);
 	free(link_path);
 
@@ -417,9 +489,9 @@ tag_file(char *name, char *tag)
 	return EXIT_SUCCESS;
 }
 
-/* Return an array with indices of tag names (:TAG) found in the ARGS array
- * The indices array is terminated by a -1
- * Returns NULL if no index was found */
+/* Return an array with indices of tag names (:TAG) found in the ARGS array.
+ * The indices array is terminated by a -1.
+ * Returns NULL if no index was found .*/
 static int *
 get_tags(char **args)
 {
@@ -445,26 +517,28 @@ get_tags(char **args)
 	return t;
 }
 
-/* Tag file names found in ARGS as all tags (:TAG) found in ARGS */
+/* Tag file names found in ARGS as all tags (:TAG) found in ARGS. */
 static int
 tag_files(char **args)
 {
 	int *tag_names = get_tags(args);
 	if (!tag_names || tag_names[0] == -1) {
 		free(tag_names);
-		fprintf(stderr, _("%s: No tag specified. Specify a tag via :TAG. "
-			"E.g. tag FILE1 FILE2 :TAG\n"), PROGRAM_NAME);
+		xerror("%s\n", _("tag: No tag specified. Specify a tag via :TAG. "
+			"E.g. 'tag add FILE1 FILE2 :TAG'"));
 		return EXIT_FAILURE;
 	}
 
 	size_t i, j, n = 0;
-	for (i = 1; args[i]; i++) {
+	size_t start = (args[1] && strcmp(args[1], "add") == 0) ? 2 : 1;
+
+	for (i = start; args[i]; i++) {
 		if (*args[i] != ':')
 			n++;
 	}
 
 	for (i = 0; tag_names[i] != -1; i++) {
-		for (j = 1; args[j]; j++) {
+		for (j = start; args[j]; j++) {
 			if (*args[j] == ':')
 				continue;
 
@@ -485,7 +559,7 @@ tag_files(char **args)
 	return EXIT_SUCCESS;
 }
 
-/* Untag file names found in ARGS tagged as ARGS[N] */
+/* Untag file names found in ARGS tagged as ARGS[N]. */
 static int
 untag(char **args, const size_t n, size_t *t)
 {
@@ -498,8 +572,10 @@ untag(char **args, const size_t n, size_t *t)
 		if (i == n || (*args[i] == ':' && *(args[1] + 1)))
 			continue;
 
+		char *ds = dequote_str(args[n] + 1, 0);
 		char dir[PATH_MAX];
-		snprintf(dir, PATH_MAX, "%s/%s", tags_dir, args[n] + 1);
+		snprintf(dir, sizeof(dir), "%s/%s", tags_dir, ds ? ds : args[n] + 1);
+		free(ds);
 
 		struct stat a;
 		if (lstat(dir, &a) == -1 || !S_ISDIR(a.st_mode))
@@ -512,24 +588,24 @@ untag(char **args, const size_t n, size_t *t)
 		if (*p == '~')
 			exp = tilde_expand(p);
 		char *q = exp ? exp : p;
-		char *r = replace_slashes(q);
+		char *r = replace_slashes(q, ':');
 
-		snprintf(f, PATH_MAX + NAME_MAX, "%s/%s", dir, r ? r : q);
+		snprintf(f, sizeof(f), "%s/%s", dir, r ? r : q);
 		free(deq);
 		free(exp);
 		free(r);
 
 		if (lstat(f, &a) != -1 && S_ISLNK(a.st_mode)) {
 			errno = 0;
-			if (unlinkat(AT_FDCWD, f, 0) == -1) {
+			if (unlinkat(XAT_FDCWD, f, 0) == -1) {
 				exit_status = errno;
-				fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, args[i], strerror(errno));
+				xerror("tag: %s: %s\n", args[i], strerror(errno));
 			} else {
 				(*t)++;
 			}
 		} else {
-			fprintf(stderr, _("%s: %s: File not tagged as %s%s%s\n"),
-				PROGRAM_NAME, args[i], colorize ? BOLD : "", args[n] + 1, df_c);
+			xerror(_("tag: %s: File not tagged as %s%s%s\n"),
+				args[i], conf.colorize ? BOLD : "", args[n] + 1, df_c);
 			continue;
 		}
 	}
@@ -537,7 +613,7 @@ untag(char **args, const size_t n, size_t *t)
 	return exit_status;
 }
 
-/* Untag file names found in ARGS as all tags (:TAG) found in ARGS */
+/* Untag file names found in ARGS as all tags (:TAG) found in ARGS. */
 static int
 untag_files(char **args)
 {
@@ -556,7 +632,7 @@ untag_files(char **args)
 	return exit_status;
 }
 
-/* Rename tag ARGS[2] as ARGS[3] */
+/* Rename tag ARGS[2] as ARGS[3]. */
 static int
 rename_tag(char **args)
 {
@@ -568,18 +644,18 @@ rename_tag(char **args)
 		return print_no_such_tag(old);
 
 	if (*old == *new && strcmp(old, new) == 0) {
-		fprintf(stderr, "%s: New and old file names are the same\n", PROGRAM_NAME);
+		xerror("%s\n", "tag: New and old file names are the same");
 		return EXIT_FAILURE;
 	}
 
 	char old_dir[PATH_MAX];
-	snprintf(old_dir, PATH_MAX, "%s/%s", tags_dir, old);
+	snprintf(old_dir, sizeof(old_dir), "%s/%s", tags_dir, old);
 
 	char new_dir[PATH_MAX];
-	snprintf(new_dir, PATH_MAX, "%s/%s", tags_dir, new);
+	snprintf(new_dir, sizeof(new_dir), "%s/%s", tags_dir, new);
 
 	if (rename(old_dir, new_dir) == -1) {
-		fprintf(stderr, "%s: %s\n", PROGRAM_NAME, strerror(errno));
+		xerror("tag: %s\n", strerror(errno));
 		return errno;
 	}
 
@@ -588,8 +664,8 @@ rename_tag(char **args)
 	return EXIT_SUCCESS;
 }
 
-/* Move all (tagged) files (symlinks) in SRC into DST
- * Returns zero if success or the appropriate ERRNO in case of error */
+/* Move all (tagged) files (symlinks) in SRC into DST.
+ * Returns zero if success or the appropriate ERRNO in case of error. */
 static int
 recursive_mv_tags(const char *src, const char *dst)
 {
@@ -597,15 +673,15 @@ recursive_mv_tags(const char *src, const char *dst)
 	char src_dir[PATH_MAX], dst_dir[PATH_MAX];
 	struct dirent **a = (struct dirent **)NULL;
 
-	snprintf(src_dir, PATH_MAX, "%s/%s", tags_dir, src);
+	snprintf(src_dir, sizeof(src_dir), "%s/%s", tags_dir, src);
 
 	n = scandir(src_dir, &a, NULL, alphasort);
 	if (n == -1) {
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, src_dir, strerror(errno));
+		xerror("tag: %s: %s\n", src_dir, strerror(errno));
 		return errno;
 	}
 
-	snprintf(dst_dir, PATH_MAX, "%s/%s", tags_dir, dst);
+	snprintf(dst_dir, sizeof(dst_dir), "%s/%s", tags_dir, dst);
 
 	for (i = 0; i < n; i++) {
 		if (SELFORPARENT(a[i]->d_name)) {
@@ -615,7 +691,7 @@ recursive_mv_tags(const char *src, const char *dst)
 		char src_file[PATH_MAX + NAME_MAX + 2];
 		snprintf(src_file, sizeof(src_file), "%s/%s", src_dir, a[i]->d_name);
 		char *cmd[] = {"mv", src_file, dst_dir, NULL};
-		if ((ret = launch_execve(cmd, FOREGROUND, E_NOFLAG)) != EXIT_SUCCESS)
+		if ((ret = launch_execv(cmd, FOREGROUND, E_NOFLAG)) != EXIT_SUCCESS)
 			exit_status = ret;
 		free(a[i]);
 	}
@@ -624,6 +700,7 @@ recursive_mv_tags(const char *src, const char *dst)
 	return exit_status;
 }
 
+/* Merge tags ARGS[2] and ARGS[3]. */
 static int
 merge_tags(char **args)
 {
@@ -637,57 +714,61 @@ merge_tags(char **args)
 
 	char *src = args[2], *dst = args[3];
 
+	if (strcmp(src, dst) == 0) {
+		xerror("%s\n", _("tag: Source and destiny are the same tag"));
+		return EXIT_FAILURE;
+	}
+
 	errno = 0;
 	int exit_status = recursive_mv_tags(src, dst);
 	if (exit_status != EXIT_SUCCESS) {
-		fprintf(stderr, _("%s: Cannot merge tags: error moving tagged files\n"),
-			PROGRAM_NAME);
+		xerror("%s\n", _("tag: Cannot merge tags: error moving tagged files"));
 		return exit_status;
 	}
 
 	char src_dir[PATH_MAX];
-	snprintf(src_dir, PATH_MAX, "%s/%s", tags_dir, src);
+	snprintf(src_dir, sizeof(src_dir), "%s/%s", tags_dir, src);
 	if (rmdir(src_dir) == -1) {
-		fprintf(stderr, "%s: %s: %s\n", PROGRAM_NAME, src_dir, strerror(errno));
+		xerror("tag: %s: %s\n", src_dir, strerror(errno));
 		return errno;
 	}
 
 	reload_tags();
 	printf(_("Successfully merged %s%s%s into %s%s%s\n"),
-		colorize ? BOLD : "", src, df_c,
-		colorize ? BOLD : "", dst, df_c);
+		conf.colorize == 1 ? BOLD : "", src, df_c,
+		conf.colorize == 1 ? BOLD : "", dst, df_c);
 
 	return EXIT_SUCCESS;
 }
 
 /* Perform the following expansions:
- * ta -> tag
- * td -> tag rm
+ * ta -> tag add
+ * td -> tag del
  * tl -> tag list
- * tm -> tag rename (mv)
+ * tm -> tag rename
  * tn -> tag new
  * tu -> tag untag
  * ty -> tag merge
  * The first string in ARGS must always be one of the left values
- * Returns an array with the expanded values */
+ * Returns an array with the expanded values. */
 static char **
 reconstruct_input(char **args)
 {
 	size_t n = 0, c;
-	for(n = 0; args[n]; n++);
+	for (n = 0; args[n]; n++);
 	char **a = (char **)xnmalloc(n + 2, sizeof(char *));
 
 	a[0] = savestring("tag", 3);
 
-	switch(args[0][1]) {
-	case 'a': c = 1; break;
-	case 'd': a[1] = savestring("rm", 2); c = 2; break;
-	case 'l': a[1] = savestring("ls", 2); c = 2; break;
-	case 'm': a[1] = savestring("mv", 2); c = 2; break;
+	switch (args[0][1]) {
+	case 'a': a[1] = savestring("add", 3); c = 2; break;
+	case 'd': a[1] = savestring("del", 3); c = 2; break;
+	case 'l': a[1] = savestring("list", 4); c = 2; break;
+	case 'm': a[1] = savestring("rename", 6); c = 2; break;
 	case 'n': a[1] = savestring("new", 3); c = 2; break;
 	case 'u': a[1] = savestring("untag", 5); c = 2; break;
 	case 'y': a[1] = savestring("merge", 5); c = 2; break;
-	default: a[1] = savestring("-h", 2); c = 2; break;
+	default:  a[1] = savestring("-h", 2); c = 2; break;
 	}
 
 	for (n = 1; args[n]; n++) {
@@ -699,6 +780,7 @@ reconstruct_input(char **args)
 	return a;
 }
 
+/* Free stuff in A (if needed) and exit the tag function. */
 static int
 end_tag_function(const int exit_status, char **a, const int free_args)
 {
@@ -713,30 +795,47 @@ end_tag_function(const int exit_status, char **a, const int free_args)
 	return exit_status;
 }
 
+/* Check whether we should print help message or not. */
+static int
+is_tag_help(char **args)
+{
+	int first_is_help = (args[1] && IS_HELP(args[1]));
+
+	if (strcmp(args[0], "tl") == 0)
+		return first_is_help;
+
+	return (!args[1] || first_is_help || (args[2] && IS_HELP(args[2])));
+}
+
+/* Handle tag actions according to ARGS. */
 int
 tags_function(char **args)
 {
 	int exit_status = EXIT_SUCCESS, free_args = 0;
 	char **a = args;
 
-	if ((a[1] && IS_HELP(a[1]))	|| (a[1] && a[2] && IS_HELP(a[2])))
+	if (is_tag_help(a))
 		{ puts(_(TAG_USAGE)); goto END; }
 
 	char b[] = "adlmnuy";
-	if (strspn(a[0] + 1, b))
+	if (strcmp(a[0], "tag") != 0 && strspn(a[0] + 1, b))
 		{ a = reconstruct_input(args); free_args = 1; }
 
-	if (!a[1] || (*a[1] == 'l' && (strcmp(a[1], "ls") == 0
-	|| strcmp(a[1], "list") == 0)))
+	if (*a[1] == 'l' && (strcmp(a[1], "ls") == 0
+	|| strcmp(a[1], "list") == 0))
 		{ exit_status = list_tags(a); goto END; }
 
+	if (*a[1] == 'l' && strcmp(a[1], "list-full") == 0)
+		{ exit_status = list_tags_full(); goto END; }
+
 	if (*a[1] == 'm' && strcmp(a[1], "merge") == 0)
 		{ exit_status = merge_tags(a); goto END; }
 
 	if (*a[1] == 'n' && strcmp(a[1], "new") == 0)
 		{ exit_status = create_tags(a); goto END; }
 
-	if ((*a[1] == 'r' && strcmp(a[1], "rm") == 0)
+	if ((*a[1] == 'r' && (strcmp(a[1], "rm") == 0
+	|| strcmp(a[1], "remove") == 0))
 	|| (*a[1] == 'd' && strcmp(a[1], "del") == 0))
 		{ exit_status = remove_tags(a); goto END; }
 
@@ -747,6 +846,7 @@ tags_function(char **args)
 	if (*a[1] == 'u' && strcmp(a[1], "untag") == 0)
 		{ exit_status = untag_files(a); goto END; }
 
+	/* Either 'tag FILE :TAG' or 'tag add FILE :TAG' */
 	exit_status = tag_files(a);
 
 END:
@@ -754,5 +854,5 @@ END:
 }
 
 #else
-void *__skip_me_tags;
+void *_skip_me_tags;
 #endif /* !_NO_TAGS */
