@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  *
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -25,7 +25,11 @@
 #ifndef TAGS_H
 #define TAGS_H
 
-int tags_function(char **args);
-int is_tag(char *name);
+__BEGIN_DECLS
+
+int is_tag(char *);
+int tags_function(char **);
+
+__END_DECLS
 
 #endif /* TAGS_H */
