@@ -3,7 +3,7 @@
 /*
  * This file is part of CliFM
  * 
- * Copyright (C) 2016-2022, L. Abramovich <johndoe.arch@outlook.com>
+ * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
  * All rights reserved.
 
  * CliFM is free software; you can redistribute it and/or modify
@@ -27,7 +27,11 @@
 
 #include <time.h>
 
-int trash_function(char **comm);
-int untrash_function(char **comm);
+__BEGIN_DECLS
+
+int trash_function(char **);
+int untrash_function(char **);
+
+__END_DECLS
 
 #endif /* TRASH_H */
